################################################################################
#
# Makefile  : LibTIFF
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <02/02/19 02:48:58 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)\win32

USE_TIFF = 1
MAKE_TIFF = 1
USE_JPEG = 1
USE_ZLIB = 1
USE_GNUW32 = 1

!include <make/common.mak>

programs = $(tiffdll)
libfiles = $(tifflib)
includefiles = tiff.h tiffconf.h
installdirs = $(bindir) $(includedir) $(libdir)

objects = \
	$(objdir)\tif_aux.obj \
	$(objdir)\tif_close.obj \
	$(objdir)\tif_codec.obj \
	$(objdir)\tif_compress.obj \
	$(objdir)\tif_dir.obj \
	$(objdir)\tif_dirinfo.obj \
	$(objdir)\tif_dirread.obj \
	$(objdir)\tif_dirwrite.obj \
	$(objdir)\tif_dumpmode.obj \
	$(objdir)\tif_error.obj \
	$(objdir)\tif_fax3.obj \
	$(objdir)\tif_fax3sm.obj \
	$(objdir)\tif_getimage.obj \
	$(objdir)\tif_jpeg.obj \
	$(objdir)\tif_flush.obj \
	$(objdir)\tif_luv.obj \
	$(objdir)\tif_lzw.obj \
	$(objdir)\tif_next.obj \
	$(objdir)\tif_open.obj \
	$(objdir)\tif_packbits.obj \
	$(objdir)\tif_pixarlog.obj \
	$(objdir)\tif_predict.obj \
	$(objdir)\tif_print.obj \
	$(objdir)\tif_read.obj \
	$(objdir)\tif_swab.obj \
	$(objdir)\tif_strip.obj \
	$(objdir)\tif_thunder.obj \
	$(objdir)\tif_tile.obj \
	$(objdir)\tif_win32.obj \
	$(objdir)\tif_version.obj \
	$(objdir)\tif_warning.obj \
	$(objdir)\tif_write.obj \
	$(objdir)\tif_zip.obj \
	$(NULL)

default: all

all: $(objdir) $(tiff)

lib: $(objdir) $(tifflib)

!ifdef TIFF_DLL
DEFS = $(DEFS) -DMAKE_TIFF_DLL

$(tifflib): $(objdir) $(objects) $(win32defdir)\libtiff.def
	$(archive) /DEF:$(win32defdir)\libtiff.def $(objects)

$(tiffdll): $(objects) $(zliblib) $(jpeglib) $(objdir)\libtiff.res $(gnuw32lib)
	$(link_dll) $(**) $(objdir)\libtiff.exp $(conlibs)
!else
$(tifflib): $(objdir) $(objects) $(jpeglib) $(zlib)
	$(archive) $(objects) $(jpeglib) $(zlib)
!ENDIF

ALPHA	= ./tiff.alpha
VERSION	= ./VERSION
RELEASEDATE = ./RELEASE-DATE

$(srcdir)\tiffvers.h: $(VERSION) $(ALPHA) $(RELEASEDATE) $(objdir) ./mkversion.c
	$(CC) /Fe$(objdir)\mkversion.exe .\mkversion.c
	-@$(del) $(objdir)\mkversion.obj .\mkversion.obj
	@$(del) $(srcdir)\tiffvers.h & .\$(objdir)\mkversion -v $(VERSION) -a $(ALPHA) -r $(RELEASEDATE) $(srcdir)\tiffvers.h

#
# The finite state machine tables used by the G3/G4 decoders
# are generated by the mkg3states program.  On systems without
# make these rules have to be manually carried out.
#
tif_fax3sm.c: $(objdir)\mkg3states.exe
	-@$(del) tif_fax3sm.c & .\$(objdir)\mkg3states -c const tif_fax3sm.c

$(objdir)\mkg3states.exe: $(objdir)\mkg3states.obj $(gnuw32lib)
	$(link) $(**) $(conlibs)

#VERSION.h: nt\VERSION.h
#	$(CP) $** $@

#tiff.spec: nt\tiff.spec
#	$(CP) $** $@

#tiff.version: nt\tiff.version
#	$(CP) $** $@

!include <make/config.mak>

!include <make/install.mak>

install:: install-exec install-include install-lib

!include <make/clean.mak>

distclean::
	-@$(del) .\tif_fax3sm.c .\VERSION.h

!include <make/rdepend.mak>
!include "./depend.mak"

# 
# Local Variables:
# mode: makefile
# End:
