/*
 * term.c - deal with termcap, and win32 terminal mode settings
 *
 * Fabrice Popineau, 1999
 */

#include "config.h"
#include "ispell.h"
#include "proto.h"
#include "msgs.h"

#include <conio.h>

/*
  FIXME: Console IO under NT is not compatible with
  normal IO. So not defining ANSI_ESC will not work
  unless everything output to stdout is changed to
  do be output to the console. On the other hand, the 
  emacs interface will not work anymore ...
  HINT: look at ispell for djgpp.
*/
#define ANSI_ESC 1

void		erase P ((void));
void		move P ((int row, int col));
void		inverse P ((void));
void		normal P ((void));
void		backup P ((void));
void		terminit P ((void));
SIGNAL_TYPE	done P ((int signo));
void		stop P ((void));
int		shellescape P ((char * buf));

#define PERR(b, msg) { if (! b) { fprintf(stderr, msg); \
  fprintf(stderr, "GetLastError = %d\n", GetLastError()); \
 exit(1); } }

static HANDLE hConsole = NULL;

static   BOOL bSuccess;

static CONSOLE_SCREEN_BUFFER_INFO csbi; /* to get buffer info */

void erase()
{
#ifdef ANSI_ESC
  puts(cl);
#else
  COORD coordScreen = { 0, 0 };    /* here's where we'll home the
				      cursor */
  DWORD cCharsWritten;
  DWORD dwConSize;                 /* number of character cells in
				      the current buffer */
  
  /* get the number of character cells in the current buffer */
  
  bSuccess = GetConsoleScreenBufferInfo( hConsole, &csbi );
  PERR( bSuccess, "GetConsoleScreenBufferInfo" );
  dwConSize = csbi.dwSize.X * csbi.dwSize.Y;
  
  /* fill the entire screen with blanks */
  
  bSuccess = FillConsoleOutputCharacter( hConsole, (TCHAR) ' ',
					 dwConSize, coordScreen, &cCharsWritten );
  PERR( bSuccess, "FillConsoleOutputCharacter" );
  
  /* get the current text attribute */
  
  bSuccess = GetConsoleScreenBufferInfo( hConsole, &csbi );
  PERR( bSuccess, "ConsoleScreenBufferInfo" );
  
  /* now set the buffer's attributes accordingly */
  
  bSuccess = FillConsoleOutputAttribute( hConsole, csbi.wAttributes,
					 dwConSize, coordScreen, &cCharsWritten );
  PERR( bSuccess, "FillConsoleOutputAttribute" );
  
  /* put the cursor at (0, 0) */
  
  bSuccess = SetConsoleCursorPosition( hConsole, coordScreen );
  PERR( bSuccess, "SetConsoleCursorPosition" );
#endif
  return;
}

void move(int row, int col)
{
#ifdef ANSI_ESC
  char buf[64];
  sprintf(buf, cm, row, col);
  puts(buf);
#else
  COORD coordScreen;
  coordScreen.X = col;
  coordScreen.Y = row;

  bSuccess = SetConsoleCursorPosition( hConsole, coordScreen );
  PERR( bSuccess, "SetConsoleCursorPosition" );
#endif
}

void inverse()
{
#ifdef ANSI_ESC
  puts(so);
#else
  bSuccess = SetConsoleTextAttribute(hConsole, 
				     BACKGROUND_RED | BACKGROUND_BLUE | BACKGROUND_GREEN | BACKGROUND_INTENSITY);
  PERR( bSuccess, "SetConsoleTextAttribute" );
#endif
}

void normal()
{
#ifdef ANSI_ESC
  puts(se);
#else
  bSuccess = SetConsoleTextAttribute(hConsole, FOREGROUND_RED | FOREGROUND_BLUE | FOREGROUND_GREEN);
  PERR( bSuccess, "SetConsoleTextAttribute" );
#endif
}

void backup()
{
#ifdef ANSI_ESC
  puts(BC);
#else
  COORD coordScreen;

  bSuccess = GetConsoleScreenBufferInfo( hConsole, &csbi );
  PERR( bSuccess, "ConsoleScreenBufferInfo" );

  coordScreen.X = (csbi.dwCursorPosition.X > 0 ? 
			csbi.dwCursorPosition.X - 1 : 0);
  coordScreen.Y = csbi.dwCursorPosition.Y;
  bSuccess = SetConsoleCursorPosition( hConsole, coordScreen );
  PERR( bSuccess, "SetConsoleCursorPosition" );
#endif
}

static SECURITY_ATTRIBUTES sa = {
  sizeof(SECURITY_ATTRIBUTES),
  NULL,
  TRUE
};

void terminit()
{
  CONSOLE_SCREEN_BUFFER_INFO csbi; /* to get buffer info */

  if (! hConsole) {
    hConsole = CreateFile("CONOUT$", 
			  GENERIC_READ | GENERIC_WRITE,
			  FILE_SHARE_READ | FILE_SHARE_WRITE,
			  &sa,
			  OPEN_EXISTING,
			  0,
			  NULL);
    if (! hConsole) {
      fprintf(stderr, "Unable to open the console.\n");
      return;
    }
    bSuccess = GetConsoleScreenBufferInfo( hConsole, &csbi );
    PERR( bSuccess, "GetConsoleScreenBufferInfo" );
#if 0
    co = csbi.dwSize.X;
    li = csbi.dwSize.Y;
#else
    co = csbi.srWindow.Right + 1;
    li = csbi.srWindow.Bottom + 1;
#endif
    attr = csbi.wAttributes;
    /*
     * Let the variables "LINES" and "COLUMNS" override the termcap
     * entry.  Technically, this is a terminfo-ism, but I think the
     * vast majority of users will find it pretty handy.
     */
    if (getenv ("COLUMNS") != NULL)
	co = atoi (getenv ("COLUMNS"));
    if (getenv ("LINES") != NULL)
	li = atoi (getenv ("LINES"));

    cl = "\33[2J";
    cm = "\33[%u;%uf";
    so = "\33[7m";
    se = "\33[0m";
    sg = strlen(so);

#if MAX_SCREEN_SIZE > 0
    if (li > MAX_SCREEN_SIZE)
      li = MAX_SCREEN_SIZE;
#endif /* MAX_SCREEN_SIZE > 0 */
#if MAXCONTEXT == MINCONTEXT
    contextsize = MINCONTEXT;
#else /* MAXCONTEXT == MINCONTEXT */
    if (contextsize == 0)
#ifdef CONTEXTROUNDUP
      contextsize = (li * CONTEXTPCT + 99) / 100;
#else /* CONTEXTROUNDUP */
    contextsize = (li * CONTEXTPCT) / 100;
#endif /* CONTEXTROUNDUP */
    if (contextsize > MAXCONTEXT)
      contextsize = MAXCONTEXT;
    else if (contextsize < MINCONTEXT)
      contextsize = MINCONTEXT;
#endif /* MAX_CONTEXT == MIN_CONTEXT */
    /*
     * Insist on 2 lines for the screen header, 2 for blank lines
     * separating areas of the screen, 2 for word choices, and 2 for
     * the minimenu, plus however many are needed for context.  If
     * possible, make the context smaller to fit on the screen.
     */
    if (li < contextsize + 8  &&  contextsize > MINCONTEXT)
      {
	contextsize = li - 8;
	if (contextsize < MINCONTEXT)
	  contextsize = MINCONTEXT;
      }
    if (li < MINCONTEXT + 8)
      (void) fprintf (stderr, TERM_C_SMALL_SCREEN, MINCONTEXT + 8);
  }
}

SIGNAL_TYPE done(int signo)
{
  /* Restore attributes */
  SetConsoleTextAttribute(hConsole, attr);
}

void stop()
{

}

int shellescape(char *buf)
{

}
