################################################################################
#
# Makefile  : Ispell
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <02/03/03 23:50:03 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)\win32

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

# To build debug version, define environment variable DEBUG, or include a
# -DDEBUG on the commandline (i.e.: nmake -DDEBUG)

USE_KPATHSEA = 1
USE_GNUW32 = 1
USE_REGEX = 1
USE_ZLIB = 1

!include <make/common.mak>

manfiles = ispell.1 sq.1
programs = $(objdir)\ispell.exe $(objdir)\buildhash.exe $(objdir)\icombine.exe \
	$(objdir)\ijoin.exe $(objdir)\sq.exe $(objdir)\unsq.exe
infofiles = ispell.info
htmldocfiles = ispell.html
pdfdocfiles = ispell.pdf
docsubdir = ispell
installinfoflags = --section=Miscellaneous \
	--entry="* ispell (ispell) :               	     Spell Checker for Emacs."

languages = languages\american languages\deutsch languages\english languages\francais-GUTenberg-v1.0

#
# Main target
#
all: $(objdir) $(programs)

#
# Link target. setargv.obj is provided in the compiler library directory.
#
objects_ispell = $(objdir)\termw32.obj $(objdir)\ispell.obj         \
	$(objdir)\correct.obj $(objdir)\defmt.obj $(objdir)\dump.obj  \
	$(objdir)\good.obj $(objdir)\lookup.obj $(objdir)\hash.obj    \
	$(objdir)\makedent.obj $(objdir)\tgood.obj $(objdir)\tree.obj \
	$(objdir)\xgets.obj

$(objdir)\ispell.exe: $(objects_ispell) $(kpathsealib) $(regexlib) $(zliblib)
	$(link) $(**)  $(conlibs)

objects_buildhash = $(objdir)\buildhash.obj $(objdir)\hash.obj   \
	$(objdir)\makedent.obj $(objdir)\parse.obj

$(objdir)\buildhash.exe: $(gnuw32lib) $(objects_buildhash) $(kpathsealib)
	$(link) $(**) $(conlibs)

objects_icombine = $(objdir)\icombine.obj $(objdir)\makedent.obj \
	$(objdir)\parse.obj

$(objdir)\icombine.exe: $(gnuw32lib) $(objects_icombine)
	$(link) $(**) $(conlibs)

$(objdir)\ijoin.exe: $(gnuw32lib) $(objdir)\ijoin.obj $(objdir)\fields.obj
	$(link) $(**) $(conlibs)

$(objdir)\sq.exe: $(gnuw32lib) $(objdir)\sq.obj
	$(link) $(**) $(conlibs)

$(objdir)\unsq.exe: $(gnuw32lib) $(objdir)\unsq.obj
	$(link) $(**) $(conlibs)

parse.c: parse.y
	$(yacc) -d -v $(srcdir)/parse.y -o parse.c

!include <make/config.mak>

installdirs = $(texmf)\ispell

!include <make/install.mak>

install:: install-exec install-data install-info install-doc install-man

subdirs = $(languages)

install-data clean distclean::
	-@echo $(verbose) & for %%d in ($(subdirs)) do \
		echo Entering %%d for $@ \
		& pushd %%d & $(make) $@ & popd


ispell.1: ispell.1X
	$(copy) ispell.1X ispell.1

ispell.texi: ispell.texinfo

!include <make/clean.mak>

!include <make/rdepend.mak>
!include "./depend.mak"

#
# Local Variables:
# mode: makefile
# End:
