/*
 * $Id: commands.c,v 1.1 1994/06/17 11:26:29 ralf Exp $
 * History:
 * $Log: commands.c,v $
 * Revision 1.1  1994/06/17  11:26:29  ralf
 * Initial revision
 *
 */
/***************************************************************************
     name : commands.c
   author : DORNER Fernando, GRANZER Andreas
  purpose : includes all LaTex-Commands which can be converted into Rtf-Commands
 ******************************************************************************/

/****************************** includes ************************************/
#include <stdio.h>
#include "main.h"
#include "funct1.h"
#include "commands.h"
#include "funct2.h"

/****************************************************************************/
void error(char *);
/*********************** typedefs *******************************************/
typedef struct commandtag
	  { char *cpCommand;		 /* LaTeX-commandname without \ */
	    void (*func)(int);		 /* name of function which converts an LaTex-command to an Rtf-command */
	    int param;			 /* optional parameter can be used in various ways */
	  } CommandArray;
/****************************************************************************/

/********************** global variables ************************************/
/* CommandArray *Environments[100] = {NULL}; */ /* list of active environments */
CommandArray *Environments[ MAXENVIRONS ] = {NULL}; /* list of active environments */
extern int iEnvCount;			 /* number of active environments */
extern FILE *fRtf;                       /* Rtf-File-Pointer */
extern char *progname;
extern BOOL GermanMode;
extern BOOL MathMode[MAXENVIRONS];
extern int curr_fontsize[MAXENVIRONS];
extern int curr_fontbold[MAXENVIRONS];
extern int curr_fontital[MAXENVIRONS];
extern int curr_fontscap[MAXENVIRONS];
extern int curr_fontnumb[MAXENVIRONS];
extern char alignment[MAXENVIRONS];
extern int parindent[MAXENVIRONS];  /* DT */
extern int environ_type[MAXENVIRONS];  /* DT */
extern int environ_mode[MAXENVIRONS];  /* DT */
extern int DefaultIndent;     /* DT */
extern long linenumber;
extern char *LatexSize[MAXENVIRONS];
extern BOOL Debug;

extern BOOL IntblDone;
extern BOOL ParagraphEmpty;
extern int TabularColumn;

CommandArray commands[300] = {
/********************************************************************/
/* commands for environment document */
/********************************************************************/
   { "begin", CmdBeginEnd, CMD_BEGIN },
   { "end", CmdBeginEnd, CMD_END },
   { "today", CmdToday, 0 },
   { "footnote", CmdFootNote, 0 },
   { "underline", CmdUnderline, 0 },
   { "tiny", CmdFontSize, 10 },
   { "scriptsize", CmdFontSize, 14 },
   { "footnotesize", CmdFontSize, 16 },
   { "small", CmdFontSize, 18 },
   { "normalsize", CmdFontSize, 20 },
   { "large", CmdFontSize, 24 },
   { "Large", CmdFontSize, 28 },
   { "LARGE", CmdFontSize, 34 },
   { "huge", CmdFontSize, 40 },
   { "Huge", CmdFontSize, 50 },
   { "HUGE", CmdFontSize, 60 },
   { "LaTeX", CmdLogo, CMD_LATEX },
   { "LaTeXe", CmdLogo, CMD_LATEXE },
   { "TeX", CmdLogo, CMD_TEX },
   { "SLiTeX", CmdLogo, CMD_SLITEX },
   { "BibTeX", CmdLogo, CMD_BIBTEX },
   { "acute", CmdLApostrophChar, 0 },
   { "grave", CmdRApostrophChar, 0 },
   { "hat", CmdRApostrophChar, 0 },
   { "tilde", CmdTildeChar, 0 },
   { "ldots", CmdLdots, 0 },
/*   { "em", CmdEmphasize, 0}, */
   { "maketitle", CmdTitle, TITLE_MAKE },
   { "title", CmdTitle, TITLE_TITLE }, 
   { "author", CmdTitle, TITLE_AUTHOR }, 
   { "date", CmdTitle, TITLE_DATE },
   { "section", CmdSection, SECT_NORM },
/*   { "section*", CmdUSection, SECT_NORM }, */ /* DT */
   { "caption", CmdCaption, 0 },
   { "c", CmdCedillaChar, 0 }, /* DT */
   { "d", CmdUnderdotChar, 0 }, /* DT */
   { "v", CmdHacekChar, 0 }, /* DT */
   { "r", CmdOaccentChar, 0 }, /* DT */
   { "chapter", CmdSection, SECT_CHAPT },
/*   { "chapter*", CmdUSection, SECT_CHAPT }, */ /* DT */
   { "subsection", CmdSection, SECT_SUB },
/*   { "subsection*", CmdUSection, SECT_SUB }, */ /* DT */
   { "subsubsection", CmdSection, SECT_SUBSUB },
/*   { "subsubsection*", CmdUSection, SECT_SUBSUB }, */ /* DT */
   { "part", CmdSection, SECT_SUBSUB },
   { "appendix",CmdIgnore,0},
   { "protect", CmdIgnore,0},
   { "paragraph", CmdSection, SECT_PARA },
   { "subparagraph", CmdSection, SECT_SUBPARA },
   { "clearpage", CmdNewPage, NewPage },
   { "cleardoublepage", CmdNewPage, NewPage },
   { "newpage", CmdNewPage, NewColumn },
   { "mbox", CmdMbox, 0},
   { "hbox", CmdMbox, 0},
   { "frenchspacing", CmdIgnore, 0},
   { "nonfrenchspacing", CmdIgnore, 0},
   { "rm", CmdSetFont, F_ROMAN},		     /* set font Roman */
   { "rmfamily", CmdSetFont, F_RMFAMILY},	     /* set font Roman */
   { "bf", CmdSetFont, F_BOLD},			     /* set font Roman bold */
   { "bfseries", CmdSetFont, F_BFSERIES},	     /* set font Roman bold */
   { "sl", CmdSetFont, F_SLANTED},		     /* set font Slanted */
   { "slshape", CmdSetFont, F_SLSHAPE},		     /* set font Slanted */
   { "it", CmdSetFont, F_ITALIC},		     /* set font Slanted */
   { "itshape", CmdSetFont, F_ITSHAPE},		     /* set font Slanted */
   { "em", CmdSetFont, F_EMSHAPE},		     /* set font Slanted */
   { "textbf", CmdTextSetFont, F_BFSERIES},		     /* set font Roman bold */
   { "textmd", CmdTextSetFont, F_MDSERIES},		     /* set font Roman bold */
   { "textup", CmdTextSetFont, F_UPSHAPE},	     /* set font Slanted */
   { "textsl", CmdTextSetFont, F_SLSHAPE},	     /* set font Slanted */
   { "textit", CmdTextSetFont, F_ITSHAPE},	     /* set font Slanted */
   { "texttt", CmdTextSetFont, F_TTFAMILY},	     /* set font Sans Serif */
   { "textrm", CmdTextSetFont, F_RMFAMILY},	     /* set font Roman */
   { "textsc", CmdTextSetFont, F_SCFAMILY},	     /* set font Small caps */
   { "textsf", CmdTextSetFont, F_SFFAMILY},	     /* set font Sans Serif */
   { "mathbf", CmdMathSetFont, F_BFSERIES},		     /* set font Roman bold */
   { "mathmd", CmdMathSetFont, F_MDSERIES},		     /* set font Roman bold */
   { "mathup", CmdMathSetFont, F_UPSHAPE},	     /* set font Slanted */
   { "mathsl", CmdMathSetFont, F_SLSHAPE},	     /* set font Slanted */
   { "mathit", CmdMathSetFont, F_ITSHAPE},	     /* set font Slanted */
   { "mathtt", CmdMathSetFont, F_TTFAMILY},	     /* set font Sans Serif */
   { "mathrm", CmdMathSetFont, F_RMFAMILY},	     /* set font Roman */
   { "mathsc", CmdMathSetFont, F_SCFAMILY},	     /* set font Small caps */
   { "mathsf", CmdMathSetFont, F_SFFAMILY},	     /* set font Sans Serif */
   { "textnormal", CmdTextSetFont, F_NORMAL},	     /* set font Normal */
   { "emph", CmdTextSetFont, F_EMSHAPE},	     /* toggle font emphasize */
   { "textsuperscript", CmdSuperscript, 0},	     /* superscript in text mode */
   { "normalfont", CmdSetFont, F_NORMAL},	     /* set font normal */
   { "sf", CmdSetFont, F_SANSSERIF},		     /* set font Sans Serif */
   { "sffamily", CmdSetFont, F_SFFAMILY},	     /* set font Sans Serif */
   { "sc", CmdSetFont, F_SMALLCAPS},		     /* set font Small Caps */
   { "scfamily", CmdSetFont, F_SCFAMILY},	     /* set font Small Caps */
   { "tt", CmdSetFont, F_TYPEWRITER},		     /* set font Typewriter */
   { "ttfamily", CmdSetFont, F_TTFAMILY},	     /* set font Typewriter */
   { "upshape", CmdSetFont, F_UPSHAPE}, 	     /* set font upright */
   { "mdseries", CmdSetFont, F_MDSERIES},	     /* set font mdseries */
/*   { "degree", CmdFrenchAbbrev, DEGREE}, */
   { "ier", CmdFrenchAbbrev, IERF},
   { "iere", CmdFrenchAbbrev, IEREF},
   { "iers", CmdFrenchAbbrev, IERSF},
   { "ieres", CmdFrenchAbbrev, IERESF},
   { "ieme", CmdFrenchAbbrev, IEMEF},
   { "iemes", CmdFrenchAbbrev, IEMESF},
   { "numero", CmdFrenchAbbrev, NUMERO},
   { "numeros", CmdFrenchAbbrev, NUMEROS},
   { "Numero", CmdFrenchAbbrev, CNUMERO},
   { "Numeros", CmdFrenchAbbrev, CNUMEROS},
/*   { "degres", CmdFrenchAbbrev, DEGREE}, */
/*   { "textdegree", CmdFrenchAbbrev, DEGREE}, */
   { "primo", CmdFrenchAbbrev, PRIMO},
   { "secundo", CmdFrenchAbbrev, SECUNDO},
   { "tertio", CmdFrenchAbbrev, TERTIO},
   { "quarto", CmdFrenchAbbrev, QUARTO},
   { "fup", CmdFrenchAbbrev, FUP},
/*   { "include", CmdInclude, 0}, 		      include Latex file */
/*   { "input", CmdInclude, 0},			      include Latex file */
   { "verb", CmdVerb, 0},
   { "verb*", CmdVerb, AST_FORM},
   { "onecolumn", CmdColumn, One_Column },
   { "twocolumn", CmdColumn, Two_Column },
   { "flushbottom", CmdBottom, 0},
   { "raggedbottom", CmdBottom, 0},
   { "setlength", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "footnotemark", CmdIgnoreParameter, One_Opt_No_NormParam },
   { "footnotetext", CmdIgnoreParameter, One_Opt_One_NormParam },
   { "newlength", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "pagestyle", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "thispagestyle", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "label", CmdLabel, LABEL },
   { "ref", CmdLabel, REF },
   { "cite", CmdLabel, CITE },
   { "pageref", CmdLabel, PAGEREF },
   { "bibliography", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "newsavebox", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "usebox", CmdIgnoreParameter, No_Opt_One_NormParam },
/*   { "fbox", CmdIgnoreParameter, No_Opt_One_NormParam }, */
   { "hspace", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "hspace*", CmdIgnoreParameter, No_Opt_One_NormParam },
/*   { "vspace", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "vspace*", CmdIgnoreParameter, No_Opt_One_NormParam }, */
   { "addvspace", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "addcontentsline", CmdIgnoreParameter, No_Opt_Three_NormParam },
   { "addcontents", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "numberline", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "stretch", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "hyphenation", CmdHyphenation, 0 },
   { "typeaout", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "index", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "indexentry", CmdIgnoreParameter, No_Opt_Two_NormParam},
   { "glossary", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "glossaryentry", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "typeout", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "Typein", CmdIgnoreParameter, One_Opt_One_NormParam },
   { "includeonly", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "nocite", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "bibliography", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "stepcounter", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "refstepcounter", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "fnsymbol", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "Alph", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "alph", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "Roman", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "roman", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "arabic", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "value", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "def", CmdIgnoreDef, 0},
   { "newcommand", IgnoreNewCmd, 0 },
   { "renewcommand", IgnoreNewCmd,0 },
   { "newenvironment", CmdIgnoreParameter, One_Opt_Three_NormParam },
   { "renewenvironment", CmdIgnoreParameter, One_Opt_Three_NormParam },
   { "newtheorem", CmdIgnoreParameter, One_Opt_Two_NormParam },
   { "renewtheorem", CmdIgnoreParameter, One_Opt_Two_NormParam },
   { "newcount", CmdIgnoreDef, 0 },
   { "output", CmdIgnoreDef, 0 },
   { "newcounter", CmdIgnoreParameter, One_Opt_One_NormParam },
   { "value", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "makebox", CmdIgnoreParameter, Two_Opt_One_NormParam },
   { "framebox", CmdIgnoreParameter, Two_Opt_One_NormParam },
   { "sbox", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "savebox", CmdIgnoreParameter, Two_Opt_Two_NormParam },
   { "parbox", CmdIgnoreParameter, One_Opt_Two_NormParam },
   { "rule", CmdIgnoreParameter, One_Opt_Two_NormParam },
   { "raisebox", CmdIgnoreParameter, Two_Opt_Two_NormParam },
   { "newfont", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "addtolength", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "settowidth", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "pagebreak", CmdIgnoreParameter, One_Opt_No_NormParam },
   { "nopagebreak", CmdIgnoreParameter, One_Opt_No_NormParam },
   { "samepage", CmdIgnore, 0 },
   { "null", CmdIgnore, 0 },
   { "linebreak", CmdIgnoreParameter, One_Opt_No_NormParam },
   { "nolinebreak", CmdIgnoreParameter, One_Opt_No_NormParam },
   { "typein", CmdIgnoreParameter, One_Opt_One_NormParam },
   { "marginpar", CmdIgnoreParameter, One_Opt_One_NormParam },
/*   { "caption", CmdIgnoreParameter, One_Opt_One_NormParam }, */
   { "addtocounter", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "setcounter", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "newcounter", CmdIgnoreParameter, One_Opt_One_NormParam },
   { "baselineskip", Cmd_OptParam_Without_braces, 0 },
   { "lineskip", Cmd_OptParam_Without_braces, 0 },
   { "vsize", Cmd_OptParam_Without_braces, 0 },
   { "setbox", Cmd_OptParam_Without_braces, 0 },
   { "pagestyle", CmdIgnoreParameter, No_Opt_One_NormParam }, /* pagestyle is ignored because there is a variable calculation
								 in the RTF-header which can't be converted by reading the LATEX-File. */
   { "pagenumbering", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "markboth", CmdIgnoreParameter, No_Opt_Two_NormParam},
   { "markright", CmdIgnoreParameter, No_Opt_One_NormParam},
   { "thanks", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "bibliographystyle", CmdIgnoreParameter,No_Opt_One_NormParam },
   { "let", CmdIgnoreLet, 0},
   { "par", CmdPar, 0},
   { "medskip", CmdSkip, 4},
   { "bigskip", CmdSkip, 6},
   { "smallskip", CmdSkip, 2},
   { "noindent", CmdNoindent, 0},
   { "parindent", CmdNoindent, 1},
   { "paperwidth", CmdPaper, 1},
   { "paperheight", CmdPaper, 2},
   { "textwidth", CmdPaper, 3},
   { "textheight", CmdPaper, 4},
   { "hoffset", CmdPaper, 5},
   { "voffset", CmdPaper, 6},
   { "vspace", CmdVspace, 0},
   { "cline",CmdCline, 0},
   { "frac",CmdFrac, 0},
   { "Frac",CmdFrac, 0},
   { "int",CmdIntegral, 0},
   { "sum",CmdIntegral, 1},
   { "prod",CmdIntegral, 2},
   { "quad",CmdQuad, 1},
   { "qquad",CmdQuad, 2},
   { "", NULL } };

CommandArray HeaderCommands[100] = {
/********************************************************************/
/* commands for environment LaTeX-header */
/********************************************************************/
   { "documentstyle", CmdDocumentStyle, 0 },
   { "documentclass", CmdDocumentStyle, 0 },
   { "setlength", CmdIgnoreParameter, No_Opt_Two_NormParam },
   { "pagestyle", CmdIgnoreParameter, No_Opt_One_NormParam }, /* pagestyle is ignored because there is a variable calculation
								 in the RTF-header which can't be converted by reading the LATEX-File. */
   { "pagenumbering", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "markboth", CmdIgnoreParameter, No_Opt_Two_NormParam},
   { "markright", CmdIgnoreParameter, No_Opt_One_NormParam},
   { "newlength", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "title", CmdTitle, TITLE_TITLE }, 
   { "usepackage", CmdUsePackage, 0}, 
   { "author", CmdTitle, TITLE_AUTHOR }, 
   { "date", CmdTitle, TITLE_DATE },
   { "begin", CmdBeginEnd, CMD_BEGIN },
   { "newcommand", CmdIgnoreParameter, One_Opt_Two_NormParam },
   { "renewcommand", CmdIgnoreParameter, One_Opt_Two_NormParam },
   { "def", CmdIgnoreDef, 0},
   { "flushbottom", CmdBottom, 0},
   { "raggedbottom", CmdBottom, 0},
   { "thanks", CmdIgnoreParameter, No_Opt_One_NormParam },
   { "address" , CmdAddress, 0 },
   { "signature", CmdSignature, 0},
   { "opening", CmdOpening, 0},
   { "closing", CmdClosing, 0},
   { "ps", CmdPs, 0},
   { "", NULL } };    /* end of list */

CommandArray ItemizeCommands[10] = {
/********************************************************************/
/* commands for environment Itemize */
/********************************************************************/
   { "item", CmdItem, ITEMIZE },
   { "", NULL } };    /* end of list */

CommandArray DescriptionCommands[10] = {
/********************************************************************/
/* commands for environment Description */
/********************************************************************/
   { "item", CmdItem, DESCRIPTION },
   { "", NULL } };    /* end of list */

CommandArray BiblioCommands[10] = {
/********************************************************************/
/* commands for environment Thebibliography */
/********************************************************************/
   { "bibitem", CmdItem, BIBLIOGRAPHY },
   { "", NULL } };    /* end of list */

CommandArray EnumerateCommands[10] = {
/********************************************************************/
/* commands for environment Enumerate */
/********************************************************************/
   { "item", CmdItem, ENUMERATE },
   { "", NULL } };    /* end of list */

CommandArray MathCommands[10] = {
/********************************************************************/
/* commands for environment Enumerate */
/********************************************************************/
   { "sqrt", CmdMathFunct, M_SQRT },
   { "", NULL } };    /* end of list */

CommandArray TabbingCommands[10] = {
/********************************************************************/
/* commands for environment tabbing */
/********************************************************************/
    { "kill" , CmdTabkill, 0 },  /* a line that ends with a \kill command produces no output */
    { "", NULL } };   /* end of list */

CommandArray TabularCommands[10] = {
/********************************************************************/
/* commands for environment tabbing */
/********************************************************************/
    { "hline" , CmdHline, 0 },  /* a line that ends with a \kill command produces no output */
    { "", NULL } };   /* end of list */

CommandArray LetterCommands[10] = {
/********************************************************************/
/* commands for environment letter */
/********************************************************************/
    { "address" , CmdAddress, 0 },
    { "signature", CmdSignature, 0},
    { "opening", CmdOpening, 0},
    { "closing", CmdClosing, 0},
    { "ps", CmdPs, 0},
    { "", NULL } };   /* end of list */

CommandArray GermanModeCommands[10] = {
/********************************************************************/
/* commands for German Mode */
/********************************************************************/
 /*   { "ck" , CmdPrintRtf , (int)"ck" },
    { "glqq" , CmdPrintRtf , (int)"\\ldblquote "},
    { "glq" , CmdPrintRtf , (int)"\\lquote "},
    { "grqq" , CmdPrintRtf , (int)"\\rdblquote "},
    { "grq" , CmdPrintRtf , (int)"\\rquote "}, */
    { "ck", GermanPrint, GP_CK},
    { "glqq", GermanPrint, GP_LDBL},
    { "glq", GermanPrint, GP_L},
    { "grqq", GermanPrint, GP_R},
    { "grq", GermanPrint, GP_RDBL},
    { "", NULL } };   /* end of list */

CommandArray empty_list[10] = {
    { "", NULL } };   /* end of list */

CommandArray params[100] = {
/********************************************************************/
/* commands for begin-end environments */
/* only strings used in the form \begin{text} or \end{text} */
/********************************************************************/
   { "center", Paragraph, PAR_CENTER },
   { "flushright", Paragraph, PAR_RIGHT },
   { "flushleft", Paragraph, PAR_LEFT},
   { "document", Environment, DOCUMENT },
   { "tabbing", Tabbing, TABBING },
   { "figure", CmdIgnoreFigure, FIGURE },
   { "figure*", CmdIgnoreFigure, FIGURE_1 },
   { "minipage", CmdIgnoreFigure, MINIPAGE },
   { "em", Format, EMPHASIZE },
   { "multicols", CmdMulticols, MULTICOLS },
   { "quote", CmdQuote, QUOTE },
   { "quotation", CmdQuote, QUOTATION },
   { "enumerate", CmdList, ENUMERATE },
   { "list", CmdList, ITEMIZE },
   { "itemize", CmdList, ITEMIZE },
   { "narrowitemize", CmdList, ITEMIZE }, /* a special addition for DT's personal comfort */
   { "description", CmdList, DESCRIPTION },
   { "thebibliography", CmdList, BIBLIOGRAPHY },
   { "verbatim", CmdVerbatim, 0 },
   { "Verbatim", CmdVerbatim, 0 },
   { "verse", CmdVerse, 0 },
   { "tabular", CmdTabular, TABULAR },
   { "tabular*", CmdTabular, TABULAR_1 },
   { "math", CmdFormula2, BEGIN_MATH },
   { "displaymath" , CmdFormula2, BEGIN_DISPLAY },
   { "equation", CmdFormula2, 0 },
   { "picture", CmdFigure, BEGIN_PICTURE },
   { "music", CmdFigure, BEGIN_MUSIC },
   { "letter", CmdLetter, LETTER },
   { "table", CmdTable, TABLE },
   { "table*", CmdTable, TABLE_1 },
   { "caption", CmdCaption, 0 },
   { "abstract", CmdAbstract, 0},
   { "titlepage", CmdTitlepage, 0},
   { "array", CmdArray, ARRAY },
   { "eqnarray", CmdArray, EQNARRAY },
   { "eqnarray*", CmdArray, EQNARRAY_1},
   { "tiny", GroupFontSize, 10 },
   { "scriptsize", GroupFontSize, 14 },
   { "footnotesize", GroupFontSize, 16 },
   { "small", GroupFontSize, 18 },
   { "normalsize", GroupFontSize, 20 },
   { "large", GroupFontSize, 24 },
   { "Large", GroupFontSize, 28 },
   { "LARGE", GroupFontSize, 34 },
   { "huge", GroupFontSize, 40 },
   { "Huge", GroupFontSize, 50 },
   { "HUGE", GroupFontSize, 60 },
   { "sf", GroupSetFont, F_SANSSERIF},		     /* set font Sans Serif */
   { "sffamily", GroupSetFont, F_SFFAMILY},	     /* set font Sans Serif */
   { "sc", GroupSetFont, F_SMALLCAPS},		     /* set font Small Caps */
   { "scfamily", GroupSetFont, F_SCFAMILY},	     /* set font Small Caps */
   { "rm", GroupSetFont, F_ROMAN},		     /* set font Roman */
   { "rmfamily", GroupSetFont, F_RMFAMILY},	     /* set font Roman */
   { "bf", GroupSetFont, F_BOLD},			     /* set font Roman bold */
   { "bfseries", GroupSetFont, F_BFSERIES},	     /* set font Roman bold */
   { "sl", GroupSetFont, F_SLANTED},		     /* set font Slanted */
   { "slshape", GroupSetFont, F_SLSHAPE},		     /* set font Slanted */
   { "it", GroupSetFont, F_ITALIC},		     /* set font Slanted */
   { "itshape", GroupSetFont, F_ITSHAPE},		     /* set font Slanted */
   { "normalfont", GroupSetFont, F_NORMAL},	     /* set font normal */
   { "sf", GroupSetFont, F_SANSSERIF},		     /* set font Sans Serif */
   { "sffamily", GroupSetFont, F_SFFAMILY},	     /* set font Sans Serif */
   { "sc", GroupSetFont, F_SMALLCAPS},		     /* set font Small Caps */
   { "scfamily", GroupSetFont, F_SCFAMILY},	     /* set font Small Caps */
   { "tt", GroupSetFont, F_TYPEWRITER},		     /* set font Typewriter */
   { "ttfamily", GroupSetFont, F_TTFAMILY},	     /* set font Typewriter */
   { "upshape", GroupSetFont, F_UPSHAPE}, 	     /* set font upright */
   { "mdseries", GroupSetFont, F_MDSERIES},	     /* set font mdseries */
   { "", NULL } };    /* end of list */



/****************************************************************************/
BOOL CallCommandFunc(char *cCommand)
/****************************************************************************
purpose: The tries to call the command-function for the commandname
params:  string with command name
returns: sucess or not
globals: command-functions have side effects or recursive calls
 ****************************************************************************/
{
  int i=0, j;

  for (j = iEnvCount-1; j >= 0; j--, i = 0)
  {
    while (strcmp(Environments[j][i].cpCommand,"")!= 0)
    {
      if (strcmp(Environments[j][i].cpCommand,cCommand)== 0)
      {
	if (Environments[j][i].func == NULL)
	  return FALSE;
	(*Environments[j][i].func)((Environments[j][i].param));
	return TRUE; /* Command Function found */
      }
      ++i;
    }
  }
  return FALSE;
}


/****************************************************************************/
BOOL CallParamFunc(char *cCommand, int AddParam)
/****************************************************************************
purpose: The tries to call the environment-function for the commandname
params:  cCommand - string with command name
	 AddParam - param "ORed"(||) to the int param of command-funct
returns: sucess or not
globals: command-functions have side effects or recursive calls
 ****************************************************************************/
{
  int i=0;
  char unknown_environment[100];
  char errormessage[256];

  while (strcmp(params[i].cpCommand,"")!= 0)
  {
    if (strcmp(params[i].cpCommand,cCommand)== 0)
      {
	 (*params[i].func)((params[i].param)|AddParam);
	 return TRUE;	 /* command function found */
      }
    ++i;
  }

  /* unknown environment must be ignored !!! */
  /* ========================================*/
  if (AddParam == ON)
      {
      sprintf(unknown_environment,"%s%s%s","end{",cCommand,"}");
	 CmdUnknownEnvir(UNKNOWN_ENVIR|AddParam);
/*      Ignore_Environment(unknown_environment); */
      sprintf(errormessage,"Environment %s ignored, because it's not defined in the command-list\n",cCommand);
      warning(errormessage);
      } else
      {
	 CmdUnknownEnvir(UNKNOWN_ENVIR|AddParam);
      }

/*  return FALSE; */
  return TRUE;
}


/****************************************************************************/
void PushEnvironment(int code)
/****************************************************************************
purpose: adds the command list for a specific environment to the list
	 of commands searched through.
params:  constant identifying the environment
globals: changes Environment - array of active environments
		 iEnvCount   - counter of active environments
 ****************************************************************************/
{
  char errormessage[256] = "";
  int z;
	
  if ( iEnvCount >= MAXENVIRONS -1 )
  { sprintf(errormessage,"More than %d environment levels",MAXENVIRONS);
    error(errormessage);
  };

	
  switch(code)
  {
    case HEADER: Environments[iEnvCount++]=HeaderCommands;
		 break;
    case DOCUMENT:
		   Environments[iEnvCount++]=commands;
		   if (GermanMode)
		      Environments[iEnvCount++]=GermanModeCommands;
		   break;
    case ITEMIZE:  Environments[iEnvCount++]=ItemizeCommands;
                   fprintf(fRtf,"{");
		   break;
    case ENUMERATE: Environments[iEnvCount++]=EnumerateCommands;
                   fprintf(fRtf,"{");
                   break;
    case TABBING : Environments[iEnvCount++]=TabbingCommands;
                   fprintf(fRtf,"{");
		   break;
    case TABULAR : Environments[iEnvCount++]=TabularCommands;
                   fprintf(fRtf,"{");
		   break;
    case LETTER : Environments[iEnvCount++]=LetterCommands;
                   fprintf(fRtf,"{");
		   break;
    case DESCRIPTION : Environments[iEnvCount++]=DescriptionCommands;
                   fprintf(fRtf,"{");
		   break;
    case BIBLIOGRAPHY : Environments[iEnvCount++]=BiblioCommands;
                   fprintf(fRtf,"{");
		   break;
    case GERMANMODE : CheckIntbl();
                   Environments[iEnvCount++]=GermanModeCommands;
                   fprintf(fRtf,"{");
		   break;
    case GROUP :   CheckIntbl();
		   Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case PAR_CENTER :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case PAR_LEFT :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case PAR_RIGHT :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case QUOTE :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case QUOTATION :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case EXPONENT : CheckIntbl();
                   Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case SUBSCRIPT : CheckIntbl();
                   Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case MATH_FORMULA : CheckIntbl();
                   Environments[iEnvCount++]=MathCommands;
                   fprintf(fRtf,"{");
		   break;
    case MATH_DISPLAY :CheckIntbl();
                   Environments[iEnvCount++]=MathCommands;
                   fprintf(fRtf,"{");
		   break;
    case MATH_ENVIR : CheckIntbl();
                   Environments[iEnvCount++]=MathCommands;
                   fprintf(fRtf,"{");
		   break;
    case DISPLAY_ENVIR : CheckIntbl();
                   Environments[iEnvCount++]=MathCommands;
                   fprintf(fRtf,"{");
		   break;
    case EQN_ENVIR : CheckIntbl();
                   Environments[iEnvCount++]=MathCommands;
                   fprintf(fRtf,"{");
		   break;
    case FOOTNOTE :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case MULTICOLS :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case PICT_ENVIR :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case MUSIC_ENVIR :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case AUTHOR :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case TITLE :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;
    case UNKNOWN_ENVIR :  Environments[iEnvCount++]=empty_list;
                   fprintf(fRtf,"{");
		   break;

    default:
      sprintf(errormessage,"assertion %d failed at function PushEnvironment",code);
      error(errormessage);
  };
  curr_fontsize[iEnvCount] = curr_fontsize[iEnvCount-1];
  alignment[iEnvCount] = alignment[iEnvCount-1];
  parindent[iEnvCount] = parindent[iEnvCount-1];
  LatexSize[iEnvCount] = LatexSize[iEnvCount-1];
  MathMode[iEnvCount] = MathMode[iEnvCount-1];
  curr_fontsize[iEnvCount] = curr_fontsize[iEnvCount-1];
  curr_fontnumb[iEnvCount] = curr_fontnumb[iEnvCount-1];
  curr_fontbold[iEnvCount] = curr_fontbold[iEnvCount-1];
  curr_fontital[iEnvCount] = curr_fontital[iEnvCount-1];
  curr_fontscap[iEnvCount] = curr_fontscap[iEnvCount-1];
  environ_type[iEnvCount] = code;
  /* keep environ mode when GROUP requested (GROUP = opening brace) */
  if (code == GROUP)
  {  environ_mode[iEnvCount] = environ_mode[iEnvCount-1];}
  else
  {  environ_mode[iEnvCount] = code;};

  if (Debug)  fprintf(OUTPUTF,"\n\nPush(iEnvCount=%d, type=%d <%d>)",iEnvCount,code,linenumber);
  if (Debug)
  { fprintf(OUTPUTF,"\n");
    for (z=0; z<=iEnvCount; z++) fprintf(OUTPUTF," %d",environ_type[z]);
  }
}


/****************************************************************************/
void PopEnvironment(int code)
/****************************************************************************
purpose: removes the environment-commands list added by the last call to
	 PushEnvironemnt;
globals: changes Environment - array of active environments
		 iEnvCount - counter of active environments
 ****************************************************************************/
{
  char errormessage[256] = "";
  int i; int z;
  
  if ( iEnvCount < 1 )
  { error("Too many \\end(<environment>)");
  };

  if (environ_type[iEnvCount] != code)
  {
    if (Debug)
    { for (i=0; i <= iEnvCount; i++)
      { sprintf(errormessage,"environ_type[%d]=%d",i,environ_type[i]);
        Message(errormessage);
      }
      fprintf(OUTPUTF,"\nPop(iEnvCount=%d, type=%d)",iEnvCount,code);
    }  	
    sprintf(errormessage,
      "\nError: environ %d closed by %d at line %d",environ_type[iEnvCount],code,linenumber);
    error(errormessage);  
  };
	
  if (Debug)  fprintf(OUTPUTF,"\nPop(iEnvCount=%d, type=%d <%d>\n)",iEnvCount,code,linenumber);
  if (Debug)
  { fprintf(OUTPUTF,"\n");
    for (z=0; z<=iEnvCount; z++) fprintf(OUTPUTF," %d",environ_type[z]);
  }

  --iEnvCount;

  fprintf(fRtf,"}");

  Environments[iEnvCount]=NULL;

     /* overlapping environments are not allowed !!!
	example: \begin{verse}\begin{enumerate}\end{verse}\end{enumerate}
	==> undefined result
	extension possible
     */

  return ;
}


/****************************************************************************/
void ClearEnvironment()
/****************************************************************************
purpose: removes all environment-commands lists
globals: changes Environment - array of active environments
		 iEnvCount - counter of active environments
 ****************************************************************************/
{
  Environments[0] = NULL;
  iEnvCount = 0;
  return;
}
