/*
 * $Id: funct2.c,v 1.2 1994/06/21 08:14:11 ralf Exp $
 * History:
 * $Log: funct2.c,v $
 * Revision 1.2  1994/06/21  08:14:11  ralf
 * Corrected Bug in keyword search
 *
 * Revision 1.1  1994/06/17  11:26:29  ralf
 * Initial revision
 *
 */
/***************************************************************************
   name : funct2.c
 author : DORNER Fernando, GRANZER Andreas
purpose : includes besides funct1.c all functions which are called from the programm commands.c;
 ****************************************************************************/

/********************************* includes *********************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <malloc.h>
#include "main.h"
#include "funct1.h"
#include "commands.h"
#include "funct2.h"
#include "stack.h"
/******************************************************************************/

/********************************   extern variables   *************************/
extern int tabcounter;
extern FILE *fRtf;
extern FILE *fTex;
extern int bPard;
extern BOOL bInDocument;
extern int BracketLevel;
extern int RecursLevel;
extern BOOL twocolumn;
extern BOOL TABBING_ON;
extern BOOL TABBING_ON_itself;
extern BOOL TABBING_RETURN;
extern BOOL article;
extern BOOL report;
extern BOOL book;
extern BOOL titlepage;
extern int fontsize;
extern char *progname;
extern char *latexname;
extern BOOL MathMode[MAXENVIRONS];
extern char alignment[MAXENVIRONS];
extern int environ_type[MAXENVIRONS];
extern int environ_mode[MAXENVIRONS];  /* DT */
extern long linenumber;
extern int iEnvCount;
extern fpos_t pos_begin_kill;
extern BOOL ParDone;          /* DT */   /* is the RTF /par already done before */
extern BOOL IgnoreSpaces;
extern int TabularColumn;
extern char CellJustif[MAXCELLS];
extern BOOL CellBorderLeft[MAXCELLS];
extern BOOL CellBorderRight[MAXCELLS];
extern BOOL CellBorderTop[MAXCELLS];
extern BOOL CellBorderBottom[MAXCELLS];
extern int curr_fontsize[MAXENVIRONS];
extern BOOL InputLineEmpty;
extern BOOL Debug;
extern int NumberColumns;
extern BOOL IntblDone;
extern BOOL ParagraphEmpty;
extern BOOL TabularSpecialFormat;
extern char ErrorMsgText[256];

/*****************************************************************************/

void OutputRowFormatSpecial();

/***************************** global variables ********************************/
int tabstoparray[100];
int number_of_tabstops=0;
BOOL FirstHline=TRUE;

int ncell=0;

char TheCellJustif[MAXCELLS];
BOOL TheCellBorderLeft[MAXCELLS];
BOOL TheCellBorderRight[MAXCELLS];
BOOL TheCellBorderTop[MAXCELLS];
BOOL TheCellBorderBottom[MAXCELLS];
int cellx[MAXCELLS]={0};

int NewNcell=0;

int NewCellx[MAXCELLS] = {0};
BOOL NewCellBorderTop[MAXCELLS];
BOOL NewCellBorderBottom[MAXCELLS];
BOOL NewCellBorderLeft[MAXCELLS];
BOOL NewCellBorderRight[MAXCELLS];
char NewCellJustif[MAXCELLS];




/*****************************************************************************/
/*----------------------------Tabbing Environment ------------------------*/
/******************************************************************************/
void Tabbing(int code)
/******************************************************************************
  purpose: pushes all tabbing-commands on a stack
parameter: code : on/off at begin/end-environment
  globals: TABBING_ON: true if tabbing-mode is on (only in this environment)
	   TABBING_RETURN, TABBING_ITSELF: true if environmend ends
 ******************************************************************************/
{
  if (code & ON)  /* on switch */
  {
    code &= ~(ON);  /* mask MSB */
    if (code == TABBING)
    {
      TABBING_ON = TRUE;
      /*TABBING_ON_itself = FALSE; */

      PushEnvironment(code);

      fprintf(fRtf,"\\par\\line ");
      fgetpos(fRtf,&pos_begin_kill);
      /* Test ConvertTabbing(); */
    }
  }
  else /* off switch */
  {
    /* TABBING_RETURN = TRUE;
    TABBING_ON_itself = TRUE; */
    TABBING_ON = FALSE;
    PopEnvironment(code);

    fprintf(fRtf,"\\par\\pard\\line\\q%c ",alignment[iEnvCount]);
  }
}

/******************************************************************************/
void CmdTabset(int code)
/******************************************************************************
 purpose: sets an tabstop
globals:  tabcounter: specifies the tabstop-position
 ******************************************************************************/
{int tabstop;
    tabstop = (tabcounter/6)*567;
    tabstoparray[number_of_tabstops] = tabstop;
    number_of_tabstops++;
    fprintf(fRtf,"\\tx%d ",tabstop); /* Tab at tabstop/567 centimeters */
}

/******************************************************************************/
void CmdTabkill(int code)
/******************************************************************************
 purpose: a line in the TABBING-Environment which ends with an kill-command won't be
	 written to the rtf-FILE
 ******************************************************************************/
{int i;

  fsetpos(fRtf,&pos_begin_kill);

  for(i=0;i<number_of_tabstops;i++)
    {
    fprintf(fRtf,"\\tx%d ",tabstoparray[i]); /* Tab at tabstop/567 centimeters */
    }

  number_of_tabstops = 0;
}
/*-------------------- End of Tabbing Environment -------------------------*/



/******************************************************************************/
void CmdIgnoreFigure(int code)
/******************************************************************************
  purpose: function, which overreads the Figure,Picture,Bibliopgraphy and Minipage
	   Environment
parameter: code: which environment to ignore
 ******************************************************************************/
{
 char endfigure[20];
 char zeichen;
 BOOL found = FALSE;
 int i, endstring=0;

    switch (code & ~(ON))  /* mask MSB */
    {
	 case FIGURE : {
			   strcpy(endfigure,"end{figure}");
			   endstring = strlen(endfigure) -1;
			   break;
		       }
	 case FIGURE_1 : {
			   strcpy(endfigure,"end{figure*}");
			   endstring = strlen(endfigure) -1;
			   break;
		       }
	 case PICTURE : {
			   strcpy(endfigure,"end{picture}");
			   endstring = strlen(endfigure) -1;
			   break;
			}
	 case MINIPAGE : {
			   strcpy(endfigure,"end{minipage}");
			   endstring = strlen(endfigure) -1;
			   break;
			}
	 case THEBIBLIOGRAPHY : {
			   strcpy(endfigure,"end{thebibliography}");
			   endstring = strlen(endfigure) -1;
			   break;
			}

    } /* end switch */

    while (fread(&zeichen,1,1,fTex) >=1)
    {
       if (zeichen == '\\')
       {
	  int found_space = FALSE;
	  for (i=0; i<=endstring; i++)
	    {
	    int nl = FALSE;
	    if (fread(&zeichen,1,1,fTex) < 1)
		numerror(ERR_EOF_INPUT);
	    if (zeichen == '\\')
		linenumber++;                   /* \\-in figure is a line !!! */
	    while(zeichen == ' ' || zeichen == '\t' || IsEol(&zeichen) && !nl)
	    {
		if(IsEol(&zeichen))
		    nl = TRUE;
		if(fread(&zeichen,1,1,fTex) !=1)
		    numerror(ERR_EOF_INPUT);
		found_space = TRUE;
	    }
	    if(found_space && zeichen != '{') /* for vi } */
		break;
	    found_space = FALSE;
	    if (zeichen != endfigure[i])
		break;
	    if (i == endstring)                 /* end-figure-found */
		found = TRUE;
	    } /* for */
       } /* if */
       if (zeichen == '%')
	  IgnoreTo('\n');
       if (IsEol(&zeichen))
	  linenumber++;                         /* count lines */
       if (found)
	   return;
    } /* while */
    numerror(ERR_EOF_INPUT);
}
/*------------------------------------------------------------------------*/


/******************************************************************************/
void CmdIgnoreParameter(int code)
/******************************************************************************
  purpose: function, which ignores both: optional and normal parameters
parameter: code: number of optional/normal parameters to ignore
 ******************************************************************************/
{ char optparam[WRKMAX];
  switch (code)
       {
	  case Two_Opt_One_NormParam : GetOptParam(optparam,WRKMAX-1);
	  case One_Opt_One_NormParam : GetOptParam(optparam,WRKMAX-1);
	  case No_Opt_One_NormParam  : GetParam(optparam,WRKMAX-1);
    				       return;
				       break;
	  case Two_Opt_Two_NormParam : GetOptParam(optparam,WRKMAX-1);
	  case One_Opt_Two_NormParam : GetOptParam(optparam,WRKMAX-1);
	  case No_Opt_Two_NormParam  : GetParam(optparam,WRKMAX-1);
				       GetParam(optparam,WRKMAX-1);
				       return;
				       break;
	  case Two_Opt_Three_NormParam: GetOptParam(optparam,WRKMAX-1);
	  case One_Opt_Three_NormParam: GetOptParam(optparam,WRKMAX-1);
	  case No_Opt_Three_NormParam: GetParam(optparam,WRKMAX-1);
				       GetParam(optparam,WRKMAX-1);
				       GetParam(optparam,WRKMAX-1);
				       return;
				       break;
	  case Two_Opt_No_NormParam  : GetOptParam(optparam,WRKMAX-1);
	  case One_Opt_No_NormParam  : GetOptParam(optparam,WRKMAX-1);
				      break;
       } /* switch */
}
/*------------------------------------------------------------------------*/


/******************************************************************************/
void Ignore_Environment(char *searchstring)
/******************************************************************************
  purpose: function, which ignores an unconvertable environment in LaTex
parameter: searchstring : includes the string to search for
	   example: \begin{unknown} ... \end{unknown}
		    searchstring="end{unknown}"
 ******************************************************************************/
{
 char zeichen;
 BOOL found = FALSE;
 int i, endstring;
    endstring = strlen(searchstring) - 1;
    while (fread(&zeichen,1,1,fTex) >=1)
    {
       if (zeichen == '\\')
       {
	  for (i=0; i<=endstring; i++)
	    {
	    if (fread(&zeichen,1,1,fTex) < 1)
		numerror(ERR_EOF_INPUT);
	    if (zeichen != searchstring[i])
		break;
	    if (i == endstring)                 /* end-environment-found */
		found = TRUE;
	    } /* for */
	} /* if */
       if (zeichen == '%')
	   IgnoreTo('\n');
       if (IsEol(&zeichen))
	   linenumber++;
       if (found)
	   return;
    } /* while */
    numerror(ERR_EOF_INPUT);
}
/*------------------------------------------------------------------------*/


/******************************************************************************/
void GetOptParam(char *string, int size)
/******************************************************************************
  purpose: function to get an optional parameter
parameter: string: returnvalue of optional parameter
	   size: max. size of returnvalue
 ******************************************************************************/
{
  char cThis;
  int i,PopLevel,PopBrack;

  if ( (fread(&cThis,1,1,fTex) < 1))
    numerror(ERR_EOF_INPUT);

  if ( cThis != '[' )
  {
    fseek(fTex,-1L,SEEK_CUR); /* reread last character */
    string = "";
    return;
  }
  else
  {
    ++BracketLevel;
    Push(RecursLevel,BracketLevel);
  }
  for (i = 0; ;i++)   /* get param from input stream */
  {
    if (fread(&cThis,1,1,fTex) < 1)
       numerror(ERR_EOF_INPUT);
    if (cThis == ']')
    {
      --BracketLevel;
      Pop(&PopLevel,&PopBrack);
      break;
    }
    if (cThis == '%')
      {
       IgnoreTo('\n');
       continue;
      }
    if (size-- > 0)
      string[i] = cThis;
  }
  string[i] = '\0';
}
/*------------------------------------------------------------------------*/

/******************************************************************************/
void CmdIgnoreEnvironment(int code)
/******************************************************************************
  purpose: overreads an ignoreable environment
parameter: code: type of environment & ON/OFF
 ******************************************************************************/
{
     switch (code & ~(ON))   /* ON/OFF-parameter exclude */
     {
	case BIBLIOGRAPHY :
			  Ignore_Environment("end{thebibliography}");
			  break;
	case LETTER : Ignore_Environment("end{letter}");
			  break;
	case TABLE : Ignore_Environment("end{table}");
			  break;
	case TABLE_1 : Ignore_Environment("end{table*}");
			  break;
	default : numerror(ERR_WRONG_COMMAND);
     } /* switch */
}

/******************************************************************************/
void CmdColumn (int code)
/******************************************************************************
  purpose: chooses beetween one/two-columns
parameter: number of columns
 globals: twocolumn: true if twocolumn-mode is set
 ******************************************************************************/
{
   switch (code)
   {
       case One_Column : fprintf(fRtf,"\\page \\colsx709\\endnhere "); /* new page & one column */
			 twocolumn = FALSE;
			 break;
       case Two_Column : fprintf(fRtf,"\\page \\cols2\\colsx709\\endnhere "); /* new page & two columns */
			 twocolumn = TRUE;
			 break;
   } /* switch */
}

/******************************************************************************/
char *GetSubString(char *s, char terminatesymbol)
/******************************************************************************
purpose:
-----------------------------------------------------------------------------
 GetString is a function which returns the substrings of the string s
   seperated by the terminatesymbol
 after calling this funtion the first substring to the terminatesymbol
   is cut off the string s
-----------------------------------------------------------------------------*/
{char *substring;
 char *hilfstring;
 int i=0;

      substring =(char*) malloc((strlen(s)+1) * sizeof(char)); /* get heap-memory */
      if (substring == NULL)
	  error(" malloc error -> out of memory!\n");
      strcpy(substring,"");

      for(i=0; (int)i<=(int)strlen(s); i++)  /* if *s == "" -> for won't be executed */
      {                            /* <= is used for recognising the endmark of the string */
	if (s[i] == '\0')
	{
	   strcpy(s,"");
	   break;
	}
	if (s[i] == terminatesymbol)
	{
	   hilfstring = &s[i+1];
	   strcpy(s,hilfstring);    /* is also replaced in the calling-function */
	   break;
	}
	substring[i] = s[i];
      }  /* for */

      substring[i] = '\0'; /* end-mark */
      return substring;
}
/*-----------------------------------------------------------------------------*/

/******************************************************************************/
void CmdNewPage(int code)
/******************************************************************************
  purpose: starts a new page
parameter: code: newpage or newcolumn-option
 globals: twocolumn: true if twocolumn-mode is set
 ******************************************************************************/
{
   switch (code)
   {
      case NewPage :  fprintf(fRtf,"\\page "); /* causes new page */
		      break;
      case NewColumn : if (twocolumn == TRUE)
			  fprintf(fRtf,"\\column "); /* new column */
		       else
			  fprintf(fRtf,"\\page ");  /* causes new page */
		       break;
   } /* switch */
}

/******************************************************************************/
void Cmd_OptParam_Without_braces(int code)
/******************************************************************************
 purpose: gets an optional parameter which isn't surrounded by braces but by spaces
 ******************************************************************************/
{char cNext=' ';
 char cLast=' ';

    do
    {
       cLast = cNext;
       if (fread(&cNext,1,1,fTex) < 1)
	   numerror(ERR_EOF_INPUT);

    } while ((cNext != ' ') &&
	     (cNext != '\\') &&
	     (cNext != '{') &&
	     (!IsEol(&cNext)) &&
	     (cNext != ',') &&
	     ((cNext != '.')  || (isdigit(cLast))) && /* . doesn't mean the end of an command inside an number of the type real */
	     (cNext != '}') &&
	     (cNext != '\"') &&
	     (cNext != '[') &&
	     (cNext != '$'));

    fseek(fTex,-1L,SEEK_CUR);
}


/******************************************************************************/
void GetInputParam(char *string, int size)
/******************************************************************************
  purpose: gets the parameter followed the \include or \input -command
parameter: string: returnvalue of the input/include-parameter
	   size: max. size of the returnvalue
		 must be determined by the calling-function
 ******************************************************************************/
{
  char cThis;
  int i,PopLevel,PopBrack;
  BOOL readuntilnewline = FALSE;

  if ( (fread(&cThis,1,1,fTex) < 1))
    numerror(ERR_EOF_INPUT);
  if ( cThis == '{' )
  {
    ++BracketLevel;
    Push(RecursLevel,BracketLevel);

  }
  else
  {
    readuntilnewline = TRUE;
    fseek(fTex,-1L,SEEK_CUR); /* reread last character */
  }
  for (i = 0; ;i++)   /* get param from input stream */
  {
    if (fread(&cThis,1,1,fTex) < 1)
       numerror(ERR_EOF_INPUT);
    if (cThis == '}')
    {
      --BracketLevel;
      Pop(&PopLevel,&PopBrack);
      break;
    }

    if ((readuntilnewline == TRUE) &&
	((cThis == ' ') || (IsEol(&cThis))))
	{
	if (IsEol(&cThis))
	    linenumber++;
	break;
	}

    if (size-- > 0)
      string[i] = cThis;
  }
  string[i] = '\0';
}

/******************************************************************************/
void ConvertTabbing(void)
/******************************************************************************
 purpose: routine which converts the tabbing-commands from LaTex to Rtf
 ******************************************************************************/
{ int read_end = 1024;
  char cCommand[MAXCOMMANDLEN];
  int i;
  long j=0;
  char cThis;
  BOOL getcommand;
  BOOL command_end_line_found;
  BOOL command_kill_found;

while (TABBING_ON)
{
command_end_line_found = FALSE;
command_kill_found = FALSE;

while (command_end_line_found == FALSE)
  {
  for (;;) /* do forever */
  {
    getcommand=FALSE;
    if (fread(&cThis,1,1,fTex) < 1)
       numerror(ERR_EOF_INPUT);
    j++;

    if (cThis == '\\')
       {
       getcommand=TRUE;
       strcpy(cCommand,"");

       for (i = 0; ;i++)   /* get command from input stream */
	   {
	   if (fread(&cThis,1,1,fTex) < 1)
	      numerror(ERR_EOF_INPUT);
	   j++;

	   if (i == 0) /* test for special characters */
	      {
	      switch(cThis)
		  {
		  case '\\':    command_end_line_found=TRUE;
				break;
		    } /* switch */
	      }  /* if */

	   if (!isalpha(cThis))
	       {
	       while (cThis == ' ')   /* all spaces after commands are ignored */
	       {
	       if (fread(&cThis,1,1,fTex) < 1)
		   numerror(ERR_EOF_INPUT);
	       j++;
	       }

	       fseek(fTex,-1L,SEEK_CUR); /* position of next character after command
					    except space */
	       j--;
	       break; /* for */
	       }
	   cCommand[i] = cThis;
	   }  /* for */

	   cCommand[i] = '\0';  /* mark end of string with zero */
    }  /* if \\ */

    if ((getcommand) &&
	((command_end_line_found) ||
	 (strcmp(cCommand,"kill") == 0) ||
	 (strcmp(cCommand,"end") == 0)))
	{
	command_end_line_found = TRUE;
	if (strcmp(cCommand,"kill") == 0)
	    command_kill_found = TRUE;
	break;
	}

    if (j >= read_end)
	{
	command_end_line_found = TRUE;
	break;
	}
  } /* for */
  } /* while command_end_line_found */

  fseek(fTex,-j,SEEK_CUR); /* re_read line */
  if (command_kill_found)
    Convert_Tabbing_with_kill();
  else
    Convert();
} /* while Tabbing_ON */

TABBING_ON = FALSE;
} /* ConvertTabbing */


/******************************************************************************/
void Convert_Tabbing_with_kill(void)
/******************************************************************************
 purpose: routine which converts the tabbing-kill-option from LaTex to Rtf
 globals: tabcounter:
 ******************************************************************************/
{ int i=0;
  BOOL command_kill_found=FALSE;
  char cThis;
  char cCommand[MAXCOMMANDLEN];

tabcounter=0;

while (command_kill_found == FALSE)
   {
    if (fread(&cThis,1,1,fTex) < 1)
       numerror(ERR_EOF_INPUT);

    strcpy(cCommand,"");

    if (cThis == '\\')
       {

       for (i = 0; ;i++)   /* get command from input stream */
	   {
	   if (fread(&cThis,1,1,fTex) < 1)
	      numerror(ERR_EOF_INPUT);

	   if (i == 0) /* test for special characters */
	      {
	      switch(cThis)
		  {
		  case '=': CmdTabset(0);
			    break;
		  default : if(!isalpha(cThis))
				 numerror(ERR_WRONG_COMMAND_IN_TABBING);
		    } /* switch */
	      }  /* if */

	   if (!isalpha(cThis))
	       {
	       while (cThis == ' ')   /* all spaces after commands are ignored */
	       {
	       if (fread(&cThis,1,1,fTex) < 1)
		   numerror(ERR_EOF_INPUT);
	       }

	       fseek(fTex,-1L,SEEK_CUR); /* position of next character after command
					    except space */
	       break; /* for */
	       }
	   cCommand[i] = cThis;
	   }  /* for */

	   cCommand[i] = '\0';  /* mark end of string with zero */
    }  /* if \\ */
    else
      tabcounter++;

      if (strcmp(cCommand,"kill") == 0)
	{
	command_kill_found = TRUE;
	tabcounter = 0;
	break;
	}
  } /* while command_kill_found */
} /* Convert_Tabbing_with_kill */


/******************************************************************************/
void CmdBottom(int code)
/******************************************************************************/
{
  /* it's conventional for the height of the text to be the same on all full pages */
}

/******************************************************************************/
void CmdAbstract(int code)
/******************************************************************************
  purpose: converts the LaTex-abstract-command to an similar Rtf-style
parameter: code: on/off-option
 globals : article and titlepage from the documentstyle
 ******************************************************************************/
{ static char oldalignment = JUSTIFIED;


  switch (code)
     {
     case  ON:
	 if ((article) && (titlepage))
	    {
	    fprintf(fRtf,"\n\r\\par\n\\par\\pard ");
	    fprintf(fRtf,"\\pard\\qj ");  /* blocked */
	    fprintf(fRtf,"{\\b\\fs%d Abstract}",fontsize);
	    }
	 else
	    {
	    fprintf(fRtf,"\n\r\\par\n\\par\\pard \\page ");
	    fprintf(fRtf,"\\pard\\qj ");   /* blocked */
	    fprintf(fRtf,"{\\b\\fs%d Abstract:}\\par ",fontsize);
	    }
	  oldalignment = alignment[iEnvCount];
	  alignment[iEnvCount] = JUSTIFIED;
	  break;
    case  OFF:
	  fprintf(fRtf,"\\pard ");
	  alignment[iEnvCount] = oldalignment;
	  fprintf(fRtf,"\n\r\\par\\q%c ",alignment[iEnvCount]);
	  break;
     } /* switch */
}



/******************************************************************************/
void CmdTitlepage(int code)
/******************************************************************************
  purpose: converts the LaTex-Titlepage-command to an similar Rtf-style
parameter: on/off option
 globals : alignment: is used for the default-alignment-setting after this environment
 ******************************************************************************/
{
  switch (code)
     {
     case  ON:
	    fprintf(fRtf,"\n\r\\par\\pard \\page ");  /* new page */
	    fprintf(fRtf,"\n\r\\par\\q%c ",alignment[iEnvCount]);
	    break;
     case  OFF:
	  fprintf(fRtf,"\\pard ");
	  fprintf(fRtf,"\n\r\\par\\q%c \\page ",alignment[iEnvCount]);
	 break;
     } /* switch */
}

/******************************************************************************/
void CmdHyphenation(int code)
/******************************************************************************
 purpose: the parameter surrrounded by braces after the hyphenation-command
	  won't be seperated at a line-end.
 ******************************************************************************/
{ char hyphenparameter[10240];
  unsigned int i;

    GetParam(hyphenparameter,10239);

/* In a future version we may correctly hyphenate all occurencies of
 * hyphenation-words
 */
# ifdef notdef
    for (i=0; i<(strlen(hyphenparameter)-1); i++)
	{
	if (hyphenparameter[i] != '-')
	    fprintf(fRtf,"%c",hyphenparameter[i]);
	else
	    fprintf(fRtf,"\\-");
	} /* for */
# endif /* notdef */
}

/******************************************************************************/
void CmdFormula2(int code)
/******************************************************************************
 purpose: the same as the function CmdFormula: see above!
 ******************************************************************************/
{ char errormessage[128];

  if(Debug) fprintf(OUTPUTF,"\n  CmdFormula2: code %8X ", code);

  switch(code)
  {
    case (BEGIN_DISPLAY | ON):
     if (MathMode[iEnvCount] == TRUE)
     {  sprintf(errormessage,"\\begin{<displaymath>} in math mode in File: %s at linenumber: %ld",latexname,linenumber);
        error(errormessage);
     }
     CmdFormula(BEGIN_DISPLAY);
     break;
    case (BEGIN_DISPLAY | OFF):
     if (MathMode[iEnvCount] == FALSE)
     {  sprintf(errormessage,"\\end{<displaymath>} not in math mode in File: %s at linenumber: %ld",latexname,linenumber);
        error(errormessage);
     }
     CmdFormula(BEGIN_DISPLAY);
     break;
    case (BEGIN_MATH | ON):
     if (MathMode[iEnvCount] == TRUE)
     {  sprintf(errormessage,"\\begin{<math>} in math mode in File: %s at linenumber: %ld",latexname,linenumber);
        error(errormessage);
     }
     CmdFormula(BEGIN_MATH);
     break;
    case (BEGIN_MATH | OFF):
     if (MathMode[iEnvCount] == FALSE)
     {  sprintf(errormessage,"\\end{<math>} not in math mode in File: %s at linenumber: %ld",latexname,linenumber);
        error(errormessage);
     }
     CmdFormula(BEGIN_MATH);
     break;
  };
}

/******************************************************************************/
void CmdLetter(int code)
/******************************************************************************
  purpose: pushes all necessary letter-commands on a stack
parameter: code: on/off-option for environment
 ******************************************************************************/
{
  if (code & ON)  /* on switch */
  {
    code &= ~(ON);  /* mask MSB */
    if (code == LETTER)
    {
      PushEnvironment(code);
    }
  }
  else /* off switch */
  {
    PopEnvironment(code);
  }
}

/******************************************************************************/
void CmdAddress(int code)
/******************************************************************************
 purpose: prints the address in a letter
 globals: alignment
 ******************************************************************************/
{ static char oldalignment = JUSTIFIED;

     oldalignment = alignment[iEnvCount];
     alignment[iEnvCount] = RIGHT;
     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);   /* address will be printed on the right top */

     Convert(); /* convert routine is called again for evaluating the contens
		 hold in braces after the \address-command */

     alignment[iEnvCount] = oldalignment;
     fprintf(fRtf,"\\par\\chdate "); /* additional to the address the actual date is printed */

     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);
}


/******************************************************************************/
void CmdSignature(int code)
/******************************************************************************
 purpose: prints the signature in a letter
 globals: alignment
 ******************************************************************************/
{ static char oldalignment = JUSTIFIED;

     oldalignment = alignment[iEnvCount];
     alignment[iEnvCount] = RIGHT;
     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);   /* signature will be printed on the right top */

     Convert(); /* convert routine is called again for evaluating the contens
		 hold in braces after the \signature-command */

     alignment[iEnvCount] = oldalignment;
     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);
}

/******************************************************************************/
void CmdOpening(int code)
/******************************************************************************
 purpose: special command in the LaTex-letter-environment will be converted to a
	  similar Rtf-style
 globals: alignment
 ******************************************************************************/
{ static char oldalignment;

     oldalignment = alignment[iEnvCount];
     alignment[iEnvCount] = LEFT;
     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);   /* opening will be printed on the right top */

     Convert(); /* convert routine is called again for evaluating the contens
		 hold in braces after the \opening-command */

     alignment[iEnvCount] = oldalignment;
     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);
}

/******************************************************************************/
void CmdClosing(int code)
/******************************************************************************
 purpose: special command in the LaTex-letter-environment will be converted to a
	  similar Rtf-style
 globals: alignment
 ******************************************************************************/
{ static char oldalignment;

     oldalignment = alignment[iEnvCount];
     alignment[iEnvCount] = LEFT;
     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);   /* closing will be printed on the right top */

     Convert(); /* convert routine is called again for evaluating the contens
		 hold in braces after the \closing-command */

     alignment[iEnvCount] = oldalignment;
     fprintf(fRtf,"\n\r\\par\\pard\\q%c ",alignment[iEnvCount]);
}

void CmdPs(int code)
{
    /* additional text to the \ps-command will be converted by the basic convert-routine */
    /* but you'll have to type the 'P.S.:'-text yourself */
}

/******************************************************************************/
void CmdArray(int code)
/******************************************************************************
 purpose: converts the LaTex-Array/eqnarray to a similar Rtf-style
	  this converting is only partially
	  so the user has to convert some part of the array/eqnarray-environment by hand
parameter: type of array-environment
 ******************************************************************************/
{
  if (code & ON)  /* on switch */
  {
    code &= ~(ON);  /* mask MSB */
    fprintf(OUTPUTF,"\n%s: WARNING: Following environment cannot be converted completely!",progname);
    fprintf(OUTPUTF,"\nSome parts of this environment have to be converted and corrected by hand");

    if (code == ARRAY)
      {
      fprintf(OUTPUTF,"\nBegin of environment: ARRAY");
      fprintf(fRtf,"\\par ***begin of environment: ARRAY ***\\par");
      }
    if (code == EQNARRAY)
      {
      fprintf(OUTPUTF,"\nBegin of environment: EQNARRAY");
      fprintf(fRtf,"\\par ***begin of environment: EQNARRAY ***\\par");
      }
    if (code == EQNARRAY_1)
      {
      fprintf(OUTPUTF,"\nBegin of environment: EQNARRAY* ");
      fprintf(fRtf,"\\par ***begin of environment: EQNARRAY*    ***\\par");
      }
  }
  else /* off switch */
  {
    code &= ~(OFF);  /* mask MSB */
    if (code == ARRAY)
      {
      fprintf(OUTPUTF,"\nEnd of environment: ARRAY");
      fprintf(fRtf,"\\par ***end of environment: ARRAY ***\\par");
      }
    if (code == EQNARRAY)
      {
      fprintf(OUTPUTF,"\nEnd of environment: EQNARRAY");
      fprintf(fRtf,"\\par ***end of environment: EQNARRAY ***\\par");
      }
    if (code == EQNARRAY_1)
      {
      fprintf(OUTPUTF,"\nEnd of environment: EQNARRAY* ");
      fprintf(fRtf,"\\par ***end of environment: EQNARRAY*    ***\\par");
      }
  }
}

/******************************************************************************/
void CmdHline(int code)
/******************************************************************************
 purpose: performs \\hline in tabular environment
 ******************************************************************************/
{
  int icell;

/*  FinishEmptyTabularLine(); */

  for (icell=0; icell<=ncell; icell++)
  {  if (FirstHline)
       {
         TheCellBorderTop[icell] = TRUE;
         NewCellBorderTop[icell] = TRUE;
       }
     else
       {
         TheCellBorderBottom[icell] = TRUE;
         NewCellBorderBottom[icell] = TRUE;
       }
     ;
  }
}

/******************************************************************************/
void CmdCline(int code)
/******************************************************************************
 purpose: performs \\cline in tabular environment
 ******************************************************************************/
{
  int icell;
  int fcell=0; int lcell=0;
  char ClineParm[WRKMAX] = {'\0'};
  char *Minus = NULL;

/*  FinishEmptyTabularLine(); */

  GetParam(ClineParm,WRKMAX-1);
  strcat(ClineParm," ");

  if(Debug)fprintf(OUTPUTF,"ClineParm='%s'\n",ClineParm);

  Minus = strrchr(ClineParm,'-');
  if (Minus == NULL)
  {  sscanf(ClineParm,"%d",&fcell); lcell=fcell;
     if(Debug)fprintf(OUTPUTF,"lcell=%d\n",lcell);
  } else
  {  sscanf(ClineParm,"%d-%d",&fcell,&lcell);
     if(Debug)fprintf(OUTPUTF,"fcell=%d lcell=%d\n",fcell,lcell);
  }
  
  if(Debug) fprintf(OUTPUTF,"ClineParm='%s'  limits= %d .. %d\n",ClineParm,fcell,lcell);

  for (icell=fcell; icell<=lcell; icell++)
  {  if (FirstHline)
       {
         TheCellBorderTop[icell] = TRUE;
         NewCellBorderTop[icell] = TRUE;
       }
     else
       {
         TheCellBorderBottom[icell] = TRUE;
         NewCellBorderBottom[icell] = TRUE;
       }
     ;
  }
}


/******************************************************************************/
void FinishEmptyTabularLine()
/******************************************************************************
 purpose: fills the previous tabular line with empty things up to \\row
 ******************************************************************************/
{ int icell;

  if (TabularColumn == 0)
  {
    fprintf(fRtf,"\\pard\\intbl\\sl-1"); TabularColumn = 1;
  }
/*  for (icell = TabularColumn; icell <= ncell; icell++) */
  for (icell = TabularColumn; icell <= NewNcell; icell++)
   {
     if (icell > TabularColumn)
     {  fprintf(fRtf,"\\sl-1\\cell");
     } else
     { fprintf(fRtf,"{}\\cell");
     }
   };
   fprintf(fRtf,"\\row\n"); IntblDone = FALSE;
   IgnoreSpaces = TRUE;
   TabularColumn = 0;
}

/******************************************************************************/
void TabularDbleSlash()
/******************************************************************************
 purpose: performs double backslash in tabular environment
 ******************************************************************************/
{
  switch (environ_type[iEnvCount])
  {
    case TABULAR :
      if (IntblDone) FinishEmptyTabularLine();
      IntblDone = FALSE;
      IgnoreSpaces = TRUE;
      ResetRowPattern();
      TabularColumn = 0;
      FirstHline = TRUE;
      break;
    default :
      warning(" \\\\ not in tabular environment, ignored");
  }
}

/******************************************************************************/
void TabulationChar()
/******************************************************************************
 purpose: performs & in tabular environment
 ******************************************************************************/
{
  switch (environ_type[iEnvCount])
  {
    case TABULAR :
      CheckIntbl();
      if (!IntblDone) fprintf(fRtf,"\n\\pard\\intbl\\sl-1");
      if (TabularColumn == 0) TabularColumn = 1;
      fprintf(fRtf,"\\cell");
      TabularColumn += 1;
      JustifyTabularField();
      break;
    default :
      warning(" & not in tabular environment, ignored");
  }
}

/******************************************************************************/
void ResetRowPattern()
/******************************************************************************
 purpose: resets the row formatting data according to initial \begin{tabular}
 ******************************************************************************/
{
      int i=0;
      for (i=0; i<MAXCELLS; i++)
      {
      	TheCellBorderLeft[i]=CellBorderLeft[i];
      	TheCellBorderRight[i]=CellBorderRight[i];
      	TheCellBorderTop[i]=CellBorderTop[i];
      	TheCellBorderBottom[i]=CellBorderBottom[i];
      	TheCellJustif[i]=CellJustif[i];
      }
}      
/******************************************************************************/
void OutputRowFormat()
/******************************************************************************
 purpose: outputs the row according to stored information about cells
 ******************************************************************************/
{
      int icell=0;
      
      fprintf(fRtf,"\n\\trowd ");
      if (environ_mode[iEnvCount-1] == PAR_CENTER) fprintf(fRtf,"\\trqc");
      if (environ_mode[iEnvCount-1] == PAR_RIGHT) fprintf(fRtf,"\\trqr");
      if (environ_mode[iEnvCount-1] == PAR_LEFT) fprintf(fRtf,"\\trql");
      for (icell = 1; icell <= ncell; icell++)
      {
        if (TheCellBorderLeft[icell]) fprintf(fRtf,"\\clbrdrl\\brdrs "); 
        if (TheCellBorderRight[icell]) fprintf(fRtf,"\\clbrdrr\\brdrs ");
        if (TheCellBorderTop[icell]) fprintf(fRtf,"\\clbrdrt\\brdrs ");
        if (TheCellBorderBottom[icell]) fprintf(fRtf,"\\clbrdrb\\brdrs ");
        fprintf(fRtf,"\\cellx%d ",cellx[icell]);
      }

}
/******************************************************************************/
void JustifyTabularField()
/******************************************************************************
 purpose: performs initialisation of a field in tabular environ
 ******************************************************************************/
{
      fprintf(fRtf,"\\pard \\intbl "); IntblDone = TRUE;

      if(CellJustif[TabularColumn] == 'c')
      {
      	fprintf(fRtf,"\\qc ");
      }
      else if(CellJustif[TabularColumn] == 'l')
      {
      	fprintf(fRtf,"\\li%d\\ri%d\\ql ",5*curr_fontsize[iEnvCount],
                                         5*curr_fontsize[iEnvCount]);
      }
      else if(CellJustif[TabularColumn] == 'p')
      {
      	fprintf(fRtf,"\\li%d\\ri%d\\ql ",5*curr_fontsize[iEnvCount],
                                         5*curr_fontsize[iEnvCount]);
      }
      else if(CellJustif[TabularColumn] == 'r')
      {
      	fprintf(fRtf,"\\li%d\\ri%d\\qr ",5*curr_fontsize[iEnvCount],
                                         5*curr_fontsize[iEnvCount]);
      }
      ;
      IgnoreSpaces = TRUE;
}

/**************************************************************************/
int Get2DoubleSlash(char *string, int size)
/**************************************************************************
   purpose: returns the sequence until the next \\ or \end{tabular}
        	    return: -> string = the line including the \\ or \end{.}
   parameter: string: look at purpose
        	 int: maximal number of characters for output string
     returns: success: string
              miss : string = ""
 **************************************************************************/
{
  char cThis = ' '; char OcThis = ' ';
  int kkk = 0;
  BOOL DbleSlashFound = FALSE;
  BOOL EndTabularFound = FALSE;
  BOOL EndTabLineFound = FALSE;
  char *ttt = NULL;

  char EndTabular[128] = "\\end{tabular}";

  kkk = 0;    /* get string from input stream */

  while (kkk < size-3 && !EndTabLineFound )
  {
    OcThis = cThis;
    if (fread(&cThis,1,1,fTex) < 1) numerror(ERR_EOF_INPUT);
    if (IsEol(&cThis)) {linenumber++; InputLineEmpty = TRUE; cThis = ' ';};

    if (cThis == ' ')
    {
      if (OcThis != ' ')
      { string[kkk] = cThis; kkk++; string[kkk] = '\0';
      }
    } else if (cThis == '%' && OcThis != '\\')
    { IgnoreTo('\n');
    } else if (cThis == '\\' & OcThis == '\\') /* \\ found */
    {
      string[kkk] = cThis; kkk++; string[kkk] = '\0';
      EndTabLineFound = TRUE;
      DbleSlashFound = TRUE;
    } else if (cThis == '}')  /* suspect \end{tabular} */
    {
      string[kkk] = cThis; kkk++; string[kkk] = '\0';
      ttt =strrchr(string,'\\');
      if (ttt != NULL)
      {
        EndTabularFound = (strcmp(EndTabular,ttt) == 0);
        if (EndTabularFound) EndTabLineFound = TRUE;
      }
    } else
    { string[kkk] = cThis; kkk++; string[kkk] = '\0';
    }
  }
  if (Debug) fprintf(OUTPUTF,"\n|%s|",string);
  if (EndTabularFound)
  { return 1;
  } else if (DbleSlashFound)
  { return 0;
  }
  else error("Neither \\\\ nor \\end{tabular} !");
}

/******************************************************************************/
void CmdTabular(int code)
/******************************************************************************
 purpose: converts the LaTex-Tabular to a similar Rtf-style
	  this converting is only partially
	  so the user has to convert some part of the Tabular-environment by hand
parameter: type of array-environment
 ******************************************************************************/
{
  char TabularParam[128];
  int icell;
  int DefaultCellSize = 2000;
  int CellSize[MAXCELLS]={2000};
  char CellSizeChar[128]="";
  char workstring[WRKMAX]="";
  char errormessage[256]="";
  char NextTabularLine[TABULARMAXLINE]="";
  int ichar;
  int pchar=0;
  BOOL pcell=FALSE;
  float DimenNumber=0.0;
  char CellUnit[3]="";

  switch (code)
  {
    case (TABULAR | ON):
      for (icell = 1; icell <= MAXCELLS; icell++)
      {
        CellBorderLeft[icell] = FALSE;
        CellBorderRight[icell] = FALSE;
        CellBorderTop[icell] = FALSE;
        CellBorderBottom[icell] = FALSE;
        CellJustif[icell] = ' ';
      }
    
      GetParam(TabularParam,127);
      PushEnvironment(TABULAR);

      /* compiling the tabular description */
      ncell = 0;
      pcell = FALSE;
      IntblDone = FALSE;
      if (Debug) fprintf(OUTPUTF,"\nTabular params=%s",TabularParam);
      for (ichar = 0; TabularParam[ichar] != '\0'; ichar++)
      { if (Debug) fprintf(OUTPUTF,"<%c>",TabularParam[ichar]);
        if ((TabularParam[ichar] == '{') && (TabularParam[ichar-1] == 'p'))
        {pcell = TRUE; pchar = 0; if(Debug) Message("pcell = TRUE!");}
        else if (pcell && (TabularParam[ichar] == '}'))
        { pcell = FALSE; CellSizeChar[pchar++]='\0';
          if (Debug) fprintf(OUTPUTF,"\nTabular-p=%s",CellSizeChar);
          sscanf(CellSizeChar,"%f%c%c",&DimenNumber,&CellUnit[0],&CellUnit[1]);
          CellUnit[2]='\0';
          CellSize[ncell] = DimenToTwips(DimenNumber,CellUnit);
          if (Debug) fprintf(OUTPUTF,"\nCell Size (twips)=%d",CellSize[ncell]);
        }
        else if (pcell)
        { if (pchar > 120) error("Tabular paragraph size > 120 chars");
          if (Debug) fprintf(OUTPUTF," '%c'",TabularParam[ichar]);
          CellSizeChar[pchar++] = TabularParam[ichar];
          CellSizeChar[pchar+1] = '\0';
        }
        else if (TabularParam[ichar] == '|')
        { CellBorderLeft[ncell+1] = TRUE;}
        else if (TabularParam[ichar] == 'c')
        {ncell++; CellJustif[ncell] = TabularParam[ichar]; pcell=FALSE; CellSize[ncell]=2000;}
        else if (TabularParam[ichar] == 'l')
        {ncell++; CellJustif[ncell] = TabularParam[ichar]; pcell=FALSE; CellSize[ncell]=2000;}
        else if (TabularParam[ichar] == 'r')
        {ncell++; CellJustif[ncell] = TabularParam[ichar]; pcell=FALSE; CellSize[ncell]=2000;}
        else if (TabularParam[ichar] == 'p')
        {ncell++; CellJustif[ncell] = TabularParam[ichar]; pcell=FALSE; CellSize[ncell]=2000;}
/*        else if (TabularParam[ichar] == ' '){ } */
/*        else if (TabularParam[ichar] == '\n'){ } */
        else
        { sprintf(errormessage,"Abnormal character '%c'[x%2x] in tabular p{}(%s)",
          TabularParam[ichar],TabularParam[ichar],TabularParam); error(errormessage);
        }
        ;
        CellBorderTop[ncell] = FALSE;
        CellBorderBottom[ncell] = FALSE;
      };
      CellBorderRight[ncell] = CellBorderLeft[ncell+1];

      if (!ParDone)
      {sprintf(errormessage,
      "\\begin{tabular} within a paragraph at line %d, \\par inserted",linenumber);
      warning(errormessage);
      CmdPar(0);
      };

      DefaultCellSize =
/* test..   DimenToTwips((17.0-(NumberColumns-1.0))/(NumberColumns*ncell+1.0),"cm"); */
        DimenToTwips((17.0-(NumberColumns-1.0))/(NumberColumns*ncell),"cm");

      for (icell = 1; icell <= ncell; icell++)
      {
        if (CellJustif[icell] != 'p'){ CellSize[icell] = DefaultCellSize; }
        cellx[icell] = cellx[icell-1] + CellSize[icell];
        ;
      };
      IntblDone = FALSE;
      IgnoreSpaces = TRUE;
      ResetRowPattern();
      TabularColumn = 0;
      FirstHline = TRUE;

      while(Get2DoubleSlash(NextTabularLine, TABULARMAXLINE-32) == 0)
      {
      	if (Debug > 0) fprintf(OUTPUTF,"\nNext: '%s'\n",NextTabularLine);
      	PerformMulticolumn(NextTabularLine);
        if (Debug > 1) fprintf(OUTPUTF,"\nNextTabularline='%s'",NextTabularLine);
        IncludeString(NextTabularLine);
      }

      if (Debug > 0) fprintf(OUTPUTF,"\nLast: '%s'\n",NextTabularLine);
      PerformMulticolumn(NextTabularLine);
      if (Debug > 1) fprintf(OUTPUTF,"\nNextTabularline='%s'",NextTabularLine);
      IncludeString(NextTabularLine);
      break;

    case (TABULAR_1 | ON):
      fprintf(OUTPUTF,"\nBegin of environment: TABULAR*  ");
      fprintf(fRtf,"{\\v\\par ***begin of environment: TABULAR*   ***\\par}");
      break;
    case (TABULAR | OFF):
/*      FinishEmptyTabularLine(); */
      OutputRowFormat();
      FinishEmptyTabularLine(); 
      FirstHline = TRUE;
      PopEnvironment(code);
      break;
    case (TABULAR_1 | OFF):
      fprintf(OUTPUTF,"\nEnd of environment: TABULAR*  ");
      fprintf(fRtf,"{\\v\\par ***end of environment: TABULAR*  ***\\par}");
      break;
  }
}

/******************************************************************************/
void CmdTable(int code)
/******************************************************************************
 purpose: converts the LaTex-Table to a similar Rtf-style
	  this converting is only partially
	  so the user has to convert some part of the Table-environment by hand
parameter: type of array-environment
 ******************************************************************************/
{
  if (code & ON)  /* on switch */
  {
    code &= ~(ON);  /* mask MSB */
    fprintf(OUTPUTF,"\n%s: WARNING: Following environment cannot be converted completely!",progname);
    fprintf(OUTPUTF,"\nSome parts of this environment have to be converted and corrected by hand");

    if (code == TABLE)
      {
      fprintf(OUTPUTF,"\nBegin of environment: TABLE");
      fprintf(fRtf,"{\\v\\par ***begin of environment: TABLE ***\\par}");
      }
    if (code == TABLE_1)
      {
      fprintf(OUTPUTF,"\nBegin of environment: TABLE*  ");
      fprintf(fRtf,"{\\v\\par ***begin of environment: TABLE*   ***\\par}");
      }
  }
  else /* off switch */
  {
    code &= ~(OFF);  /* mask MSB */
    if (code == TABLE)
      {
      fprintf(OUTPUTF,"\nEnd of environment: TABLE");
      fprintf(fRtf,"{\\v\\par ***end of environment: TABLE ***\\par}");
      }
    if (code == TABLE_1)
      {
      fprintf(OUTPUTF,"\nEnd of environment: TABLE*  ");
      fprintf(fRtf,"{\\v\\par ***end of environment: TABLE*  ***\\par}");
      }
  }
};

/******************************************************************************/
void CmdFrac(int code)
/******************************************************************************
 purpose: converts the \frac command (amsmath) with 2 arguments
parameter: type of operand (ignored)
 ******************************************************************************/
{
  char numerateur[WRKMAX]=" ";	
  char denominateur[WRKMAX]=" ";

  GetParam(numerateur,sizeof(numerateur));
  GetParam(denominateur,sizeof(denominateur));

  fprintf(fRtf,"\\\\F(");
  ConvertString(numerateur);
  fprintf(fRtf,";");
  ConvertString(denominateur);
  fprintf(fRtf,")");

}

/******************************************************************************/
void CmdIntegral(int code)
/******************************************************************************
 purpose: converts integral symbol + the "exponent" and "subscript" fields
parameter: type of operand
 ******************************************************************************/
{
  char exponent[WRKMAX]=" ";
  char subscript[WRKMAX]=" ";

  char cThis = '\n';
  float FloatFsize=20.0;
  float UpSize=6.0; int upsize;
  float DnSize=4.0; int dnsize;

  int NumScripts = 0;


  /* is there an exponent/subscript ? */

  cThis = ' ';
  while (cThis == ' ') 
  {   if ( (fread(&cThis,1,1,fTex) < 1))   numerror(ERR_EOF_INPUT);
      if (cThis == '%')
         { IgnoreTo('\n'); cThis = ' ';}
      if (IsEol(&cThis)) {linenumber++; cThis = ' ';}
  }

  if(cThis == '_')
  { GetParam(subscript, sizeof(subscript));  NumScripts++;
  } else if (cThis == '^')
  { GetParam(exponent, sizeof(exponent));  NumScripts++;
  } else
  { fseek(fTex,-1L,SEEK_CUR); NumScripts = 0; /* reread last character */
  }

  /* is there a second exponent/subscript ? */

  cThis = ' ';
  while (cThis == ' ') 
  {   if ( (fread(&cThis,1,1,fTex) < 1))   numerror(ERR_EOF_INPUT);
      if (Debug > 1) fprintf(OUTPUTF,"\nfound: '%c'",cThis);
      if (cThis == '%')
         { IgnoreTo('\n'); cThis = ' ';}
      if (IsEol(&cThis)) {linenumber++; cThis = ' ';}
  }

  if(cThis == '_' && subscript[0] == ' ')
  { GetParam(subscript, sizeof(subscript));  NumScripts++;
  } else if (cThis == '^' && exponent[0] == ' ')
  { GetParam(exponent, sizeof(exponent));  NumScripts++;
  } else
  { fseek(fTex,-1L,SEEK_CUR); NumScripts = 0; /* reread last character */
  }

  if (Debug > 0)
  { fprintf(OUTPUTF,"\nsubscript= '%s'",subscript);
    fprintf(OUTPUTF,"\nexponent= '%s'",exponent);
  }

  switch(code)
  {
    case 0 : fprintf(fRtf,"\\\\i\\\\in("); break;	
    case 1 : fprintf(fRtf,"\\\\i\\\\su("); break;
    case 2 : fprintf(fRtf,"\\\\i\\\\pr("); break;
    default: error("Illegal code to CmdIntegral");
  }

  PushEnvironment(EXPONENT);
  if(curr_fontsize[iEnvCount] > 14)
       { FloatFsize = 0.75*curr_fontsize[iEnvCount];
         curr_fontsize[iEnvCount] = FloatFsize;
       };
  fprintf(fRtf,"\\fs%d ",curr_fontsize[iEnvCount]);
  IncludeString(subscript);
  PopEnvironment(EXPONENT);

  fprintf(fRtf,";");

  PushEnvironment(EXPONENT);
  if(curr_fontsize[iEnvCount] > 14)
       { FloatFsize = 0.75*curr_fontsize[iEnvCount];
         curr_fontsize[iEnvCount] = FloatFsize;
       };
  fprintf(fRtf,"\\fs%d ",curr_fontsize[iEnvCount]);
  IncludeString(exponent);
  PopEnvironment(EXPONENT);
  fprintf(fRtf,";");
  /* include a thin high space */
  UpSize=1.25*curr_fontsize[iEnvCount]; upsize=UpSize;
  DnSize=0.75*curr_fontsize[iEnvCount]; dnsize=DnSize;
  fprintf(fRtf,"{\\fs2{\\up%d  }{\\dn%d  }}",upsize,dnsize);
  fprintf(fRtf,")");
}

/******************************************************************************/
void CmdQuad(int kk)
/******************************************************************************
 purpose: inserts kk quad spaces
 
 ******************************************************************************/
{
   int z;	
   fprintf(fRtf,"{\\emspace ");
   for (z=0; z<kk; z++) fprintf(fRtf," ");
   fprintf(fRtf,"}");
}
/******************************************************************************/
void CmdSpace(float kk)
/******************************************************************************
 purpose: inserts a space of width kk*space
 ******************************************************************************/
{
  float FloatFsize=20.0;

  PushEnvironment(GROUP);
  if(curr_fontsize[iEnvCount] > 14)
       { FloatFsize = kk*curr_fontsize[iEnvCount];
         curr_fontsize[iEnvCount] = FloatFsize+0.5;
       };
  fprintf(fRtf,"\\fs%d  ",curr_fontsize[iEnvCount]);
  PopEnvironment(GROUP);
}	
/******************************************************************************/
void CmdExponent(int code)
/******************************************************************************
 purpose: converts the exponent field after the '^' and the subscript after '_'
parameter: type of operand
 ******************************************************************************/
{
  char exponent[WRKMAX]=" ";
  char subscript[WRKMAX]=" ";
  char errormessage[1100]="";
  char cThis = '\n';
  float FloatFsize=20.0;
  float UpSize=6.0; int upsize;
  float DnSize=4.0; int dnsize;

  int NumScripts = 0;

  switch(code)
  {
    case EXPONENT:  GetParam(exponent,sizeof(exponent)); NumScripts=1; break;
    case SUBSCRIPT: GetParam(subscript,sizeof(subscript)); NumScripts=1; break;
    default: sprintf(errormessage,"Abnormal CmdExponent %d",code);
              error(errormessage);
  }

  /* is there a second exponent/subscript ? */


  cThis = ' ';
  while (cThis == ' ') 
  {   if ( (fread(&cThis,1,1,fTex) < 1))   numerror(ERR_EOF_INPUT);
      if (cThis == '%')
         { IgnoreTo('\n'); cThis = ' ';}
      if (IsEol(&cThis)) {linenumber++; cThis = ' ';}
  }

  switch(code)
  {
    case EXPONENT:   if(cThis == '_')
                     { GetParam(subscript, sizeof(subscript));  NumScripts=2;
                     } else
                     { fseek(fTex,-1L,SEEK_CUR); /* reread last character */
                     }
                     break;

    case SUBSCRIPT:  if(cThis == '^')
                     { GetParam(exponent, sizeof(exponent)); NumScripts=2;
                     } else
                     { fseek(fTex,-1L,SEEK_CUR); /* reread last character */
                     }
                     break;
  }

  

  if (Debug) fprintf(OUTPUTF,"\nexponen[, iEnvCount=%d, type=%d",iEnvCount,environ_type[iEnvCount]);
  PushEnvironment(EXPONENT);
  if (Debug) fprintf(OUTPUTF,"\nexponent, iEnvCount=%d, type=%d",iEnvCount,environ_type[iEnvCount]);
               
  if(curr_fontsize[iEnvCount] > 14)
       { FloatFsize = 0.75*curr_fontsize[iEnvCount];
         curr_fontsize[iEnvCount] = FloatFsize;
       };
  UpSize = 0.6*curr_fontsize[iEnvCount];  upsize=UpSize;     
  DnSize = 0.4*curr_fontsize[iEnvCount];  dnsize=DnSize;     

  if (NumScripts == 2)
  { fprintf(fRtf,"\\\\S(\\up6{");
    CmdSetFont(F_ROMAN);
    fprintf(fRtf,"\\fs%d ",curr_fontsize[iEnvCount]);
/*    ConvertString(exponent); */
    IncludeString(exponent);
    fprintf(fRtf,"};");
    CmdSetFont(F_ROMAN);
    fprintf(fRtf,"\\fs%d ",curr_fontsize[iEnvCount]);
/*    ConvertString(subscript); */
    IncludeString(subscript);
    fprintf(fRtf,")\n");
  } else if (code == EXPONENT)
  {
    fprintf(fRtf,"\\up%d ",upsize);
    CmdSetFont(F_ROMAN);
    fprintf(fRtf,"\\fs%d ",curr_fontsize[iEnvCount]);
/*    ConvertString(exponent); */
    IncludeString(exponent);
  } else if (code == SUBSCRIPT)
  {
    fprintf(fRtf,"\\dn%d ",dnsize);
    CmdSetFont(F_ROMAN);
    fprintf(fRtf,"\\fs%d ",curr_fontsize[iEnvCount]);
/*    ConvertString(subscript); */
    IncludeString(subscript);
  }

  if (Debug) fprintf(OUTPUTF,"\nexponent, iEnvCount=%d, type=%d",iEnvCount,environ_type[iEnvCount]);
  PopEnvironment(EXPONENT);
  if (Debug) fprintf(OUTPUTF,"\nexponen], iEnvCount=%d, type=%d",iEnvCount,environ_type[iEnvCount]);
}

/*****************************************************************************/
char * strpos(char *source, char *old_text)
/*****************************************************************************
recherche la chaine 'old_text' dans 'source' et retourne le pointeur
vers la position de la chaine egale  'old_text'.
******************************************************************************/
{
  char * found_string = NULL;
  char * test_string = source;
  int z = 0;
  int old_len = strlen(old_text);


  if (source == NULL) error("null argument to strpos");
  if (old_len == 0) return (NULL);
  if(Debug)  fprintf(OUTPUTF,"strpos('%s','%s')\n",source,old_text); 
  for (z = 0; z <= strlen(source) - old_len; z++)
  {
    found_string = strchr(test_string, old_text[0]);
    if (found_string == NULL) return(NULL);
    if (strlen(found_string) < old_len) return (NULL);
    if (strncmp(found_string,old_text,old_len) == 0)
    {  if(Debug) fprintf(OUTPUTF,"found: '%s'<->'%s'\n",found_string,old_text); 
       return (found_string);
    }
    test_string++;
  }
  return(NULL);
}	

/*****************************************************************************/
int PerformMulticolumn(char *NextTabularLine)
/*****************************************************************************
recherche la chaine \multicolumn dans une ligne de Tabular
******************************************************************************/
{
  char TheNextTabularLine[TABULARMAXLINE]="";
  char TheEndTabularLine[TABULARMAXLINE]="";
  char *TheRemainder;

  int NumberAmpersandBefore = 0;
  int NumberMulticol = 0;
  int BracketCount = 0;
  int k, kk, m, kk_dep;

  BOOL NumberBegun;
  BOOL SpecifFound;
  BOOL Finished;

  TabularSpecialFormat = FALSE;

  NewNcell = ncell;  /* this must be always performed -> the actual ncell to be used */

  for (k=0; k <= ncell; k++)
  {
    NewCellx[k] = cellx[k];
    NewCellBorderTop[k] = TheCellBorderTop[k];
    NewCellBorderBottom[k] = TheCellBorderBottom[k];
    NewCellBorderLeft[k] = TheCellBorderLeft[k];
    NewCellBorderRight[k] = TheCellBorderRight[k];
    NewCellJustif[k] = CellJustif[k];
  }

    if (Debug > 1)
    {for (m = 1; m <= NewNcell; m++)
      { if (m == 1) fprintf(OUTPUTF,"\n NewCellx");
        fprintf(OUTPUTF," [%d]=%d", m, NewCellx[m]);
      }
      for (m = 1; m <= ncell; m++)
      { if (m == 1) fprintf(OUTPUTF,"\n    cellx");
        fprintf(OUTPUTF," [%d]=%d", m, cellx[m]);
      }
    }

  strcpy(TheNextTabularLine, NextTabularLine);

  while ((TheRemainder = strpos(TheNextTabularLine,"\\multicolumn")) != NULL)
  {
    strcpy(TheEndTabularLine, TheRemainder);
    TheRemainder[0] = '\0';
    
    if (Debug > 1) fprintf(OUTPUTF,"\nTheEndTabularline='%s'",TheEndTabularLine);
    if (Debug > 1) fprintf(OUTPUTF,"\n1-TheNextTabularline='%s'",TheNextTabularLine);
    NumberAmpersandBefore=0;

    /* count the number of '&' before \multicolumn */

    for (k=0; k<strlen(TheNextTabularLine); k++)
    {
      if(TheNextTabularLine[k] == '&' &&
          (k == 0 || TheNextTabularLine[k-1] != '\\'))
      { NumberAmpersandBefore ++;
      }
    }

    /* Now get the number of columns to be posted as one */

    BracketCount = 0;
    NumberMulticol = 0;
    Finished = FALSE; NumberBegun = FALSE;
    for (k = strlen("\\multicolumn");  (k< strlen(TheEndTabularLine) && !Finished) ; k++)
    { kk_dep = k;
      if (Debug > 0) fprintf(OUTPUTF,"\nk=%d, TheEndTabularLine[k]='%c'",k,TheEndTabularLine[k]);
      switch(TheEndTabularLine[k])
      {
      	case ' ': break;
      	case '{': if (NumberBegun)
                  { Finished = TRUE;
                  } else
                  { BracketCount++;
                  }
                  break;
      	case '}': BracketCount--;
                  if (BracketCount < 0)
                  { sprintf(ErrorMsgText,"syntax error '%c' in count: '%s'",
                    TheEndTabularLine[k], TheEndTabularLine);
                    error (ErrorMsgText);
                  }
                  if (BracketCount == 0)
                  { Finished = TRUE;  kk_dep++; /* skip that character */
                  }
                  break;
        case '0': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '1': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '2': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '3': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '4': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '5': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '6': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '7': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '8': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
        case '9': NumberMulticol = 10*NumberMulticol+TheEndTabularLine[k]-'0'; 
                  NumberBegun = TRUE; break;
      	default:  if (NumberBegun)
                  { Finished = TRUE;
                  } else
                  { sprintf(ErrorMsgText,"syntax error '%c' in count: '%s'",
                    TheEndTabularLine[k], TheEndTabularLine);
                    error (ErrorMsgText);
                  }
      }
    }

    if (Debug > 0) fprintf(OUTPUTF,"\n3-TheNextTabularline='%s', k=%d",TheNextTabularLine,k); 
    if (Debug > 0) fprintf(OUTPUTF,"\n3-TheEndTabularline='%s', k=%d",TheEndTabularLine,k); 
    if (Debug > 0) fprintf(OUTPUTF,"\nNumberMulticol = %d",NumberMulticol);

    /* Now adjust the cell positions */

    NewNcell = NewNcell - NumberMulticol + 1;

    for (m= NumberAmpersandBefore+1; m<=NewNcell ; m++)
    {
      NewCellx[m] = NewCellx[m+NumberMulticol-1];
      NewCellBorderTop[m] = TheCellBorderTop[m+NumberMulticol-1];
      NewCellBorderBottom[m] = TheCellBorderBottom[m+NumberMulticol-1];
      NewCellBorderLeft[m] = TheCellBorderLeft[m+NumberMulticol-1];
      NewCellBorderRight[m] = TheCellBorderRight[m+NumberMulticol-1];
      NewCellJustif[m] = CellJustif[m+NumberMulticol-1];
    }
    if (Debug > 1) fprintf(OUTPUTF,"\n4-TheNextTabularline='%s'",TheNextTabularLine);

    if (Debug > 1)
    { for (m = 1; m <= NewNcell; m++)
      { if (m == 1) fprintf(OUTPUTF,"\n NewCellx");
        fprintf(OUTPUTF," [%d]=%d", m, NewCellx[m]);
      }
    }

    for (m = NewNcell+1; m < ncell; m++)
    {
      NewCellBorderTop[m] = FALSE;
      NewCellBorderBottom[m] = FALSE;
      NewCellBorderLeft[m] = FALSE;
      NewCellBorderRight[m] = FALSE;
      NewCellJustif[m] = 'p';
    }

    if (Debug > 0) fprintf(OUTPUTF,"\n5-TheNextTabularline='%s'",TheNextTabularLine);
    if (Debug > 0) fprintf(OUTPUTF,"\n5-TheEndTabularline='%s', k=%d",TheEndTabularLine,k); 
    
    /* Now scan the description */

    BracketCount = 0;
    Finished = FALSE; SpecifFound = FALSE;
    for (kk = kk_dep;  (kk< strlen(TheEndTabularLine) && !Finished) ; kk++)
    {
      if (Debug > 0) fprintf(OUTPUTF, "\nDescr: '%c'", TheEndTabularLine[kk]);
      switch(TheEndTabularLine[kk])
      {
      	case ' ': break;
      	case '{': if (SpecifFound)
                  { Finished = TRUE;
                  } else
                  { BracketCount++;
                  }
                  break;
      	case '}': BracketCount--;
                  if (BracketCount < 0)
                  { sprintf(ErrorMsgText,"syntax error '%c' in spefic.: '%s'",
                    TheEndTabularLine[kk], TheEndTabularLine);
                    error (ErrorMsgText);
                  }
                  if (BracketCount == 0) Finished = TRUE;
                  break;
        case '|': if (!SpecifFound)
                  {
                    NewCellBorderLeft[NumberAmpersandBefore+1] = TRUE;
                  } else
                  {
                    NewCellBorderRight[NumberAmpersandBefore+1] = TRUE;
                  }
                  break;
        case 'c': NewCellJustif[NumberAmpersandBefore+1] = TheEndTabularLine[kk];
                  SpecifFound = TRUE;
                  break;
        case 'r': NewCellJustif[NumberAmpersandBefore+1] = TheEndTabularLine[kk];
                  SpecifFound = TRUE;
                  break;
        case 'l': NewCellJustif[NumberAmpersandBefore+1] = TheEndTabularLine[kk];
                  SpecifFound = TRUE;
                  break;
        default : 
                  { sprintf(ErrorMsgText,"syntax error '%c' in specif.: '%s'",
                    TheEndTabularLine[kk], TheEndTabularLine);
                    error (ErrorMsgText);
                  }
                  break;
      }
    }
    /* Now, remove \\multicols{#1}{#2} and keep the text */

    TheRemainder = TheEndTabularLine;
    TheRemainder += kk;

    if (Debug > 1) fprintf(OUTPUTF,"\nRemainder= '%s'",TheRemainder);
    if (Debug > 1) fprintf(OUTPUTF,"\nTheNextTabularline='%s'",TheNextTabularLine);
    strcat(TheNextTabularLine,TheRemainder);
    if (Debug > 1) fprintf(OUTPUTF,"\nTheNextTabularline='%s'",TheNextTabularLine);

    TabularSpecialFormat = TRUE;

    if (Debug > 1)
    { for (m = 1; m <= NewNcell; m++)
      { if (m == 1) fprintf(OUTPUTF,"\n NewCellx");
        fprintf(OUTPUTF," [%d]=%d", m, NewCellx[m]);
      }
    }
  }
  /* OutputRowFormatSpecial(); */
  /*  IntblDone = TRUE; */
  if (TabularSpecialFormat)
  {
    strcpy(NextTabularLine, TheNextTabularLine);
    if (Debug > 0) fprintf(OUTPUTF,"\nNextTabularline='%s'",NextTabularLine);
  }
}

/******************************************************************************/
void OutputRowFormatSpecial()
/******************************************************************************
 purpose: outputs the row according to special stored information about cells
 ******************************************************************************/
{
      int icell=0;
      
      fprintf(fRtf,"\n\\trowd ");
      if (environ_mode[iEnvCount-1] == PAR_CENTER) fprintf(fRtf,"\\trqc");
      if (environ_mode[iEnvCount-1] == PAR_RIGHT) fprintf(fRtf,"\\trqr");
      if (environ_mode[iEnvCount-1] == PAR_LEFT) fprintf(fRtf,"\\trql");
      for (icell = 1; icell <= NewNcell; icell++)
      {
        if (NewCellBorderLeft[icell]) fprintf(fRtf,"\\clbrdrl\\brdrs "); 
        if (NewCellBorderRight[icell]) fprintf(fRtf,"\\clbrdrr\\brdrs ");
        if (NewCellBorderTop[icell]) fprintf(fRtf,"\\clbrdrt\\brdrs ");
        if (NewCellBorderBottom[icell]) fprintf(fRtf,"\\clbrdrb\\brdrs ");
        fprintf(fRtf,"\\cellx%d ",NewCellx[icell]);
      }

}

