/*
 * $Id: main.c,v 1.4 1997/10/01 19:00:00 D.Taupin Exp $
 * History:
 * $Log: main.c,v $
 * Revision 1.4  1997/10/01   D. Taupin
 * Corrected some bugs and added \OE, \oe, Umlauts, cedillas
 *
 * Revision 1.3  1994/06/21  08:14:11  ralf
 * Corrected some bugs
 *
 * Revision 1.2  1994/06/17  14:19:41  ralf
 * Corrected various bugs, for example interactive read of arguments
 *
 * Revision 1.1  1994/06/17  11:26:29  ralf
 * Initial revision
 */
/***************************************************************************
     name : main.c
   author : DORNER Fernando, GRANZER Andreas + TAUPIN Daniel
  purpose : main convert-routine from LaTex2Rtf/ltx2rtf
 *****************************************************************************/


/****************************** includes *************************************/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#ifdef KPATHSEA
#include <kpathsea/config.h>
#include <kpathsea/c-dir.h>
#include <kpathsea/tex-file.h>
#include <kpathsea/lib.h>
#include <kpathsea/variable.h>
#include <kpathsea/getopt.h>
#include <kpathsea/absolute.h>
#include <kpathsea/expand.h>
#endif
/* introduced by D. Taupin to have O_BINARY defined */
#ifdef MSDOS
#include <fcntl.h>
#endif
/* end O_BINARY */
#include "main.h"
#include "commands.h"
#include "funct1.h"
#include "fonts.h"
#include "stack.h"
#include "funct2.h"
#include "cdirect.h"
#include "ignore.h"
#include "version.h"
/****************************************************************************/

/********************************* macros ***********************************/
#ifdef WIN32

#if defined (_POSIX_)
#  define FPOS_EQ(p1, p2) ((p1) == (p2))
#else /* _POSIX_ */
#  if     !__STDC__ && _INTEGRAL_MAX_BITS >= 64
#    define FPOS_EQ(p1, p2) ((p1) == (p2))
#  else
#    define FPOS_EQ(p1, p2) \
            ((p1).lopart == (p2).lopart \
             && (p1).hipart == (p2).hipart)
#  endif
#endif  /* _POSIX_ */
#else
#define FPOS_EQ(p1, p2) ((p1) == (p2))
#endif
/****************************************************************************/

/********************************* global variables *************************/
FILE *fTex = NULL;		/* file pointer to Latex file */
FILE *fRtf = NULL;		/* file pointer to RTF file */
FILE *fStd = NULL;		/* file pointer to LOG file */
FILE *fMathTeX = NULL;          /* Maths-LaTex-File-Pointer */
FILE *fUtil = NULL;             /* Utility File-Pointer */
FILE *fPbm = NULL;		/* file pointer to input PBM file */
FILE *fDat = NULL;		/* file pointer to LaTeX output data */
BOOL StdToScreen = TRUE;        /* NO need to duplicate error messages */
char *progname;			/* name of the executable file */
char latexname[PATHMAX] = {'\0'};	/* name of LaTex-File */
fpos_t pos_begin_kill;
char InitialDir[PATHMAX] = {'\0'};  /* save initial directory */
char TempDir[PATHMAX] = {'\0'};	/* Temp directory */
char exe_path[1024] = {'\0'};

BOOL AutoNumSections = FALSE;/* DT */
BOOL AutoNumItems = FALSE;   /* DT */
BOOL AutoNumBibItems = FALSE;   /* DT */
BOOL Code850 = FALSE;   /* DT */
BOOL CodeISO = FALSE;   /* DT */
BOOL Debug = FALSE;
int iEnvCount = 0;           /* DT */
BOOL MakeEquationsTeX = FALSE;
BOOL MakeMathsTeX = FALSE;
BOOL TabularSpecialFormat = FALSE; /* activated if \multicolumns in tabular line */
int NumberColumns = 1;
int ColSep = 360; /* columns separation in TWIPS */
#ifndef NO_ADD_BRACES  
BOOL AddBraces = TRUE;
#else
BOOL AddBraces = FALSE;
#endif

int paperwidth = 12240;
int paperheight = 15840;
int textwidth =  12240;
int textheight =  12240;
int hoffset = 0;
int voffset = 0;


/****************************************************************************/
/*** function prototypes ***/
BOOL PrepareTex(char *filename, FILE **f);  /* opens file for reading */
BOOL PrepareStdout(char *filename, FILE **f);  /* opens file for std output */
BOOL PrepareRtf(char * filename, FILE **f); /* creates file and writes */
        				   /* RTF header */
BOOL OpenMathTeX(char *filename, FILE **f);  /* opens file for TeX maths */

BOOL CloseTex(FILE **f);
BOOL CloseRtf(FILE **f);
BOOL CloseMath(FILE **f);

BOOL TranslateCommand();  /* converts commands */
BOOL TranslateSpecKey();  /* converts special keys */
void *GetCommandFunc(char *cCommand);
void numerror(int num);
void errorchar(char text);
void OutputError(char *text);
void Message(char *text);
BOOL GetTempDir();
BOOL SaveInitialDir();
BOOL RestoreInitialDir();
BOOL SetTempDir();
void Usage();
/* void CheckNospace(char *DirectoryName); */

/****************************************************************************/
int main(int argc, char **argv)
/****************************************************************************
purpose: checks parameter and starts convert routine
params:  command line arguments argc, argv
globals: initializes in- and output-file fTex, fRtf
 ****************************************************************************/
{

  int c, errflag = 0;
  int zz = 0;
  char *input = NULL;
  char *output = NULL;
  char *ttt = NULL;
  char def_out[PATHMAX] = {"\0"};
  char errormessage[256] = "";
#ifndef KPATHSEA
#ifndef HAS_GETOPT
  extern int getopt(int, char **, char *);
#endif
  extern char *optarg;
  extern int optind, opterr;
#endif

  fTex = stdin;
  fStd = stdout;

  PushEnvironment(HEADER);
  PushEnvironment(DOCUMENT);
  progname=argv[0];

#ifdef KPATHSEA
   kpse_set_progname(progname);
#endif

  printf("%s:\n %s", progname, Version);

  sprintf(exe_path,"%s",progname);
    ttt = strrchr(exe_path,'/');
    if (ttt != NULL)
    { ttt[0] = '\0';
    } else
    { exe_path[0] = '\0';
    }
  
  if (argc <= 1)
  { Usage(); return(1);
  }


  
  SaveInitialDir();
  GetTempDir();
  
  while((c = getopt(argc, argv, "PIVDBMZmhC:o:-:")) != EOF)
  {
    switch(c)
    {
      case 'o':
        output = optarg;
        break;
      case '-':
        if(strcmp(optarg,"-help") == 0)
        { Usage(); return(1);
        } else
        { errflag = 1;
        }
        break;
/* new options DT */	
      case 'h':
        Usage();
        return(1);
      case 'D':
        Debug = TRUE;
        break;
      case 'B':
        AutoNumBibItems = TRUE;
        break;
      case 'I':
        AutoNumItems = TRUE;
        break;
      case 'P':
        AutoNumSections = TRUE;
        break;
      case 'm':
        MakeEquationsTeX = TRUE;
        break;
      case 'M':
        MakeEquationsTeX = TRUE;
        MakeMathsTeX = TRUE;
        break;
      case 'Z':
        AddBraces = FALSE;
        break;
      case 'C':
        if (strcmp(optarg,"850") == 0)
        {
          Code850 = TRUE;
          fprintf(fStd, "\nConverting Code Page=%s",optarg);
        }
        else if (strcmp(optarg,"ISO") == 0)
        {
          CodeISO = TRUE;
          fprintf(fStd, "\nConverting Code Page=%s",optarg);
        }
        else
        {
            sprintf(errormessage,"Code Page %s not accepted.",optarg);
            warning(errormessage);
       	};
        break;
/* end new options DT */	
      default:
        errflag = 1;
    }
  }
  if(argc > optind + 1 || errflag)
  {
    Usage();
    return(1);
  }
  if(argc == optind + 1)
  {
    input = argv[optind];
    if (strlen(input) > PATHMAX-5) {error("LaTeX name too long!");};
    strcpy(latexname,input);
    strcpy(def_out,latexname);
    if (ttt = strrchr(def_out,'.'))
    {
      ttt[0] = '\0';
    } else
    {
      /* The user has specified no extension, .tex will be tried by default */
      strcat(latexname, ".tex");
    }
    strcat(def_out,".rtf");
    if (strlen(def_out) > PATHMAX-1) {error("LaTeX name too long!");};
    if (strlen(latexname) > PATHMAX-1) {error("LaTeX name too long!");};
  };

  if (output == NULL)
  { output = def_out; sprintf(errormessage,"Output taken from LaTeX name: %s",
    output); OutputError(errormessage);
  }

  if (Debug) fprintf(OUTPUTF,"\nPath to ltx2rtf executable = %s",exe_path);
  
  input = latexname;
#ifdef KPATHSEA
  input = concat(kpse_var_expand("$KPSE_DOT/"), input);
  output = concat(kpse_var_expand("$KPSE_DOT/"), output);
#endif

  GetConfigs();
  
  CmdPaper(0); /* initialize paper sizes to A4 */
  PrepareTex(input,&fTex);
  PrepareRtf(output,&fRtf);
  Push(1,0);
  /* Convert(fTex, fRtf); */
  Convert();
  /* For vi {{{{{ */
  if(AddBraces)
    fprintf(fRtf,"}}}}}");

  CloseTex(&fTex);
  CloseRtf(&fRtf);
  RemoveFontlist();
  sprintf(errormessage,"Final iEnvCount=%d",iEnvCount);
  Message(errormessage);
  sprintf(errormessage,"Convert successfull\r");
  OutputError(errormessage);

  return(0);
}


/****************************************************************************/
/* Global Flags for Convert Routine */
/****************************************************************************/
BOOL AcceptInputText = TRUE;
int RecursLevel = 0;
int BracketLevel = 1;
int ret = 0;
BOOL mbox = FALSE;
BOOL MathMode[MAXENVIRONS] = {FALSE};
BOOL bNewPar = FALSE;
int indent = 0;
int right_indent = 0;
int left_item_indent = 240;  /* DT */
int left_enum_indent = 360;  /* (3*left_item_indent)/2; */
int indent_quantum = 512;    /* DT */
int enum_level =0;           /* DT */
BOOL ParDone = FALSE;
int ConvertFlag;
BOOL bPard = TRUE;
BOOL bInDocument = FALSE;
int tabcounter = 0;
int fontsize = 20;
int curr_fontsize[MAXENVIRONS] = {20};
int curr_fontbold[MAXENVIRONS] = {0};
int curr_fontital[MAXENVIRONS] = {0};
int curr_fontscap[MAXENVIRONS] = {0};
int curr_fontnumb[MAXENVIRONS] = {0};
int environ_type[MAXENVIRONS] = {0};
int environ_mode[MAXENVIRONS] = {0};
BOOL twocolumn = FALSE;
BOOL titlepage = FALSE;
BOOL article = FALSE;
BOOL report = FALSE;
BOOL book = FALSE;
BOOL TABBING_ON = FALSE;
BOOL TABBING_RETURN = FALSE;
BOOL TABBING_ON_itself = FALSE;
BOOL TITLE_AUTHOR_ON = FALSE;
BOOL IgnoreSpaces = FALSE;
BOOL GermanMode = FALSE;  /* switches support for germanstyle on or off */
/* the Germand Mode supports most of the commands defined in GERMAN.STY file
   by H.Partl(TU Wien) 87-06-17 */

char alignment[MAXENVIRONS] = {JUSTIFIED};	/* default for justified: */
int parindent[MAXENVIRONS] = {360};	/* default is zero ??? */
int DefaultIndent = 360;    /* DT */
int section_style[SECT_MAXLEVEL] = {0};
int section_fontsize[SECT_MAXLEVEL] = {20};
int section_sb[SECT_MAXLEVEL] = {240};  /* space before section titles */
int section_sa[SECT_MAXLEVEL] = {60};  /* space before section titles */
int title_fontsize = 20;
int author_fontsize = 20;
char section_fontdesc[100][SECT_MAXLEVEL] = {'\0'};
char TheFont[MAXPNLEN] = "";
int TabularColumn = 0;
BOOL IntblDone = FALSE;
BOOL ParagraphEmpty = TRUE;
BOOL CellBorderLeft[MAXCELLS]={FALSE};
BOOL CellBorderRight[MAXCELLS]={FALSE};
BOOL CellBorderTop[MAXCELLS]={FALSE};
BOOL CellBorderBottom[MAXCELLS]={FALSE};
char CellJustif[MAXCELLS]={' '}; /* DT */
int FutureSkip = 0;   /* DT stores the medskip, smallskip, bigskip requested */
BOOL NextIndent = TRUE;   /* DT shall next paragraph be indented */
BOOL InputLineEmpty = TRUE;  /* DT */
int EqNumber = 0; /* DT */
char LaTeXingCommand[PATHMAX] = "";
char PBMingCommand[PATHMAX] = "";
char LaTeXinputName[PATHMAX]="eqn-xxx.tmp";
char TeXoutputName[PATHMAX]="dir/eqn-xxx.tmp";
char TeXoutputLog[PATHMAX]="dir/eqn-xxx.log";
char TeXoutputAux[PATHMAX]="dir/eqn-xxx.aux";
char TeXoutputNone[PATHMAX]="eqn-xxx";
char TeXoutput[PATHMAX]="dir/eqn-xxx";
char PBMname[PATHMAX]="dir/eqn-xxx.pbm";
char *UsePackageText[PATHMAX]= { NULL };
char *OptPackageText[PATHMAX]= { NULL };

   
long linenumber=1;		       /* counts lines in the LaTex-document */
int pbm_linelength=0;                  /* line length in pbm file read */ 
int pbm_linenumber=0;                  /* number of lines in pbm file read */

char PointSize[10] = "10pt";
char normalsize[20] = "\\normalsize";
char *LatexSize[MAXENVIRONS] = {normalsize};

char ErrorMsgText[256] = {'\0'};

BOOL IsEol(char *c)
{
  int cNext;
  if (*c == '\n') return TRUE;
    else if (*c == '\r') {
      cNext = getc(fTex);
      if ((cNext != EOF) && (cNext != '\n'))
	ungetc(cNext, fTex);
      *c = '\n';
      return TRUE;
    }
  return FALSE;
}

/****************************************************************************/
BOOL Convert()
/****************************************************************************
purpose: converts inputfile and writes result to outputfile
globals: fTex, fRtf and all global flags for convert (see above)
 ****************************************************************************/
{
  char errormessage[256];
  char exponent[1000];
  char cLastNoSpace = 'a';
  int PopLevel=0,PopBrack,PPopLevel,PPopBrack,size,retlevel;
  int count = 0;
  int i,kchar;
  float FloatFsize = fontsize;
  char cThishilf;
  char cThis = '\n';
  char cLast = '\n';
  char cLast2 = '\n';

  RecursLevel++;
  Push(RecursLevel,BracketLevel);
  ++ConvertFlag;
  while ((fread(&cThis, 1,1,fTex) == 1) && AcceptInputText)
  {
    if (IsEol(&cThis)) linenumber++;
    if (cThis == '\t') cThis = ' ';  /* tabulation = space */
    if ( (cThis != ' ') && (cThis != '\n') && (cThis != '%') ) IgnoreSpaces = FALSE; /* DT */
    if ( (cThis != ' ') && (cThis != '\n') ) InputLineEmpty = FALSE;

/* insert math-ign.c */      
    
    switch(cThis)
    {
    case '\\':{
              fpos_t pos1,pos2;
              fgetpos(fRtf,&pos1);
              Push(RecursLevel,BracketLevel);
              TranslateCommand();

              /* erase double identic values on stack top */
              for(;;)
              {
        	if ((size = Pop(&PPopLevel, &PPopBrack)) <= 0 )
        	{
        	  Push(PPopLevel,PPopBrack);
        	  break;
        	}
        	if ((size = Pop(&PopLevel, &PopBrack)) <= 0 )
        	{
        	  Push(PopLevel,PopBrack);
        	  break;
        	}
        	if ( (PPopLevel == PopLevel) && (PPopBrack == PopBrack) )
        	{
        	  Push(PopLevel,PopBrack);
        	}
        	else
        	{
        	  Push(PopLevel,PopBrack);
        	  Push(PPopLevel,PPopBrack);
        	  break;
        	}
              };

              if (ret > 0)
              {
        	--ret;
        	--RecursLevel;
        	return TRUE;
              }

              cThis = '\\';
              fgetpos(fRtf,&pos2);
              if (FPOS_EQ(pos1, pos2))    /* no RTF output */
              {
        	if (cLast == '\n')
        	  cThis = '\n';
        	else
        	  cThis = ' ';
              }
              break;
              }
    case '%': IgnoreTo('\n');
              if (IgnoreSpaces)
                {cThis = '\4';} else {cThis = ' ';};   /* Test 22 mai */
              break;
    case '&': TabulationChar();
              break;
    case '{':
              Push(RecursLevel,BracketLevel);
              ++BracketLevel;
              PushEnvironment(GROUP);
              break;
    case '}':
              PopEnvironment(GROUP);
              BracketLevel--;
              PPopLevel = RecursLevel;
              PPopBrack = BracketLevel;
              size = Pop(&PopLevel, &PopBrack);
              while ( (size = Pop(&PopLevel, &PopBrack)) >= 0 )
              {
        	if ( PopBrack < BracketLevel )
        	{
        	  break;
        	}
        	PPopLevel = PopLevel;
        	PPopBrack = PopBrack;
              } /* while */
              Push(PopLevel,PopBrack);	/* push back */
              retlevel = PPopLevel;
              ret = RecursLevel-retlevel;
              Push(retlevel,BracketLevel);

              if (ret > 0)
              {
        	ret--;
        	RecursLevel--;
        	return TRUE;
              }
              else
        	break;
#if 0
    case '\r':sprintf(errormessage,"error in input file: %s at linenumber: %ld",latexname,linenumber);
              error(errormessage);
              exit(-1);
              break;
#endif
    case ' ': if (!bInDocument) continue;
              /* DT */
              if ( (cLast != ' ') && (cLast != '\n') &&
                (IgnoreSpaces == FALSE) )
              {
        	if (mbox == FALSE)
               /*   if (bNewPar == FALSE)*/
        	  if(MathMode[iEnvCount])
                  {
                  } else
                  { fprintf(fRtf," ");
                  }
        	else
        	  fprintf(fRtf,"\\~");
              };
              break;
    case '~': if (!bInDocument) numerror(ERR_WRONG_COMMAND);
              fprintf(fRtf,"\\~");
              break;
    case '\n':
              tabcounter=0;


              if (InputLineEmpty)
              {
              	 if (bInDocument) CmdPar(0);
              }else if (!IgnoreSpaces)
              {
        	  if(bInDocument && !MathMode[iEnvCount]) fprintf(fRtf," ");	 /* treat as 1 space */
              };
              InputLineEmpty = TRUE;
              break;
    case '^': if (!bInDocument) numerror(ERR_WRONG_COMMAND);
                CmdExponent(EXPONENT); 
                break;
    case '_': if (!bInDocument) numerror(ERR_WRONG_COMMAND);
                CmdExponent(SUBSCRIPT);
                break;
    case '$': count++;
               while ((fread(&cThishilf,1,1,fTex) >= 1) && (cThishilf == '$'))
        	   count++;
               switch (count)
               {
        	 case 1: CmdFormula(FORM_DOLLAR);
        		 break;
        	 case 2: CmdFormula(FORM_DISPLAY);
        		 break;
        	 default:
        	         sprintf(errormessage,
                         "More than two consecutive $ at line %d",linenumber);
                         error(errormessage);
               }
               fseek(fTex,-1L,SEEK_CUR); /* reread last character */
               count = 0;

              break;
    case '-' : if (MathMode[iEnvCount])
               { fprintf(fRtf,"\\endash ");
               } else
               { count++;
                 DoBeginParagraph();
                 while ((fread(&cThishilf,1,1,fTex) >= 1) && (cThishilf == '-'))
          	   count++;
                 switch (count)
                 {
                   case 1: fprintf(fRtf,"-");
        	           break;
                   case 2: fprintf(fRtf,"\\endash ");
        	           break;
                   case 3: fprintf(fRtf,"\\emdash ");
        	           break;
                   default:
                 	 { for (i = count-1; i >= 0; i--)
                	   fprintf(fRtf,"-");
        	         }
                 }
                 fseek(fTex,-1L,SEEK_CUR); /* reread last character */
                 count = 0;
               }
               break;
    case '<' : count++;
               DoBeginParagraph();   	
               while ((fread(&cThishilf,1,1,fTex) >= 1) && (cThishilf == '<'))
        	   count++;
               switch (count)
               {
        	 case 1: fprintf(fRtf,"<");
        		 break;
        	 case 2: fprintf(fRtf,"\\ldblquote ");
        		 break;
        	 default:
        	 { for (i = count-1; i >= 0; i--)
        	   fprintf(fRtf,"<");
        	 }
               }
               fseek(fTex,-1L,SEEK_CUR); /* reread last character */
               count = 0;
        	break;
    case '>' : count++;
               DoBeginParagraph();   	
               while ((fread(&cThishilf,1,1,fTex) == 1) && (cThishilf == '>'))
        	   count++;

               switch (count)
               {
        	 case 1: fprintf(fRtf,">");
        		 break;
        	 case 2: fprintf(fRtf,"\\rdblquote ");
        		 break;
        	 default:
        	 { for (i = count-1; i >= 0; i--)
        	   fprintf(fRtf,">");
        	 }
               }
               fseek(fTex,-1L,SEEK_CUR); /* reread last character */
               count = 0;
        	break;
    case '\4' : break;		
    case '\'' : count++;
               DoBeginParagraph();   	
               while ((fread(&cThishilf,1,1,fTex) >= 1) && (cThishilf == '\''))
        	   count++;
               if (count != 2)
        	 { for (i = count-1; i >= 0; i--)
        	   fprintf(fRtf,"\\rquote ");
        	 }
               else
/*        	   fprintf(fRtf,"\\rdblquote ");  */
        	   fprintf(fRtf,"\\'94");
               fseek(fTex,-1L,SEEK_CUR); /* reread last character */
               count = 0;
               break;
    case '`' : count++;
               DoBeginParagraph();   	
               while ((fread(&cThishilf,1,1,fTex) >= 1) && (cThishilf == '`'))
        	   count++;
               if (count != 2)
        	 { for (i = count-1; i >= 0; i--)
        	   fprintf(fRtf,"\\lquote ");
        	 }
               else
/*        	   fprintf(fRtf,"\\ldblquote ");    */
        	   fprintf(fRtf,"\\'93");
               fseek(fTex,-1L,SEEK_CUR); /* reread last character */
               count = 0;
               break;
    case '\"':
               DoBeginParagraph();   	
               if (GermanMode)
        	 TranslateGerman();
               else
        	 fprintf(fRtf,"\"");
               break;
    case '\t': break;
             
    default:
             kchar = cThis;
             if (Code850 && (kchar >= 128))
             {
                if (Debug) fprintf (OUTPUTF,"\nkchar=%d, cThis='%c'",kchar,cThis);
                switch(cThis)
                {
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c9");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e9");
                              break;
                   case '' : DoBeginParagraph(); /* textcent */
        	              fprintf(fRtf, "\\'a2");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'b6");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'dd");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'fd");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e3");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c3");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f0");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d0");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f5");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d5");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'de");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'fe");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c2");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ca");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ce");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d4");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'db");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e2");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ea");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ee");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f4");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'fb");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c7");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e7");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c0");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c8");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d9");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e0");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e8");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f9");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c4");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'cb");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'cf");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d6");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'dc");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e4");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'eb");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ef");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f6");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'fc");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ff");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'df");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e5");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c5");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d8");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f8");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f2");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d2");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ec");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'cc");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e6");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c6");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'e1");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'c1");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'ed");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'cd");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f3");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d3");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'da");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'fa");
                              break;
                   case 245 : DoBeginParagraph(); /* \S  */
        	              fprintf(fRtf, "\\'a7");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'f1");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\'d1");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf, "\\ldblquote ");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf,"\\rdblquote ");
                              break;
                   case 166 : DoBeginParagraph(); /* ordfeminine  */
        	              fprintf(fRtf,"\\'aa");
                              break;
                   case 167 : DoBeginParagraph(); /* ordmasculine  */
        	              fprintf(fRtf,"\\'ba");
                              break;
                   case 248 : DoBeginParagraph(); /* textdegree  */
        	              fprintf(fRtf,"\\'b0");
                              break;
                   case 156 : DoBeginParagraph(); /* pound */
        	              fprintf(fRtf,"\\'a3");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf,"\\'bf");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf,"\\'a1");
                              break;
                   case '' : DoBeginParagraph();
        	              fprintf(fRtf,"\\'ac");
                              break;
                   case 169 : DoBeginParagraph(); /* textregistered */
        	              fprintf(fRtf,"\\'ae");
                              break;
                   case 184 : DoBeginParagraph(); /* textcopyright */
        	              fprintf(fRtf,"\\'a9");
                              break;
                   case 190 : DoBeginParagraph(); /* textyen */
        	              fprintf(fRtf,"\\'a5");
                              break;
                   case 207 : DoBeginParagraph(); /* textmonetary */
        	              fprintf(fRtf,"\\'ae");
                              break;
                   case 241 : DoBeginParagraph(); /* textpm */
        	              fprintf(fRtf,"\\'b1");
                              break;
                   case 246 : DoBeginParagraph(); /* textdivide */
        	              fprintf(fRtf,"\\'f7");
                              break;
                   case 251 : DoBeginParagraph(); /* textonesuperior  */
        	              fprintf(fRtf,"\\'b9");
                              break;
                   case 252 : DoBeginParagraph(); /* textthreesuperior  */
        	              fprintf(fRtf,"\\'b3");
                              break;
                   case 253 : DoBeginParagraph(); /* texttwosuperior  */
        	              fprintf(fRtf,"\\'b2");
                              break;
                   default: errorchar(cThis);           
                }
                break;
             } else if (CodeISO && (kchar >= 128))
             { if (kchar >= 160)
                { DoBeginParagraph();
                  fprintf(fRtf, "\\'%2x", cThis); 
                } else
                { errorchar(cThis);
                };
                break;
             } else if (kchar >= 128)
             {
               errorchar(cThis);           
             };
	
             if (bInDocument)
             {
               DoBeginParagraph();   	
/*              if ((isupper(cThis)) &&
        	 ((cLast=='.') || (cLast=='!') || (cLast=='?') || (cLast==':')))
                 fprintf(fRtf," ");  */
               if (isalpha(cThis) && MathMode[iEnvCount])
                {fprintf(fRtf,"{\\i %c}",cThis);}
               else
                {fprintf(fRtf,"%c",cThis);};
               bNewPar = FALSE;
               ParDone = FALSE;  /* DT */
             }
          break;
    }

  tabcounter++;
  cLast2 = cLast;
  cLast = cThis;
  if (cThis != ' ')
    cLastNoSpace = cThis;
  }
  RecursLevel--;
  return TRUE;
}


/****************************************************************************/
void errorchar(char text)
/****************************************************************************
purpose: writes character error message
 ****************************************************************************/
{
  char thebadchar[5] = "";
  char errmsg[128] = "";

  thebadchar[0] = text;
  thebadchar[1] = '\0';
  sprintf(errmsg,"Unsupported input character %s at line %d",thebadchar,linenumber);
  warning(errmsg);	
}


/****************************************************************************/
void Message(char *text)
/****************************************************************************
purpose: writes message on stdout
 ****************************************************************************/
{
  fprintf(fStd,"\n%s",text);  	
}

/****************************************************************************/
void OutputError(char *text)
/****************************************************************************
purpose: writes error or diagnostic message on both stderr and stdout
 ****************************************************************************/
{
  if (!StdToScreen)
  {
    fprintf(stderr,"\n%s",text);  	
  };
  fprintf(fStd,"\n%s",text);  	
}

/****************************************************************************/
void error(char * text)
/****************************************************************************
purpose: writes error message
globals: reads progname;
 ****************************************************************************/
{
  char errormessage[256] = "";
  sprintf(errormessage,"%s: ERROR: %s",progname,text);
  OutputError(errormessage);  	
  OutputError("Program aborted\n");
  exit(-1);
}
/****************************************************************************/
void Error(char * text)
/****************************************************************************
purpose: writes error message without progname
 ****************************************************************************/
{
  char errormessage[256] = "";
  sprintf(errormessage,"ERROR: %s",text);
  OutputError(errormessage);  	
  OutputError("Program aborted\n");
  exit(-1);
}
/****************************************************************************/
void warning(char * text)
/****************************************************************************
purpose: writes diagnostic message
 ****************************************************************************/
{
  char errormessage[256] = "";
  sprintf(errormessage,"WARNING: %s",text);
  OutputError(errormessage);  	
}

/****************************************************************************/
void numerror(int num)
/****************************************************************************
purpose: writes error message identified by number - for messages on many
         places in code. Calls function error.
globals: reads progname;
 ****************************************************************************/
{

  char text[1024];

  switch(num)
  {
    case ERR_EOF_INPUT:
      sprintf(text,"%s%s%s%ld%s","unexpected end of input file in: ",latexname," at linenumber: ",linenumber,"\n");
      error(text);
      break;
    case ERR_WRONG_COMMAND:
      sprintf(text,"%s%s%s%ld%s","unexpected command or character in: ",latexname," at linenumber: ",linenumber,"\n");
      error(text);
      break;
    case ERR_Param:
      error("wrong number of parameters\n");
      break;
    case ERR_WRONG_COMMAND_IN_TABBING:
      sprintf(text,"%s%s%s%ld%s","wrong command in Tabbing-kill-command-line in: ",latexname," at linenumber: ",linenumber,"\n");
      error(text);
      break;
    default :
      error("internal error");
      break;
  }
}


/****************************************************************************/
BOOL PrepareRtf(char *filename, FILE **f)  /* creates file */
/****************************************************************************
purpose: creates output file.
params: filename - name of outputfile
        f - pointer to filepointer to store file ID
 ****************************************************************************/
/* Note: fopen yields a pointer to a FILE */
{
  if(filename != NULL)
  {
      if ((*f = fopen(filename,"wb")) == NULL)	 /* open file */
      {
      	sprintf(ErrorMsgText,"Error opening RTF-file %s",filename);
        error(ErrorMsgText);
        exit(1);
      }
  };
  return TRUE;
 }

/****************************************************************************/
BOOL OpenMathTeX(char *filename, FILE **f)  /* creates file */
/****************************************************************************
purpose: creates output file for Maths TeX
params: filename - name of outputfile
        f - pointer to filepointer to store file ID
 ****************************************************************************/
{
  if(filename != NULL)
  {
      if ((*f = fopen(filename,"wb")) == NULL)	 /* open file */
      {
      	sprintf(ErrorMsgText,"Error opening Math-TeX-file %s",filename);
        error(ErrorMsgText);
        exit(1);
      }
  };
  return TRUE;
 }

/****************************************************************************/
BOOL PrepareStdout(char *filename, FILE **f)  /* creates file */
/****************************************************************************
purpose: creates standard output file.
params: filename - name of outputfile
        f - pointer to filepointer to store file ID
 ****************************************************************************/
{
  if(filename != NULL)
  {
      if ((*f = fopen(filename,"wb")) == NULL)	 /* open file */
      {
      	sprintf(ErrorMsgText,"Error opening Output file %s",filename);
        error(ErrorMsgText);
        exit(1);
      }
  };
  return TRUE;
 }

/****************************************************************************/
BOOL PrepareTex(char *filename, FILE **f)  /* opens file for reading */
/****************************************************************************
purpose: opens input file.
params: filename - name of inputfile
        f - pointer to filepointer to store file ID
 ****************************************************************************/
 
{ 
  if(filename != NULL)
  {
      if ((*f = fopen(filename,"rb")) == NULL)	 /* open file */
      { 
      	sprintf(ErrorMsgText,"Error opening LATEX-file %s",filename);
        error(ErrorMsgText);
/*	error("Error opening LATEX-file"); */
        exit(1);
      }
  }
#ifdef O_BINARY  
  else if (fTex != NULL)
  {
    setmode(fileno(fTex), O_BINARY);
  }
#endif  
  return TRUE;
}


/****************************************************************************/
BOOL CloseTex(FILE **f)
/****************************************************************************
purpose: closes input file.
params: f - pointer to filepointer to invalidate
 ****************************************************************************/
{
  if(*f != stdin)
      fclose(*f);
  *f = NULL;
  return TRUE;
}


/****************************************************************************/
BOOL CloseMath(FILE **f)
/****************************************************************************
purpose: closes output file.
params: f - pointer to filepointer to invalidate
 ****************************************************************************/
{
  fprintf(*f,"\n");
  if(*f != NULL)
  {
      if(fclose(*f) == EOF)
      {
        warning("error closing Math-TeX-File");
      }
  }
  *f = NULL;
  return TRUE;
}

/****************************************************************************/
BOOL CloseRtf(FILE **f)
/****************************************************************************
purpose: closes output file.
params: f - pointer to filepointer to invalidate
 ****************************************************************************/
{
/*  fprintf(*f,"}\n"); */
  if(*f != stdout)
  {
      if(fclose(*f) == EOF)
      {
        warning("error closing RTF-File");
      }
  }
  *f = NULL;
  return TRUE;
}


/****************************************************************************/
BOOL TranslateCommand()
/****************************************************************************
purpose: The function is called on a backslash in input file and
         tries to call the command-function for the following command.
returns: success or not
globals: fTex, fRtf, command-functions have side effects or recursive calls
 ****************************************************************************/
{
  char errormessage[256] = "";	
  char cCommand[MAXCOMMANDLEN] = {'\0'};
  int i;
  BOOL ParToMakeAfter = FALSE;
  
  char cThis;
  
  for (i = 0; ;i++)   /* get command from input stream */
  {
    if (fread(&cThis,1,1,fTex) < 1)
       break;
    if (IsEol(&cThis))  linenumber++;
    if (i == 0) /* test for special characters */
    {
      switch(cThis)
      {
        case '}': fprintf(fRtf,"\\}"); return TRUE;
        case '{': fprintf(fRtf,"\\{"); return TRUE;
        case '#': fprintf(fRtf,"#"); return TRUE;
        case '$': fprintf(fRtf,"$"); return TRUE;
        case '&': fprintf(fRtf,"&"); return TRUE;
        case '%': fprintf(fRtf,"%%"); return TRUE;
        case '_': fprintf(fRtf,"_"); return TRUE;
        case '\\':    /* produces a newline in output */
        	      /*	  no new paragraph    */
        	  if (fread(&cThis,1,1,fTex) < 1)
        	      cThis='a';

        	  if ((cThis == ' ') ||
        	      (cThis == '*'))  /* ignore * after \\ -> it's the same command as \\ */
        	    {
        	    if (fread(&cThis,1,1,fTex) < 1)
        	      cThis='a';
        	    while(cThis == ' ') 		   /*space  ignore */
        	       if (fread(&cThis,1,1,fTex) != 1)
        		  break;
        	    fseek(fTex,-1,SEEK_CUR);
        	    }
        	  else
        	    {
        	    fseek(fTex,-1,SEEK_CUR);
        	    }

        	  if (environ_type[iEnvCount] == TABULAR)
        	  {
        	    TabularDbleSlash();
        	  }else
        	  {
                    fprintf(fRtf,"\\par ");
                  }
        	  bNewPar = TRUE;
        	  tabcounter = 0;
        	  if (TABBING_ON)
        	     fgetpos(fRtf,&pos_begin_kill);

        	  return TRUE;
        case ' ': fprintf(fRtf," ");	/* ordinary interword space */
        	  while (cThis == ' ')	 /* all spaces after commands are ignored */
        	  {
        	     if (fread(&cThis,1,1,fTex) < 1)
        		numerror(ERR_EOF_INPUT);
        	  }

        	  fseek(fTex,-1L,SEEK_CUR);
        	  return TRUE;
        	  break;
        case '-': /* hyphen */
        	  /* caution: this character has another effect in the tabbing-environment */
        	  if (TABBING_ON);
        	  else
        	     fprintf(fRtf,"\\-");
        	  return TRUE;
        	  break;
        case '+': /* see tabbing-environment */
        	  /* no harm for RTF-output */
        	  break;
        case '<': /* see tabbing-environment */
        	  /* no harm for RTF-output */
        	  break;
        case '~': CmdTildeChar(0);
        	  return TRUE;
        case '^': CmdSpitzeChar(0);
        	  return TRUE;
        case '\"':CmdUmlaute(0);
        	  return TRUE;
        case '`': if (TABBING_ON);
        	  else
        	    CmdLApostrophChar(0);
        	  return TRUE;
        case '\'':if (TABBING_ON);
        	  else
        	     CmdRApostrophChar(0);  /* char ' =?= \' */
        	  return TRUE;
        case '(': CmdFormula(FORM_RND_OPEN);
        	  return TRUE;
        case '[': CmdFormula(FORM_ECK_OPEN);
        	  return TRUE;
        case ')': CmdFormula(FORM_RND_CLOSE);
        	  return TRUE;
        case ']': CmdFormula(FORM_ECK_CLOSE);
        	  return TRUE;
        case '/': CmdIgnore(0);
        	  return TRUE;
        case ',': CmdSpace(0.33);  /* \, produces a thin space */
        	  return TRUE;
        case '>': CmdSpace(0.50);  /* \, produces a med space */
        	  return TRUE;
        case ';': CmdSpace(0.75);  /* \, produces a thick space */
        	  return TRUE;
        case '@': CmdIgnore(0);  /* \@ produces an "end of sentence" space */
        	  return TRUE;
        case '=': if ( TABBING_ON )
        	  {CmdTabset(0);}
        	  else
                  {CmdMacronChar(0);};
                  
        	  return TRUE;
        case '3': fprintf(fRtf, "\\'df");   /* german symbol '' */
        	  return TRUE;
      }
    }

    if (!isalpha(cThis))
    {
      int found_nl = FALSE;
      /* all spaces after commands are ignored, a single \n may occur */
      while (cThis == ' ' || cThis == '\t' || (cThis == '\n' && (!found_nl)))
        {
        if(IsEol(&cThis))
            found_nl = TRUE;
        if (fread(&cThis,1,1,fTex) < 1)
            break;
        }
      if (IsEol(&cThis) && found_nl)
      {
        ParToMakeAfter = TRUE;
      };

      fseek(fTex,-1L,SEEK_CUR); /* position of next character after command
        			   except space */
      break;
    };
    cCommand[i] = cThis;
  }

  cCommand[i] = '\0';  /* mark end of string with zero */

  if (Debug) fprintf(OUTPUTF,"\nTrans [%d]: %s ",linenumber,cCommand);

  if ( i == 0)
    return FALSE;
  if (CallCommandFunc(cCommand)==TRUE) /*call handling function for command*/
    { if (ParToMakeAfter) CmdPar(0); return TRUE;}
  else if (TryDirectConvert(cCommand, fRtf) == TRUE)
    { if (ParToMakeAfter) CmdPar(0); return TRUE;}
  else if (TryVariableIgnore(cCommand, fTex) == TRUE)
    { if (ParToMakeAfter) CmdPar(0); return TRUE;}
  else
    {
      sprintf(errormessage,"command: %s at line %d not found - ignored",cCommand,linenumber);
      warning(errormessage);
      if (ParToMakeAfter) CmdPar(0);
      return FALSE;
    };
  if (ParToMakeAfter) CmdPar(0);
  return TRUE;
}

/****************************************************************************/
void IgnoreTo(char cEnd)
/****************************************************************************
purpose: ignores anything from inputfile till character cEnd
params:  character to stop ignoring
globals: changes inputfile fTex
 ****************************************************************************/
{
  char c;
  while (fread(&c, 1,1,fTex) >= 1)
  {
    if (c == cEnd || (cEnd == '\n' && c == '\r'))
    {
      if (IsEol(&cEnd))
        fseek(fTex,-1L,SEEK_CUR);   /*linenumber is set in convert-routine */
        /*linenumber++;*/
      return;
    }
    else
    {
     if (IsEol(&c))
       { linenumber++; InputLineEmpty = TRUE;
       }
    }
  }
  fprintf(stderr, "IgnoreTo %d\n", cEnd);
  numerror(ERR_EOF_INPUT);
}
/****************************************************************************/
FILE *open_cfg(const char *name)
/****************************************************************************
purpose: open the given *.cfg file looking at various possible paths
returns file pointer
 ****************************************************************************/
{
    char errormessage[256] = "";	
    FILE *fp;
#ifdef KPATHSEA
    if ( Debug) fprintf(OUTPUTF,"\n(kpse) trying to open:%s|",name);
    if ((fp = kpse_open_file(name, kpse_program_text_format)) != NULL) return(fp);
    sprintf(errormessage, "Cannot open file '%s'.",name);
    fclose(fp);
    error(errormessage);
#else
    char *cfg_path = getenv("RTFPATH");
    char path[PATHMAX] = ""; char *ttt=NULL;

    if (Debug) fprintf(OUTPUTF,"\n%s: Libdir=%s|", progname, LIBDIR );

/* trying to open in current directory */
    if (Debug) fprintf(OUTPUTF,"\ntrying to open:%s|",name);
    if((fp = fopen(name,"r")) != NULL)	return(fp);

/* trying to open in RTFPATH directory */
    if(cfg_path != NULL)
    {
    	sprintf(path,"%s/%s",cfg_path,name);
        if ( Debug) fprintf(OUTPUTF,"\ntrying to open:%s|",path);
        if((fp = fopen(path,"r")) != NULL)	return(fp);
    };

/* trying to open in the directory where ltx2rtf has been found */
    
    sprintf(path,"%s/%s",exe_path,name);
    if ( Debug) fprintf(OUTPUTF,"\ntrying to open:%s|",path);
    if((fp = fopen(path,"r")) != NULL)    return(fp);

/* trying to open in LIBDIR directory */
    sprintf(path,"%s/%s",LIBDIR,name);
    if ( Debug) fprintf(OUTPUTF,"\ntrying to open:%s|",path);
    if((fp = fopen(path,"r")) != NULL)    return(fp);

    sprintf(errormessage, "Cannot open file '%s'.",name);
    fclose(fp);
    error(errormessage);
#endif /* KPATHSEA */
}
/******************************************************************************/
BOOL GetConfigs()
/******************************************************************************
  purpose: reads from the file config.cfg where programs can be found
           to latex an equation TeX file inside the TMPDIR directory
           to convert a DVI file inside the TMPDIR directory to a *.PGM file
           stores them into global char[..] variables.
 ******************************************************************************/
{
  FILE *fp;
  char buffer[512];
  int i; int k;
  char *buffpoint;
  char *ttt = NULL;
  char *qqq = NULL;
  char command[512] = "";
  char label[512]="";

#ifdef KPATHSEA
  string ltx2rtfdir = 
    xdirname(kpse_find_file("config.cfg", kpse_program_text_format, true));
  char *p;

  if (ltx2rtfdir == NULL) {
    error("No config files found.");
    return FALSE;
  }

  for (p = ltx2rtfdir; p && *p; p++)
    *p = (*p == '\\' ? '/' : *p);   /* keep dir separators as slashes by default */
#endif
  fp = open_cfg("config.cfg");
  for(;;)
  {
    if ( (fgets( buffer, 512, fp )) == NULL)
    {
      fclose(fp);
      return FALSE;
    }
    if (strlen(buffer) > 256)
    {
      fclose(fp);
      error("line too long in CONFIG.CFG - only 256 characters.");
      return FALSE;
    }
    buffpoint = buffer;
    i = strlen(buffer);
    while((buffer[i-1] <= ' ') && (i > 0)) {buffer[i-1] = '\0'; i--;};
    if(buffer[0] != '#')
    { ttt = strchr(buffer,'=');
      if (ttt == NULL)
      { sprintf(ErrorMsgText,"in config.cfg, line '%s' has no '=', ignored",buffer);
        if (buffer[0] != '\0')  warning(ErrorMsgText);
      }
      else
      { /* fprintf(OUTPUTF,"\nbuffer=|%s|",buffer); */
        /* fprintf(OUTPUTF,"\nttt   =|%s|",ttt); */
        ttt++;
        /* fprintf(OUTPUTF,"\nttt   =|%s|",ttt); */
        sprintf(command,"%s",ttt);
        ttt--;
        ttt[0]='\0';
        sprintf(label,"%s",buffer);
        if (Debug) fprintf(OUTPUTF,"\nlabel   =|%s|",label);
        buffpoint = command;
        i = strlen(command);
        while((command[i-1] <= ' ') && (i > 0)) {command[i-1] = '\0'; i--;};
        if (Debug) fprintf(OUTPUTF,"\ncommand =|%s|",command);

#ifdef KPATHSEA
	/* The command is probably located in some specific
	 directory referenced by kpathsea variables. Find the command. */
	if (!kpse_absolute_p(command, TRUE)) {
	  /* if the command is found on the ltx2rtf specific area, use it */
	  char *p = kpse_find_file(command, kpse_program_text_format, true);
	  if (p) {
	    strcpy(command, p);
	    free(p);
	  }
	  /* else it will be searched along the PATH */
	}
#endif

        if (strcmp(label,"latextmp") == 0)
/*        Never restrict hardly the flexibility of config.cfg  */
        { sprintf(LaTeXingCommand,"%s", command);}
        else if (strcmp(label,"dvi2pbm") == 0)
/*        Never restrict hardly the flexibility of config.cfg  */
        { sprintf(PBMingCommand,"%s",command);}
        else if (strcmp(label,"tmpdir") == 0)
        { 
	  /* Do not overwrite the environment defined TempDir */
	  if (!TempDir || !*TempDir) {
	    sprintf(TempDir,"%s",command);
	    if (Debug) fprintf(OUTPUTF,"\n config.cfg => TempDir = %s", TempDir);
	  }
        }
        else
        { sprintf(ErrorMsgText,"config.cfg: specification %s not understood, ignored",label);
          warning(ErrorMsgText);
        };
      };
    };
  }  /* end while */

#if defined(MSDOS) || defined(DOSISH)
  SlashToBack(PBMingCommand);
  SlashToBack(LaTeXingCommand);
#endif  

      fclose(fp);
      return TRUE;
}

/******************************************************************************/
void CheckNospace(char *DirectoryName)
/******************************************************************************
checks that the TempDir given toes not contain spaces in its name
 ******************************************************************************/
{ int k;
  int DirLen=strlen(DirectoryName);

  if (Debug) fprintf(OUTPUTF,"\nChecking '%s' directory name",DirectoryName);
  for (k=0; k<DirLen; k++)
  {  if(DirectoryName[k] <= ' ')
     { sprintf(ErrorMsgText,"Temp. dir. name '%s' contains spaces or control chars",DirectoryName);
       error(ErrorMsgText); exit(1);
     }
  }
}	
/******************************************************************************/
BOOL SaveInitialDir()
/******************************************************************************
get the current directory position, in order to be able to come back!
 ******************************************************************************/
{
#if defined (HAVE_GETCWD) && !defined (GETCWD_FORKS)
  char *path = malloc (PATHMAX + 1);

  if (getcwd (path, PATHMAX + 1) == 0)
    {
      fprintf (stderr, "getcwd: %s", path);
      exit (1);
    }
  sprintf(InitialDir,"%s",path);

#elif defined (HAVE_GETWD)
  char *path = malloc (PATHMAX + 1);

  if (getwd (path) == 0)
    {
      fprintf (stderr, "getwd: %s", path);
      exit (1);
    }
  sprintf(InitialDir,"%s",path);
#else
    
  int zz;	
#ifdef MSDOS  
  system ("cd > cd.log");
#else
  system ("pwd > cd.log");
#endif
  fUtil = fopen("cd.log","r");
  fgets(InitialDir,126,fUtil);
  zz = strlen(InitialDir);
  while((InitialDir[zz-1] <= ' ') && (zz > 0)) {InitialDir[zz-1] = '\0'; zz--;};
#endif  

  fprintf(OUTPUTF,"\nInitial directory is %s",InitialDir);

 }
/******************************************************************************/
BOOL GetTempDir()
/******************************************************************************
get the temporaty directory, if not possible use initial directory
Musc be called after setting the InitialDir, of course!
 ******************************************************************************/
{
#ifdef WIN32
  if (GetTempPath(sizeof(TempDir), TempDir) != 0) {
	char buf[PATHMAX];
	win32_get_long_filename(TempDir, buf, sizeof(buf));
	dostounix_filename(buf);
	strcpy(TempDir, buf);
	fprintf(stderr, "GetTempPath = %s\n", TempDir);
  }
  else 
#endif
    if (getenv("TMPDIR") != NULL) {
      strcpy(TempDir, getenv("TMPDIR"));
  }
  else if (getenv("TMP") != NULL) {
    strcpy(TempDir, getenv("TMP"));
  }
  else if (getenv("TEMP") != NULL) {
    strcpy(TempDir, getenv("TEMP"));
  }
  else
  {
    fprintf(stderr, 
    "Unable to find temp. dir, will work in curr. dir: %s\n",InitialDir);
    strcpy(TempDir,InitialDir);
  }
  if (Debug)
  { sprintf(ErrorMsgText,"Temporary directory TempDir is %s\n",TempDir);
     Message(ErrorMsgText);
  }
  return TRUE;
 }
/******************************************************************************/
BOOL RestoreInitialDir()
/******************************************************************************
restores to the current directory position, needed by MSDOS!
 ******************************************************************************/
{
  if (chdir(InitialDir) != 0)
  {
    perror(InitialDir);
    return FALSE;
  }
  else
  {
  if (Debug) fprintf(OUTPUTF, "\nChanging cwd to %s\n", InitialDir);
    return TRUE;
  }
 }

/******************************************************************************/
BOOL SetTempDir()
/******************************************************************************
moves the the current directory position to TempDir
 ******************************************************************************/
{
  if (chdir(TempDir) != 0)
  {
    perror(TempDir);
    return FALSE;
  }
  else
  {
  if (Debug) fprintf(OUTPUTF, "\nChanging cwd to %s\n", TempDir);
    return TRUE;
  }
 }

/******************************************************************************/
int BackToSlash(char *Name)
/******************************************************************************
Changes all backslashes to slashes in given string
 ******************************************************************************/
{
  int z; int num_changes = 0;
  int m = strlen(Name);

  for (z=0; z<strlen(Name); z++)
  {
    if(Name[z] == '\\')
    { Name[z] = '/'; num_changes++;
    }
  }
  return num_changes;
}

/******************************************************************************/
int SlashToBack(char *Name)
/******************************************************************************
Changes all slashes to backslashes in given string
 ******************************************************************************/
{
  int z; int num_changes = 0;
  int m = strlen(Name);

  for (z=0; z<strlen(Name); z++)
  {
    if(Name[z] == '/')
    { Name[z] = '\\'; num_changes++;
    }
  }
  return num_changes;
}

/****************************************************************************/
void Usage()
/****************************************************************************
purpose: displays a summary of usage options
 ****************************************************************************/
{
    sprintf(ErrorMsgText,"Usage: %s [options] inputfile[.tex]",
      progname);
    OutputError(ErrorMsgText);
    OutputError("Options:");
    OutputError(" -o outputfile : name of output RTF file (default=inputfile with .rtf suffix)");
    OutputError(" -D            : debug option (verbose)");
    OutputError(" -C 850|ISO    : 8 bit input code (ISO or 850)");
    OutputError(" -m            : invoke LaTeX to generate displayed equations");
    OutputError(" -M            : invoke LaTeX to generate all maths");
    OutputError(" -h            : display this help");
    OutputError(" -Z            : do not add safety }}}}} at end of RTF");
    OutputError(" LaTeX Working directory defined by environment vars. TMPDIR, TMP or TEMP");
    OutputError(" Config files (*.cfg) sought in directories:");
#ifdef KPATHSEA
    sprintf(ErrorMsgText,"\t%s\n", kpse_path_expand("$TEXMF/ltx2rtf//"));
#else
    sprintf(ErrorMsgText,"   ./   env.('RTFPATH')  %s  %s",
      exe_path, LIBDIR);
#endif
    OutputError(ErrorMsgText); OutputError("\r");

}
