###############################################################################
# updmap: utility to maintain map files for outline fonts.
#
# Thomas Esser, (C) 2002. Public domain.
# Fabrice Popineau, for the Perl version.
#
# Commands:
#  --edit                     edit config file
#  --help                     show help message
#
# Options:
#  --cnffile file             specify configuration file
#  --outputdir directory      specify output directory
#  --quiet                    reduce verbosity
###############################################################################

require "newgetopt.pl";
use Cwd;
use Strict;
use File::Copy;

#  my $IsWin32 = ($^O =~ /MSWin32/i);

#  if ($IsWin32) {
#    use Win32::Registry;
#    use Win32::API;
#  }

my $cnf_file;
my $outputdir;
my $quiet;
my $texhashEnabled;

my $mode;
my $dvipsPreferOutline;

my @cfg = ( );

my @psADOBE = (
		    's/ URWGothicL-Demi / AvantGarde-Demi /x',
		    's/ URWGothicL-DemiObli / AvantGarde-DemiOblique /x',
		    's/ URWGothicL-Book / AvantGarde-Book /x',
		    's/ URWGothicL-BookObli / AvantGarde-BookOblique /x',
		    's/ URWBookmanL-DemiBold / Bookman-Demi /x',
		    's/ URWBookmanL-DemiBoldItal / Bookman-DemiItalic /x',
		    's/ URWBookmanL-Ligh / Bookman-Light /x',
		    's/ URWBookmanL-LighItal / Bookman-LightItalic /x',
		    's/ NimbusMonL-Bold / Courier-Bold /x',
		    's/ NimbusMonL-BoldObli / Courier-BoldOblique /x',
		    's/ NimbusMonL-Regu / Courier /x',
		    's/ NimbusMonL-ReguObli / Courier-Oblique /x',
		    's/ NimbusSanL-Bold / Helvetica-Bold /x',
		    's/ NimbusSanL-BoldCond / Helvetica-Condensed-Bold /x',
		    's/ NimbusSanL-BoldItal / Helvetica-BoldOblique /x',
		    's/ NimbusSanL-BoldCondItal / Helvetica-Condensed-BoldObl /x',
		    's/ NimbusSanL-Regu / Helvetica /x',
		    's/ NimbusSanL-ReguCond / Helvetica-Condensed /x',
		    's/ NimbusSanL-ReguItal / Helvetica-Oblique /x',
		    's/ NimbusSanL-ReguCondItal / Helvetica-Condensed-Oblique /x',
		    's/ CenturySchL-Bold / NewCenturySchlbk-Bold /x',
		    's/ CenturySchL-BoldItal / NewCenturySchlbk-BoldItalic /x',
		    's/ CenturySchL-Roma / NewCenturySchlbk-Roman /x',
		    's/ CenturySchL-Ital / NewCenturySchlbk-Italic /x',
		    's/ URWPalladioL-Bold / Palatino-Bold /x',
		    's/ URWPalladioL-BoldItal / Palatino-BoldItalic /x',
		    's/ URWPalladioL-Roma / Palatino-Roman /x',
		    's/ URWPalladioL-Ital / Palatino-Italic /x',
		    's/ StandardSymL / Symbol /x',
		    's/ NimbusRomNo9L-Medi / Times-Bold /x',
		    's/ NimbusRomNo9L-MediItal / Times-BoldItalic /x',
		    's/ NimbusRomNo9L-Regu / Times-Roman /x',
		    's/ NimbusRomNo9L-ReguItal / Times-Italic /x',
		    's/ URWChanceryL-MediItal / ZapfChancery-MediumItalic /x',
		    's/ Dingbats / ZapfDingbats /x'
		   );

my @fileADOBEkb = (
			's/\buagd8a.pfb\b/pagd8a.pfb/x',
			's/\buagdo8a.pfb\b/pagdo8a.pfb/x',
			's/\buagk8a.pfb\b/pagk8a.pfb/x',
			's/\buagko8a.pfb\b/pagko8a.pfb/x',
			's/\bubkd8a.pfb\b/pbkd8a.pfb/x',
			's/\bubkdi8a.pfb\b/pbkdi8a.pfb/x',
			's/\bubkl8a.pfb\b/pbkl8a.pfb/x',
			's/\bubkli8a.pfb\b/pbkli8a.pfb/x',
			's/\bucrb8a.pfb\b/pcrb8a.pfb/x',
			's/\bucrbo8a.pfb\b/pcrbo8a.pfb/x',
			's/\bucrr8a.pfb\b/pcrr8a.pfb/x',
			's/\bucrro8a.pfb\b/pcrro8a.pfb/x',
			's/\buhvb8a.pfb\b/phvb8a.pfb/x',
			's/\buhvb8ac.pfb\b/phvb8ac.pfb/x',
			's/\buhvbo8a.pfb\b/phvbo8a.pfb/x',
			's/\buhvbo8ac.pfb\b/phvbo8ac.pfb/x',
			's/\buhvr8a.pfb\b/phvr8a.pfb/x',
			's/\buhvr8ac.pfb\b/phvr8ac.pfb/x',
			's/\buhvro8a.pfb\b/phvro8a.pfb/x',
			's/\buhvro8ac.pfb\b/phvro8ac.pfb/x',
			's/\buncb8a.pfb\b/pncb8a.pfb/x',
			's/\buncbi8a.pfb\b/pncbi8a.pfb/x',
			's/\buncr8a.pfb\b/pncr8a.pfb/x',
			's/\buncri8a.pfb\b/pncri8a.pfb/x',
			's/\buplb8a.pfb\b/pplb8a.pfb/x',
			's/\buplbi8a.pfb\b/pplbi8a.pfb/x',
			's/\buplr8a.pfb\b/pplr8a.pfb/x',
			's/\buplri8a.pfb\b/pplri8a.pfb/x',
			's/\busyr.pfb\b/psyr.pfb/x',
			's/\butmb8a.pfb\b/ptmb8a.pfb/x',
			's/\butmbi8a.pfb\b/ptmbi8a.pfb/x',
			's/\butmr8a.pfb\b/ptmr8a.pfb/x',
			's/\butmri8a.pfb\b/ptmri8a.pfb/x',
			's/\buzcmi8a.pfb\b/pzcmi8a.pfb/x',
			's/\buzdr.pfb\b/pzdr.pfb/x'
		       );

my @fileURW = (
		    's/\buagd8a.pfb\b/a010015l.pfb/x',
		    's/\buagdo8a.pfb\b/a010035l.pfb/x',
		    's/\buagk8a.pfb\b/a010013l.pfb/x',
		    's/\buagko8a.pfb\b/a010033l.pfb/x',
		    's/\bubkd8a.pfb\b/b018015l.pfb/x',
		    's/\bubkdi8a.pfb\b/b018035l.pfb/x',
		    's/\bubkl8a.pfb\b/b018012l.pfb/x',
		    's/\bubkli8a.pfb\b/b018032l.pfb/x',
		    's/\bucrb8a.pfb\b/n022004l.pfb/x',
		    's/\bucrbo8a.pfb\b/n022024l.pfb/x',
		    's/\bucrr8a.pfb\b/n022003l.pfb/x',
		    's/\bucrro8a.pfb\b/n022023l.pfb/x',
		    's/\buhvb8a.pfb\b/n019004l.pfb/x',
		    's/\buhvb8ac.pfb\b/n019044l.pfb/x',
		    's/\buhvbo8a.pfb\b/n019024l.pfb/x',
		    's/\buhvbo8ac.pfb\b/n019064l.pfb/x',
		    's/\buhvr8a.pfb\b/n019003l.pfb/x',
		    's/\buhvr8ac.pfb\b/n019043l.pfb/x',
		    's/\buhvro8a.pfb\b/n019023l.pfb/x',
		    's/\buhvro8ac.pfb\b/n019063l.pfb/x',
		    's/\buncb8a.pfb\b/c059016l.pfb/x',
		    's/\buncbi8a.pfb\b/c059036l.pfb/x',
		    's/\buncr8a.pfb\b/c059013l.pfb/x',
		    's/\buncri8a.pfb\b/c059033l.pfb/x',
		    's/\buplb8a.pfb\b/p052004l.pfb/x',
		    's/\buplbi8a.pfb\b/p052024l.pfb/x',
		    's/\buplr8a.pfb\b/p052003l.pfb/x',
		    's/\buplri8a.pfb\b/p052023l.pfb/x',
		    's/\busyr.pfb\b/s050000l.pfb/x',
		    's/\butmb8a.pfb\b/n021004l.pfb/x',
		    's/\butmbi8a.pfb\b/n021024l.pfb/x',
		    's/\butmr8a.pfb\b/n021003l.pfb/x',
		    's/\butmri8a.pfb\b/n021023l.pfb/x',
		    's/\buzcmi8a.pfb\b/z003034l.pfb/x',
		    's/\buzdr.pfb\b/d050000l.pfb/x'
		   );

my @fileADOBE = (
		 's/\buagd8a.pfb\b/agd_____.pfb/x',
		 's/\buagdo8a.pfb\b/agdo____.pfb/x',
		 's/\buagk8a.pfb\b/agw_____.pfb/x',
		 's/\buagko8a.pfb\b/agwo____.pfb/x',
		 's/\bubkd8a.pfb\b/bkd_____.pfb/x',
		 's/\bubkdi8a.pfb\b/bkdi____.pfb/x',
		 's/\bubkl8a.pfb\b/bkl_____.pfb/x',
		 's/\bubkli8a.pfb\b/bkli____.pfb/x',
		 's/\bucrb8a.pfb\b/cob_____.pfb/x',
		 's/\bucrbo8a.pfb\b/cobo____.pfb/x',
		 's/\bucrr8a.pfb\b/com_____.pfb/x',
		 's/\bucrro8a.pfb\b/coo_____.pfb/x',
		 's/\buhvb8a.pfb\b/hvb_____.pfb/x',
		 's/\buhvb8ac.pfb\b/hvcb____.pfb/x',
		 's/\buhvbo8a.pfb\b/hvbo____.pfb/x',
		 's/\buhvbo8ac.pfb\b/hvcbo___.pfb/x',
		 's/\buhvr8a.pfb\b/hv______.pfb/x',
		 's/\buhvr8ac.pfb\b/hvc_____.pfb/x',
		 's/\buhvro8a.pfb\b/hvo_____.pfb/x',
		 's/\buhvro8ac.pfb\b/hvcdo___.pfb/x',
		 's/\buncb8a.pfb\b/ncb_____.pfb/x',
		 's/\buncbi8a.pfb\b/ncbi____.pfb/x',
		 's/\buncr8a.pfb\b/ncr_____.pfb/x',
		 's/\buncri8a.pfb\b/nci_____.pfb/x',
		 's/\buplb8a.pfb\b/pob_____.pfb/x',
		 's/\buplbi8a.pfb\b/pobi____.pfb/x',
		 's/\buplr8a.pfb\b/por_____.pfb/x',
		 's/\buplri8a.pfb\b/poi_____.pfb/x',
		 's/\busyr.pfb\b/sy______.pfb/x',
		 's/\butmb8a.pfb\b/tib_____.pfb/x',
		 's/\butmbi8a.pfb\b/tibi____.pfb/x',
		 's/\butmr8a.pfb\b/tir_____.pfb/x',
		 's/\butmri8a.pfb\b/tii_____.pfb/x',
		 's/\buzcmi8a.pfb\b/zcmi____.pfb/x',
		 's/\buzdr.pfb\b/zd______.pfb/x'
		);

&main;

exit 0;

#  if ($IsWin32) {

#    sub GetTempPath {
#      my $GetTempPath = new Win32::API('kernel32', 'GetTempPath', 'NP', 'N');
#      if(not defined $GetTempPath) {
#        die "Can't import API GetTempPath: $!\n";
#      }
#      my $lpBuffer = " " x 260;

#      $GetTempPath->Call(80, $lpBuffer);
#    }

#    sub ShellExecute {
#      my ($command, $file) = @_;
#      my $ShellExecute = new Win32::API('shell32', 'ShellExecute', 'NPPPPN', 'N');
#      if(not defined $ShellExecute) {
#        die "Can't import API ShellExecute: $!\n";
#      }
#      $ShellExecute->Call(0, $command, $file, '', '', 0);
#    }

#    sub GetHomeDir {
#      my $home = $ENV{'HOME'};
#      # Should check for CSIDL_...
#      return $home;
#    }
#  }

###############################################################################
# getLines()
#   return the lines in $filename
###############################################################################
sub getLines {
  my @lines;
  for my $fname (@_) {
    open FILE, "<$fname" or die "can't get lines from $fname: $!";
    push @lines, <FILE>;
    close FILE;
  }
  return @lines;
}

###############################################################################
# writeLines()
#   write the lines in $filename
###############################################################################
sub writeLines {
  my ($fname, @lines) = @_;
  open FILE, ">$fname" or die "can't write lines to $fname: $!";
  print FILE @lines;
  close FILE;
}

###############################################################################
# copyFile()
#   copy file $src to $dst, sets $dst creation and mod time
###############################################################################
sub copyFile {
  my ($src, $dst) = @_;
  open IN, "<$src" or die "can't open source file $src for copying: $!";
  open OUT, ">$dst";

  binmode(IN);
  binmode(OUT);
  print OUT <IN>;
  close(OUT);
  close(IN);
  my @t = stat($src);
  utime($t[8], $t[9], $dst);
}

###############################################################################
# help()
#   display help message and exit
###############################################################################
sub help {
  print "Usage: $0 [option] ... [command]\n";
  print "\n";
  print "Valid options:\n";
  print "  --cnffile file             specify configuration file\n";
  print "  --outputdir directory      specify output directory\n";
  print "  --quiet                    reduce verbosity\n";
  print "\n";
  print "Valid commands:\n";
  print "  --edit                     edit updmap.cfg file\n";
  print "  --help                     show this message\n";
  exit 0;
}

###############################################################################
# abort(errmsg)
#   print `errmsg' to stderr and exit with error code 1
###############################################################################
sub abort {
  local ($msg) = @_;
  print STDERR "$progname: $msg\n";
  exit 1;
}

###############################################################################
# cfgval(variable)
#   read variable ($1) from config file
###############################################################################
sub cfgval {
  my ($variable) = @_;
  my $value;

  if ($#cfg < 0) {
    open FILE, "<$cnf_file" or die "can't open configuration file $cnf_file: $!";
    @cfg = <FILE>;
    close FILE;
  }
  for $line (@cfg) {
    if ($line =~ m/^\s*${variable}[\s=]+(.*)\s*$/) {
      $value = $1;
      break;
    }
  }
  return $value;
}

###############################################################################
# setupDvipsLink()
#   set symlink for psfonts.map according to dvipsPreferOutline variable
###############################################################################
sub setupDvipsLink {
  my $src;
  if ($dvipsPreferOutline) {
    $src = "psfonts_t1.map";
  } else {
    $src = "psfonts_pk.map";
  }
  unlink "$outputdir/psfonts.map";
  &copyFile("$outputdir/$src", "$outputdir/psfonts.map");
}

###############################################################################
# transLW35(args ...)
#   transform fontname and filenames according to transformation specified
#   by mode
###############################################################################
sub transLW35 {
  my ($name) = @_;
  my @maplist = &getLines($name);

  if ($mode eq "" || $mode eq "URWkb") {
    # do nothing
  } elsif ($mode eq "URW") {
    for my $r (@fileURW) {
      @maplist = map { my $l = $_; eval("\$l =~ $r"); $_ = ($l eq "" ? $_ : $l); } @maplist;
    }
  } elsif ($mode eq "ADOBE" || $mode eq "ADOBEkb") {
    for my $r (@psADOBE) {
      @maplist = map { my $l = $_; eval("\$l =~ $r"); $_ = ($l eq "" ? $_ : $l); } @maplist;
    }
    my @filemode = eval ("\@file" .$mode);
    for my $r (@filemode) {
      @maplist = map { my $l = $_; eval("\$l =~ $r"); $_ = ($l eq "" ? $_ : $l); } @maplist;
    }
  }
  return @maplist;
}

###############################################################################
# locateWeb2c (file ...)
#   apply kpsewhich with format 'web2c files'
###############################################################################
sub locateWeb2c {

  my @files = @_;

  return @files if ($#files < 0);

  @files = split (/\n/, `kpsewhich --format="web2c files" @files`);

  if (wantarray) {
    return @files;
  }
  else {
    return $files[0];
  }
}

###############################################################################
# locateDvipsCfg (file ...)
#   apply kpsewhich with format 'dvips config'
###############################################################################
sub locateDvipsCfg {
  my @files = @_;

  return @files if ($#files < 0);

  @files = `kpsewhich --format="dvips config" @files`;
  if (wantarray) {
    return @files;
  }
  else {
    return $files[0];
  }
}

###############################################################################
# locateMaps()
#   apply filter (regex $1) to config file to extract filenames of map files
#   run list through uniq, finally apply kpsewhich.
###############################################################################
sub locateMaps {
  my ($regex) = @_;
  my $regex1 = $regex . "\\s+([^\\s]*)\\s*\$";
  my $n = $regex1;
  $n =~ s@[^\(]@@g;
  $n = length $n;
  my %count = ( );
  my @maps = @cfg;
  my @maps = grep { $_ !~ m/^\#/
		    and $_ =~ m/$regex/
		    and $_ =~ s@$regex1@\1@
		    and $_ = ${$n}
                    and ++$count{$_} < 2; } @maps;
  @maps = &locateDvipsCfg(@maps);
  return @maps;
}

###############################################################################
# initVars()
#   initialize global variables
###############################################################################
sub initVars {
  $progname="updmap";

  $quiet = 0;
  $texhashEnabled = 1;

  $cnf_file = "";
  $outputdir = "";
  $pdftex35 =  &locateDvipsCfg("pdftex35.map");
  $ps2pk35 = &locateDvipsCfg("ps2pk35.map");
  $dvips35 = &locateDvipsCfg("dvips35.map");
  $TEXMFMAIN =`kpsewhich --expand-var="$TEXMFMAIN"`;
  chomp($TEXMFMAIN);

  #    tmpdir=${TMP-/tmp}/$progname.$$
  #    tmp1=$tmpdir/a
  #    tmp2=$tmpdir/b
  #    tmp3=$tmpdir/c
}

###############################################################################
# setupTmpDir()
#   set up a temp directory and a trap to remove it
###############################################################################
sub setupTmpDir {

}

###############################################################################
# setupDestDir()
#   find an output directory if none specified on cmd line. First choice is
#   $VARTEXMF/dvips/config (if VARTEXMF is set), next is relative to config
#   file location. Fallback is $TEXMFMAIN/dvips/config.
###############################################################################
sub setupDestDir {

  if (! $outputdir) {
    my $tf = `kpsewhich -expand-var="$VARTEXMF"`;
    chomp($tf);
    if ($tf eq "") {
      $tf = $cnf_file;
      $tf =~ s@/web2c/[^/]*$@@;
    }
    $tf = $TEXMFMAIN if ($tf eq "");
    $outputdir = "$tf/dvips/config";
    system("$TEXMFMAIN/web2c/mktexdir \"$outputdir\"");
  }
  
  &abort("output directory `$outputdir' does not exist") if (! -d "$outputdir");

}

###############################################################################
# setupCfgFile()
#   find config file if none specified on cmd line.
###############################################################################
sub setupCfgFile {

  if ($cnf_file eq "") {
    $cnf_file = &locateWeb2c("updmap.cfg");
    &abort("config file updmap.cfg not found") if (! -f "$cnf_file");
  }
}

###############################################################################
# processOptions()
#   process cmd line options
###############################################################################
sub processOptions {

  unless (&NGetOpt ("quiet", "cnffile=s", "outputdir=s",
		    "edit", "nohash", "verbose", "debug", "help")) {
    print STDERR "Try `$0 --help'";
    exit 1;
  }

  $quiet = 1 if ($opt_quiet);
  $texhashEnabled = 0 if ($opt_norehash);

  if ($opt_cnffile) {
    if ( ! -f $opt_cnffile) {
      &abort("config file `$opt_cnffile' not found");
    }
    else {
      $cnf_file = $opt_cnffile;
    }
  }

  if ($opt_outputdir) {
    if ( ! -d $opt_outputdir) {
      &abort("output directory `$opt_outputdir' not found");
    }
    else {
      $outputdir = $opt_outputdir;
    }
  }
}

###############################################################################
# mkMaps()
#   the main task of this script: create the output files
###############################################################################
sub mkMaps {

  $mode = &cfgval("LW35");
  $dvipsPreferOutline = &cfgval("dvipsPreferOutline");
  $dvipsPreferOutline = undef if ($dvipsPreferOutline =~ /^(f|n|0)/);

  if (! $quiet) {
    print "\
updmap is creating new map files using the following configuration:\
\
  config file:     $cnf_file\
  output dir:      $outputdir\
  prefer outlines: $dvipsPreferOutline\
  texhash enabled: $texhashEnabled\
";
  }
  my @tmp1 = &locateMaps('^(Mixed|)Map');
  my @tmp2 = &locateMaps('^Map');

  # Create psfonts_t1.map, psfonts_pk.map, ps2pk.map and pdftex.map:
  for my $file ("psfonts_t1.map", "psfonts_pk.map", "ps2pk.map", "pdftex.map") {
    open FILE, ">$outputdir/$file";
    print FILE "% $file: maintained by the script updmap.\
%   Don't change this file directly. Edit texmf/web2c/updmap.cfg\
%   and run updmap to recreate this file.\
";
    close FILE;
  }

  my %count = ();
  my @psfonts_t1 = &transLW35($dvips35);
  push @psfonts_t1, &getLines(@tmp1);

  @psfonts_t1 = grep { $_ !~ m/^%/ and $_ ne "" and ++$count{$_} < 2 } (sort @psfonts_t1);
  writeLines(">$outputdir/psfonts_t1.map", @psfonts_t1);

  %count = ();
  my @psfonts_pk = &transLW35($dvips35);
  push @psfonts_pk, getLines(@tmp2);
  @psfonts_pk = grep { $_ !~ m/^%/ and $_ ne "" and ++$count{$_} < 2 } (sort @psfonts_pk);
  writeLines(">$outputdir/psfonts_pk.map", @psfonts_pk);

  %count = ();
  my @ps2pk = &transLW35($ps2pk35);
  push @ps2pk, getLines(@tmp1);
  @ps2pk = grep { $_ !~ m/^%/ and ++$count{$_} < 2 } (sort @ps2pk);
  writeLines(">$outputdir/ps2pk.map", @ps2pk);

  %count = ();
  my @tmp3 = &transLW35($pdftex35);
  push @tmp3, getLines(@tmp1);
  @tmp3 = grep { $_ !~ m/(^%|PaintType)/; } @tmp3;

  my @pdftex = grep { $_ =~ m/\".*\b(Slant|Extend)Font\b.*\"/x and
                   $_ =~ s/\s[^\s]+\s/ /; } @tmp3;
  push @pdftex, grep { $_ !~ m/\".*\b(Slant|Extend)Font\b.*\"/x } @tmp3;
 
  @pdftex = grep { ++$count{$_} < 2; } (sort @pdftex);
  writeLines(">$outputdir/pdftex.map", @pdftex);

}

###############################################################################
# main()
#   execution starts here
###############################################################################
sub main {

  &initVars;
  &processOptions;
  &setupCfgFile;

  &help if ($opt_help);

  if ($opt_edit) {
    my $editor = `kpsewhich --expand-var \$TEXEDIT`;
    chomp($editor);
    $editor = "notepad" if ($editor eq "");
    system("$editor $cnf_file");
    exit 0;
  }


  &setupDestDir;
  &mkMaps;
  &setupDvipsLink;

  system("mktexlsr") if (! $opt_norehash);

}
