/**************************************************************************

DDEOpen.c

	This file is furnished to you by Adobe Systems Incorporated
	under the terms of the Acrobat (r) Plug-ins Software
	Development Kit License Agreement.

	Copyright (C) 1995-1999, Adobe Systems Inc.  All Rights Reserved.
 
PURPOSE
  
	This implements a very simple DDE client that communicates with
	the viewer to open a file called "test.pdf" that is located at
	the root level of the c drive.  The viewer must be running in
	order for the DDE transaction to take place.

Caveats:
  None. 

***************************************************************************/

// #include <afx.h>
#include <win32lib.h>
#include <ddeml.h>
#include <dde.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <shellapi.h>

#include <getopt.h>

/* The raw base name */
char *argv0 = NULL;
int nPage = 0;
char *sFileName = NULL;
int bAll = 0;
int bServer = 0;
int iDebug = 0;
char *sGoto = NULL;

const char *pdfdde_version_string = "0.1";

/* Test whether getopt found an option ``A''.
   Assumes the option index is in the variable `option_index', and the
   option table in a variable `long_options'.  */
#define ARGUMENT_IS(a) (_stricmp(long_options[option_index].name, a) == 0)
#define PROGRAM_IS(a) (_stricmp(a, argv0) == 0)

static struct option long_options [] = {
    { "debug",               0, 0, 0},
    { "help",                0, 0, 0},
    { "version",             0, 0, 0},
    { "file",                1, 0, 0},
    { "page",                1, 0, 0},
    { "goto",                1, 0, 0},
    { "all",                 0, 0, 0},
    {0, 0, 0, 0}
};

void usage()
{
  fprintf(stderr, "Usage:\n");
  fprintf(stderr, "\tpdfopen  --file <filename> [--page <page>]\n");
  fprintf(stderr, "\t\t       Opens <filename> (at page <page> if available)\n");
  fprintf(stderr, "\t\t             in Acrobat or Acrobat Reader.\n");
  fprintf(stderr, "\tpdfclose --file <filename>\n");
  fprintf(stderr, "\t         Closes <filename> in Acrobat or Acrobat Reader.\n");
  fprintf(stderr, "\tpdfclose --all\n");
  fprintf(stderr, "\t         Closes all documents in Acrobat or Acrobat Reader.\n");
  fprintf(stderr, "\tpdfdde\n");
  fprintf(stderr, "\t\t       Read commands on stdin\n");
  fprintf(stderr, "Beware: only documents opened by `pdfopen' can be closed\n");
  fprintf(stderr, "        by `pdfclose'.\n");
  exit(0);
}

char sLongFileName[MAXPATHLEN];

char *CheckFileName(char *p)
{
  char sTmp[MAXPATHLEN];
  char *fp;

  sLongFileName[0] = '\0';

  if (GetFileAttributes(p) == -1) {
    fprintf(stderr, "%s: non existent file %s\n", argv0, p);
    return NULL;
  }
  if (GetFullPathName(p, sizeof(sTmp), sTmp, &fp) == 0) {
    fprintf(stderr, "%s: failed to get full path name for %s (Error %d)\n", argv0, p, GetLastError());
    return NULL;
  }
  if (GetLongPathName(sTmp, sLongFileName, sizeof(sLongFileName)) == 0) {
    fprintf(stderr, "%s: failed to get long path name for %s (Error %d)\n", argv0, sTmp, GetLastError());
    return NULL;
  }
  return sLongFileName;
}

/* DDE variables */
DWORD dwDDEid = 0;
HSZ hszServerName, hszTopicName;
HCONV hConversation = NULL;
HDDEDATA bRetVal;
int bDDEConnected = false,
  bDDEStringsAllocated = false,
  bDDEInitialized = false;
#define DDE_ACROBAT_SERVER "acroview"
#define DDE_ACROBAT_TOPIC "control"

HDDEDATA CALLBACK DDE_ProcessMessage(  UINT uType,     // transaction type
				       UINT uFmt,      // clipboard data format
				       HCONV hconv,    // handle to the conversation
				       HSZ hsz1,       // handle to a string  HSZ hsz2,       
				       // handle to a string
				       HDDEDATA hdata, // handle to a global memory object
				       DWORD dwData1,  // transaction-specific data
				       DWORD dwData2   // transaction-specific data
				       )
{
  return NULL;
}

int DDEInitialize()
{
  int rtn;

  if (bDDEInitialized) return bDDEInitialized;

  rtn = DdeInitialize(&dwDDEid, (PFNCALLBACK)&DDE_ProcessMessage,
		      APPCMD_CLIENTONLY, 0);
  
  switch (rtn) {

  case DMLERR_NO_ERROR:
    bDDEInitialized = TRUE;
    hszServerName = DdeCreateStringHandle(dwDDEid, DDE_ACROBAT_SERVER, 0);
    hszTopicName = DdeCreateStringHandle(dwDDEid, DDE_ACROBAT_TOPIC, 0);
    bDDEStringsAllocated = TRUE;
    break;
  
  case DMLERR_INVALIDPARAMETER:
    fprintf(stderr, "%s: failed to initialize DDE (invalid parameter)\n", argv0);
    exit(EXIT_FAILURE);
    break;

  }

  return rtn;
}

int DDEConnect(char *sPath)
{
  int count = 0;
  int res = 0;

  if (bDDEConnected == TRUE) return bDDEConnected;

  do {
    hConversation = DdeConnect(dwDDEid, hszServerName, hszTopicName, NULL);
    res = DdeGetLastError(dwDDEid);

    if (iDebug) {
      fprintf(stderr, "%s: DDE connecting server=%s topic=%s (%d)\n", argv0, DDE_ACROBAT_SERVER, DDE_ACROBAT_TOPIC, res);
    }
    
    if (res == DMLERR_NO_CONV_ESTABLISHED) {
      if (count == 0) {
	ShellExecute(0, "open", sPath, 0, 0, SW_SHOWNORMAL);
	if (iDebug) {
	  fprintf(stderr, "Waiting for Acrobat to open up ");
	}
      }
      Sleep(500);
      if (iDebug) fprintf(stderr, ".");
    }
  } while (res != DMLERR_NO_ERROR && count++ < 20);
  
  if (iDebug && count > 0)
    fprintf(stderr, "\n");
  
  if (res == DMLERR_NO_ERROR) {
    bDDEConnected = TRUE;
  }
  else {
    fprintf(stderr, "%s: Unable to establish DDE connection (Error %ld)\n", argv0, res);
  }

  return bDDEConnected;
}

int DDECommand(char *format, ...)
{
  HDDEDATA rtn;
  va_list va;
  char buf[1024];
  DWORD dwResult;

  if (!bDDEInitialized || !bDDEConnected) return FALSE;

  va_start(va, format);
  (void)_vsnprintf(buf, sizeof(buf), format, va);
  va_end(va);

  rtn = DdeClientTransaction((unsigned char *)buf, (DWORD)strlen(buf),
			     (HCONV)hConversation, NULL,
			     (UINT)CF_TEXT, (UINT)XTYP_EXECUTE, (DWORD)1000, &dwResult);
  if (iDebug) {
    fprintf(stderr, "%s: DDE command `%s' (%d)\n", argv0, buf, rtn);
  }
  return (rtn == DMLERR_NO_ERROR);
}

int DDEDisconnect()
{
  if (bDDEConnected) {
    DdeDisconnect(hConversation);
    bDDEConnected = FALSE;
  }
  if (bDDEStringsAllocated) {
    DdeFreeStringHandle(dwDDEid, hszServerName);
    DdeFreeStringHandle(dwDDEid, hszTopicName);
    bDDEStringsAllocated = FALSE;
  }
  return TRUE;
}

int DDEUninitialize()
{
  if (bDDEInitialized) {
    DdeUninitialize(dwDDEid);
    bDDEInitialized = FALSE;
  }
  return TRUE;
}    

int pdfopen()
{
  if (sLongFileName[0] == '\0') {
    fprintf(stderr, "%s: mandatory `--file' argument not found.\n", argv0);
    return EXIT_FAILURE;
  }

  DDEInitialize();
  
  DDEConnect(sLongFileName);
  
  DDECommand("[DocOpen(\"%s\")]", sLongFileName); 
  DDECommand("[FileOpen(\"%s\")]", sLongFileName); 

  if (nPage > 0) {
    DDECommand("[DocGoTo(\"%s\", %d)]", sLongFileName, nPage); 
  }

  if (sGoto != NULL) {
    DDECommand("[DocGoToNameDest(\"%s\", %s)]", sLongFileName, sGoto); 
  }

  DDEDisconnect();
  DDEUninitialize();
#if 0
  /* Problem : it seems that these calls can put AR into a strange state.
     It would have been simpler if Adobe had simply iconified their application. */
  if (stricmp(argv0, "pdfclose") == 0) {
    sprintf(buf, "[AppHide()]");
  }
  else if (stricmp(argv0, "pdfopen") == 0) {
    sprintf(buf, "[AppShow()]");
  }
  bRetVal = DdeClientTransaction((unsigned char *)buf, (DWORD)strlen(buf),
				 (HCONV)hConversation, NULL,
				 (UINT)CF_TEXT, (UINT)XTYP_EXECUTE, (DWORD)1000, &dwResult);
#endif
  
  return EXIT_SUCCESS;
}

int pdfclose()
{
  DDEInitialize();
  
  DDEConnect(sLongFileName);
  
  if (bAll) {
    DDECommand("[CloseAllDocs()]");
  }
  else if (sLongFileName[0] != '\0') {
    DDECommand("[DocClose(\"%s\")]", sLongFileName);
  }
  
#if 0
  fprintf(stderr, "cmd : %s\n", buf);
#endif
  
  DDEDisconnect();

  return EXIT_SUCCESS;
}

int pdfserver()
{
  char line[1024];

  DDEInitialize();
  
  fprintf(stdout, "Type Ctrl-Z to quit.\n");

  while (fgets(line, sizeof(line), stdin)) {
#define WHITE_DELIM " \t\n"
    char *p = strtok(line, WHITE_DELIM);
    if (stricmp(p, "open") == 0) {
      p = strtok(NULL, WHITE_DELIM);
      if (CheckFileName(p) == NULL) goto next;

      if (! bDDEConnected) {
	DDEConnect(sLongFileName);
      }
  
      DDECommand("[DocOpen(\"%s\")]", sLongFileName); 
    }
    else if (stricmp(p, "close") == 0) {
      p = strtok(NULL, WHITE_DELIM);
      if (CheckFileName(p) == NULL) goto next;
      DDECommand("[DocClose(\"%s\")]", sLongFileName);
    }
    else if (stricmp(p, "closeall") == 0) {
      DDECommand("[CloseAllDocs()]");
    }
    else if (stricmp(p, "goto") == 0) {
      p = strtok(NULL, WHITE_DELIM);
      if (p == NULL) goto next;
      if (CheckFileName(p) == NULL) goto next;
      p = strtok(NULL, WHITE_DELIM);
      if (p == NULL) goto next;
      nPage = atoi(p) - 1;
      DDECommand("[DocGoTo(\"%s\", %d)]", sLongFileName, nPage); 
    }
    else if (stricmp(p, "gotoname") == 0) {
      p = strtok(NULL, WHITE_DELIM);
      if (p == NULL) goto next;
      if (CheckFileName(p) == NULL) goto next;
      p = strtok(NULL, WHITE_DELIM);
      if (p == NULL) goto next;
      sGoto = p;
      DDECommand("[DocGoToNameDest(\"%s\", %s)]", sLongFileName, sGoto); 
    }
    else if (stricmp(p, "show") == 0) {
      DDECommand("[AppShow()]");

    }
    else if (stricmp(p, "hide") == 0) {
      DDECommand("[AppHide()]");

    }
    else if (stricmp(p, "exit") == 0) {
      DDECommand("[AppExit()]");
      DDEDisconnect();
    }
    else if (stricmp(p, "list") == 0) {
      fprintf(stdout, "open <file>		open the file\n");
      fprintf(stdout, "close <file>		close the file\n");
      fprintf(stdout, "closeall		close all files\n");
      fprintf(stdout, "goto <file> <page>	goto the given <page> in <file>\n");
      fprintf(stdout, "gotoname <file> <dest>	goto the <dest> named destination in <file>\n");
      fprintf(stdout, "show			show acroview\n");
      fprintf(stdout, "hide			hide acroview\n");
      fprintf(stdout, "exit			exit acroview and %s\n", argv0);
    }
  next: ;   
  }

#if 0
  fprintf(stderr, "cmd : %s\n", buf);
#endif
  
  DDEDisconnect();
  DDEUninitialize();

  return EXIT_SUCCESS;
}

int main (int argc, char *argv[])
{
  BOOL bRet = FALSE;
  char *p;
  int i;
  int argv0len = 0;
  int g; /* getopt return code */
  int option_index;

  /* Make argv0 to point to the raw base name */
  argv0 = argv[0];
  if ((p = strrchr(argv0, '/'))
      || (p = strrchr(argv0, '\\'))) {
    argv0 = _strdup(p+1);
  }
  if ((argv0len = strlen(argv0)) > 4 
      && _strnicmp(argv0 + argv0len - 4, ".exe", 4) == 0)
    argv0[argv0len - 4] = '\0';

  for(;;) {
    g = getopt_long_only (argc, argv, "", long_options, &option_index);
    
    if (g == EOF)
      break;

    if (g == '?') {
      usage();			/* Unknown option.  */
      exit(EXIT_FAILURE);
    }

    /* assert (g == 0); */ /* We have no short option names.  */
    /*
      FIXME : try 'mktexpk --mfmode --bdpi 600 ...'
    */

    if (ARGUMENT_IS ("debug")) {
#if 0
      if (! isalnum(*optarg)) {
	fprintf(stderr, "%s: warning, `%s' bad debug argument.\n", argv0, optarg);
      }
      iDebug |= atoi (optarg);
#endif
      iDebug = TRUE;
    }
    else if (ARGUMENT_IS ("help")) {
      usage();
      exit(EXIT_SUCCESS);
    }
    else if (ARGUMENT_IS ("version")) {
      fprintf(stderr, "%s of %s.\n", argv0, pdfdde_version_string);
      exit(EXIT_SUCCESS);
    }
    else if (PROGRAM_IS("pdfopen")) {
      if (ARGUMENT_IS("file")) {
	CheckFileName(optarg);
      }
      else if (ARGUMENT_IS("page")) {
	nPage = atoi(optarg) - 1;
      }
      else if (ARGUMENT_IS("goto")) {
	sGoto = optarg;
      }
    }
    else if (PROGRAM_IS("pdfclose")) {
      if (ARGUMENT_IS("file")) {
	CheckFileName(optarg);
      }
      else if (ARGUMENT_IS("all")) {
	bAll = TRUE;
      }
    }
  }

#if 0
  /* shifting options from argv[] list */
  for (i = 0; i < argc - optind; i++)
    argv[i] = argv[optind+i];
  argv[i] = NULL;

  argc = i;
#endif

  if (PROGRAM_IS("pdfopen")) {
    bRet = pdfopen();
  }
  else if (PROGRAM_IS("pdfclose")) {
    bRet = pdfclose();
  }
  else if (PROGRAM_IS("pdfdde")) {
    bRet = pdfserver();
  }
  else {
    fprintf(stderr, "This program has been incorrectly copied to the name %s.\n", argv[0]);
    bRet = EXIT_FAILURE;
  }
  return bRet;
}






