################################################################################
#
# Makefile  : Common, definitions to build win32 tools
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <02/06/02 14:16:06 popineau>
#
################################################################################

################################################################################
#
# Configurable section
#
################################################################################

# MSVC main variables
CPU=i386
MSVCROOT=c:\Program Files\Microsoft Visual Studio .NET\VC7
# Change to 'on' if you want verbose builds
verbose = off
# Define for profiling
#PROFILE = 1
# define for optimized code
#NODEBUG = 1
# define for static binaries (no kpathsea.dll, standalone exe files)
#STATIC=
# define for debug malloc
#MEMDBG=1
# define for BoundsChecker FinalCheck
#BOUNDSCHECKER=1

# Configurable section

# Where to install things
# This would be better if it where
# relative to the main directory, but it clutters
# things too much.
#
prefix = d:\Local\TeXLive
webdir = d:\InetPub\htdocs\fptex
ftpdir = d:\InetPub\ftp\fptex

#
# Source directories
#
!ifndef root_srcdir
root_srcdir = D:\Source\fptex
!endif
top_srcdir = $(root_srcdir)\texk
srcdir=	.
win32=	$(root_srcdir)\win32

#
# Tools
#
gnutools = d:\Local\Gnu
netpbmtools = d:\Local\NetPBM
perltools = d:\Local\Perl

# Cmd commands
copy    = copy /y
copydir = xcopy /q/y/r/i/e/k
del     = del /f/q
deldir  = rmdir /q/s
echo    = echo
make    = nmake -f win32.mak -nologo -$(MAKEFLAGS) verbose=$(verbose)
mkdir   = mkdir
move    = move /y
shell   = cmd /x

# Gnu and various commands
awk	= $(gnubin)\gawk
diff	= $(gnubin)\diff.exe -u
eqn	= $(gnubin)\eqn
expand	= $(gnubin)\expand
grep	= $(gnubin)\grep.exe
grodvi	= $(gnubin)\grodvi
grops	= $(gnubin)\grops
gzip	= $(gnubin)\gzip -f
lex	= $(gnubin)\flex
!ifndef STATIC
lnexe	= $(copy)
!else
lnexe	= $(root_srcdir)\utils\win32-cmd\$(objdir)\lnexe.exe
!endif
makedepend = $(gnubin)\makedepend # -Y
perl	= perl -I$(win32perldir)
rman	= $(gnubin)\rman
runperl	= $(root_srcdir)\utils\win32-cmd\$(objdir)\runperl.exe
sed	= $(gnubin)\sed.exe
sort	= $(gnubin)\sort.exe
tbl	= $(gnubin)\tbl
touch 	= $(gnubin)\touch.exe
tr	= $(gnubin)\tr.exe
troff	= $(gnubin)\groff
yacc	= $(gnubin)\bison -y

###############################################################################
#
# End of configurable section
#
###############################################################################

################################################################################
#
# Version numbers
#
################################################################################
fptexversion = 0.6
#
web2cmaj = 7
web2cmin = 3
web2cbeta = 3x
#
kpsemaj = 3
kpsemin = 3
kpsebeta = 3x
#
buildmaj = 7
buildmin = 3
buildbeta = 3x
buildgamma = 1
################################################################################

#
# Subdirectories of win32
#
win32defdir = $(win32)\def
win32rcdir = $(win32)\rc
win32perldir = $(win32)\perl
win32makedir = $(win32)\make
win32seddir = $(win32)\sed
win32srcdir = $(win32)\src

#
# Where to find gnu tools and to put other stuff ?
# tr, sed, grep, cat are needed
#
gnuroot = $(gnutools)^\
gnubin = $(gnutools)\bin
gnuinclude = $(gnutools)\include
gnulib = $(gnutools)\lib
gnushare = $(gnutools)\share
gnuinfo = $(gnutools)\info
gnuman = $(gnutools)\man

perlbin = $(perltools)\bin
perllib = $(perltools)\lib

#
# To make the build process more silent
#
!if ("$(verbose)" == "on")
!else
silent = @
redir_stdout = >nul
redir_stderr = 2>nul
!endif

!if "$(OS)" == "Windows_NT"
null=
!else 
null=nul
!endif 

################################################################################
#
# Several targets:
#	* debug means non optimized
#	* static means statically linked
#	* profile implies static
# Choose one obj target among these
#
################################################################################
#
objtargets = static dynamic static-debug dynamic-debug profile profile-debug
#
CONFIG_MT=1
!ifndef STATIC
MSVC_DLL=1
!endif
!ifdef PROFILE
# Profile implies Static. Too much complicated otherwise
objdir=profile
!elseifdef STATIC
objdir=static
!else
objdir=dynamic
!endif


#
!ifndef NODEBUG
objdir=$(objdir)-debug
!endif
#
################################################################################


################################################################################
#
# The Perl Compiler
#
################################################################################

perlcomp = perlapp
perlcompflags = --force --freestanding --lib $(texmf)/context/perltk --xclude --exe $@ --verbose

################################################################################
#
# Your C Compiler
#
################################################################################
#
# BoundsChecker
#
!ifdef BOUNDSCHECKER
BOUNDSCHECKERPATH = C:\Program Files\Compuware\BoundsChecker
PATH = $(BOUNDSCHECKERPATH);$(PATH)
CC = $(silent)nmcl
CCLD = $(silent)nmlink
optlibs = "$(BOUNDSCHECKER)"\bcinterf.lib
!elseifdef INTEL_COMPILER
#
# Intel compiler definitions
#
PATH = c:\program files\Intel\compiler45\bin;$(PATH)
CC = $(silent)icl
LIB=C:\Program Files\Intel\compiler45\lib;$(LIB)
CCLD = $(silent)xilink
!else
#
# Microsoft C/C++ Compiler
#
CC = $(silent)cl
CCLD = $(silent)link
!endif
# Build libraries
AR = $(silent)lib
# resource compiler
RC = $(silent)rc
################################################################################

PATH = $(kpathseadir)\$(objdir);.\$(objdir);$(PATH)

# Define to get strict Ansi conformance
STRICT_ANSI = 1
# Define to use oldnames.lib instead of macros
USE_OLDNAMES = 1

# Ensure that if MSVCRT is choosen, MT is defined.
!ifdef MSVC_DLL
!ifndef CONFIG_MT
CONFIG_MT=1
!endif
!endif

cvars =

!IF "$(CPU)" == "i386"
cvars = $(cvars) -D_X86_=1
lflags = /MACHINE:IX86
libflags = /MACHINE:IX86
!ELSE
!IF "$(CPU)" == "MIPS"
cvars = $(cvars) -D_MIPS_=1
!ELSE
!IF "$(CPU)" == "PPC"
cvars = $(cvars) -D_PPC_=1
!ELSE
!IF "$(CPU)" == "ALPHA"
cvars = $(cvars) -D_ALPHA_=1
!endif
!endif
!endif
!endif

# For this release, for all values of APPVER, WINVER is 0x0400
cvars = $(cvars) -DWINVER=0x0400

# For Windows applications that use the C Run-Time libraries
# Use the DLL MultiThread C Library
cvars   = $(cvars) -DWIN32 -D_WIN32 -D_WINDOWS
!ifdef CONFIG_MT
cvars   = $(cvars) -D_MT
!endif
!ifdef MSVC_DLL
cvars   = $(cvars) -D_DLL
!endif

# Compilation flags.
!ifdef NODEBUG
optflags = -O2x -G6r -Gy # -QaxiK -Qipo_obj -Qunroll # Intel specific
!ELSE
optflags = -Od -G6r -Zi
#optflags = -O2x -G5rs -Gy -Zi
!endif

!ifdef PROFILE
optflags = $(optflags) -Gh -Fm
lmap = /MAP /FIXED:NO /PROFILE
!endif

#
# Preprocessor flags
#
XCFLAGS = -DFPTEX=1
!ifdef MEMDBG
XCFLAGS = $(XCFLAGS) -D_DEBUG -D_CRTDBG_MAP_ALLOC
!endif

CFLAGS = -W3 -DCRTAPI1=_cdecl -DCRTAPI2=_cdecl -nologo \
	$(optflags) $(XCFLAGS)
CPPFLAGS = $(XCPPFLAGS)
DEFS = $(XDEFS) $(cvars) -I.

!if defined(STRICT_ANSI)
#CFLAGS = $(CFLAGS) -Za
DEFS = $(DEFS) -D__STDC__=1
!endif

# The default `compile' command
CFLAGS = $(CFLAGS) $(DEFS)

compile = $(CC) $(CFLAGS) -c /Fo$@

makedepend_cflags = -I. -DCRTAPI1=_cdecl -DCRTAPI2=_cdecl -D_M_IX86=1 $(XCFLAGS) $(DEFS)

!ifdef NODEBUG
ldebug = /RELEASE /OPT:REF /VERSION:$(buildmaj).$(buildmin)
!ELSE
ldebug = -debug:full -debugtype:cv
# ldebug = -debug:full -debugtype:cv -opt:ref
!endif

lflags =  $(lflags) /NODEFAULTLIB /INCREMENTAL:NO /NOLOGO \
	$(ldebug) $(lmap)
conlflags = $(lflags) -subsystem:console -entry:mainCRTStartup
dlllflags = $(lflags) -entry:_DllMainCRTStartup@12 -dll
guilflags = $(lflags) -subsystem:windows -entry:WinMainCRTStartup

!ifdef PROFILE
optlibs =  $(optlibs) cap.lib
!endif

basewinlibs = kernel32.lib $(optlibs)

!ifdef USE_OLDNAMES
basewinlibs = oldnames.lib $(basewinlibs)
!else
DEFS = $(DEFS) -DOLDNAMES_WITH_MACROS=1
!endif

!if (defined(NODEBUG) && !defined(MEMDBG))
! ifdef MSVC_DLL
libc = msvcrt.lib $(basewinlibs)
! else
!  ifdef CONFIG_MT
libc = libcmt.lib $(basewinlibs)
!  else
libc = libc.lib $(basewinlibs)
!  endif
! endif
!else
! ifdef MSVC_DLL
libc = msvcrtd.lib $(basewinlibs)
! else
!  ifdef CONFIG_MT
libc = libcmtd.lib $(basewinlibs)
!  else
libc = libcd.lib $(basewinlibs)
!  endif
! endif
!endif /* NODEBUG */

winlibs = user32.lib gdi32.lib comdlg32.lib winspool.lib \
	version.lib comctl32.lib

!if defined(USE_GSW32)
USE_ADVAPI = 1
!endif

!ifdef USE_ADVAPI
advapiflags = /delayload:advapi32.dll
advapilibs = advapi32.lib

libc = $(libc) $(advapilibs)
lflags = $(lflags) $(advapiflags)
!endif

# Delay load wsock32 and wininet when required
!ifdef USE_NETWORK
networklibs = wsock32.lib wininet.lib
networkflags = /delayload:wsock32.dll /delayload:wininet.dll

libc = $(libc) $(networklibs)
lflags = $(lflags) $(networkflags)
!endif

!ifdef USE_NETWORK2
networklibs = ws2_32.lib wininet.lib
networkflags = /delayload:ws2_32.dll /delayload:wininet.dll

libc = $(libc) $(networklibs)
lflags = $(lflags) $(networkflags)
!endif

!if defined(USE_ADVAPI) || defined(USE_NETWORK) || defined(USE_NETWORK2)
libc = $(libc) delayimp.lib
!endif
#
# This is stupid with MSVC7 : using winsock2
# you need to define WIN32_LEAN_AND_MEAN
# else winsock2.h conflicts with winsock.h !
#
!if defined(USE_NETWORK)
DEFS = $(DEFS) -DUSE_WINSOCK
!else
DEFS = $(DEFS) -DWIN32_LEAN_AND_MEAN
!endif

rcflags = /r /I$(win32rcdir)
rcvars = -DWIN32 -D_WIN32 -DWINVER=0x0400 $(xrcvars)

.SUFFIXES: .c .cc .obj .l .y .res .rc .pl # in case the suffix list has been cleared, e.g., by web2c

{}.c{$(objdir)}.obj:
	$(compile) $<

{}.cc{$(objdir)}.obj:
	$(compile) /Tp $<

{}.cpp{$(objdir)}.obj:
	$(compile) /Tp $<

{}.pl{$(objdir)}.exe:
	$(perlcomp) $(perlcompflags) $<

.l.c:
	$(lex) -t $< | $(SED) "/^extern int isatty YY/d" > $@

.y.c:
	$(yacc) -d -v $< -o $@

{$(win32)\rc}.rc{$(objdir)}.res:
	$(RC) $(rcvars) $(rcflags) /fo$@ $<

.rc{$(objdir)}.res:
	$(RC) $(rcvars) $(rcflags) /fo$@ $<

# We might link :
# - by default for console programs
# - for building dlls
# - for building gui apps
link = $(CCLD) /OUT:$@ $(conlflags)
link_dll = $(CCLD) /OUT:$@ $(dlllflags)
link_gui = $(CCLD) /OUT:$@ $(guilflags)

conlibs = $(libc) user32.lib $(XLIBS)
guilibs = $(libc) $(winlibs) $(XLIBS)

# for OLE applications
olelibs      = ole32.lib uuid.lib oleaut32.lib

archive = $(AR) -nologo $(libflags) /OUT:$@

################################################################################
#
# Various paths definitions
#
################################################################################
!ifdef GNU
!include <make/gnupaths.mak>
!elseifdef NETPBM
!include <make/netpbmpaths.mak>
!else
!include <make/paths.mak>
!endif

################################################################################
#
# Various personal libraries definitions
#
################################################################################
!include <make/libsdef.mak>

#
# Local Variables:
# mode: Makefile
# End:
