################################################################################
#
# Makefile  : libsdef, definitions for using and building libraries
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <02/03/15 22:28:57 popineau>
#
################################################################################

#gifver=-0.9
#jpgver=-6b
#pngver=-1.0.5
#regexver=-0.12
#tiffver=-v3.5.2
#ttfver=-1.3
#wwwver=-5.2.1
#xpdfver=-0.90
#zlibver=-1.1.3
#bzip2ver=-0.9.0c
#t1ver=-0.9
#gnuw32ver=-0.1
xpmver = -3.4k
openldapver = -2.0.7

#
# No dlls if compiling statically
#
!if defined(STATIC) || defined(PROFILE)
!undef BZIP2_DLL
!undef COMPFACE_DLL
!undef CURL_DLL
!undef EXPAT_DLL
!undef FREETYPE_DLL
!undef GETURL_DLL
!undef GIF_DLL
!undef GNUW32_DLL
!undef GSW32_dll
!undef JPEG_DLL
!undef KPSE_DLL
!undef MKTEX_DLL
!undef NETPBM_DLL
!undef OPENLDAP_DLL
!undef PNG_DLL
!undef REGEX_DLL
!undef T1_DLL
!undef TEX_DLL
!undef TIFF_DLL
!undef TTF_DLL
!undef UNZIP_DLL
!undef WMF_DLL
!undef WWW_DLL
!undef XPDF_DLL
!undef XPM_DLL
!undef ZLIB_DLL
!else
BZIP2_DLL = 1
CURL_DLL = 1
COMPFACE_DLL = 1
EXPAT_DLL = 1
FREETYPE_DLL = 1
GETURL_DLL = 1
GIF_DLL = 1
# GNUW32_DLL = 1
# GSW32_DLL = 1
JPEG_DLL = 1
KPSE_DLL = 1
MKTEX_DLL = 1
NETPBM_DLL = 1
OPENLDAP_DLL = 1
PNG_DLL = 1
REGEX_DLL = 1
T1_DLL = 1
TEX_DLL = 1
TIFF_DLL = 1
TTF_DLL = 1
UNZIP_DLL = 1
WMF_DLL = 1
WWW_DLL = 1
# XPDF_DLL = 1
XPM_DLL = 1
ZLIB_DLL = 1
!endif

#
# The kpathsea library
#
kpathseadir = $(top_srcdir)\kpathsea
!ifdef USE_KPATHSEA
kpathsealib=$(kpathseadir)\$(objdir)\kpathsea.lib
!ifdef KPSE_DLL
DEFS = $(DEFS) -DKPSE_DLL -DKPATHSEA=1
PATH=$(kpathsea_dir)\$(objdir);$(PATH)
kpathseadll=$(kpathseadir)\$(objdir)\kpathsea.dll
!else
DEFS = $(DEFS) -DKPATHSEA=1
!endif
DEFS = $(DEFS) -I$(kpathseadir) -I$(kpathseadir)\..
kpathsea = $(kpathsealib) $(kpathseadll)
!endif

#
# The mktex library
#
mktexdir = $(top_srcdir)\contrib
!ifdef USE_MKTEX
mktexlib=$(mktexdir)\$(objdir)\libmktex.lib
!ifdef MKTEX_DLL
DEFS = $(DEFS) -DMKTEX_DLL -DMKTEX=1
PATH=$(mktex_dir)\$(objdir);$(PATH)
mktexdll=$(mktexdir)\$(objdir)\libmktex.dll
!else
DEFS = $(DEFS) -DMKTEX=1
!endif
DEFS = $(DEFS) -I$(mktexdir) -I$(mktexdir)\..
mktex = $(mktexlib) $(mktexdll)
!endif

#
# The bzip2 library
#
!ifdef USE_BZIP2
bzip2dir=$(root_srcdir)\utils\bzip2$(bzip2ver)
bzip2lib = $(bzip2dir)\$(objdir)\bz2lib.lib
!ifdef BZIP2_DLL
bzip2dll = $(bzip2dir)\$(objdir)\bz2lib.dll
DEFS = $(DEFS) -DBZIP2_DLL
PATH = $(bzip2dir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(bzip2dir)
bzip2 = $(bzip2lib) $(bzip2dll)
!endif

#
# The gifreader library
#
!ifdef USE_GIF
gifdir = $(root_srcdir)\libs\gifreader$(gifver)
giflib = $(gifdir)\$(objdir)\libgifreader.lib
!ifdef GIF_DLL
gifdll = $(gifdir)\$(objdir)\libgifreader.dll
DEFS = $(DEFS) -DGIF_DLL
PATH = $(gifdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(gifdir)
gif = $(giflib) $(gifdll)
!endif

#
# The gnuw32 unix compatibility library
#
!ifdef USE_GNUW32
gnuw32dir=$(root_srcdir)\libs\libgnuw32
gnuw32lib = $(gnuw32dir)\$(objdir)\libgnuw32.lib
!ifdef GNUW32_DLL
gnuw32dll = $(gnuw32dir)\$(objdir)\libgnuw32.dll
DEFS=$(DEFS) -DGNUW32_DLL
PATH = $(gnuw32dir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(gnuw32dir)
!ifndef USE_KPATHSEA
DEFS = $(DEFS) -I$(kpathseadir)
!endif
gnuw32 = $(gnuw32lib) $(gnuw32dll)
!endif

#
# The GS W32 unix compatibility library
#
!ifdef USE_GSW32
gsw32dir=$(root_srcdir)\libs\libgsw32
gsw32lib = $(gsw32dir)\$(objdir)\libgsw32.lib
!ifdef GSW32_DLL
gsw32dll = $(gsw32dir)\$(objdir)\libgsw32.dll
DEFS=$(DEFS) -DGSW32_DLL
PATH = $(gsw32dir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(gsw32dir)
!ifndef USE_KPATHSEA
DEFS = $(DEFS) -I$(kpathseadir)
!endif
gsw32 = $(gsw32lib) $(gsw32dll)
!endif

#
# The jpeg library
#
!ifdef USE_JPEG
jpegdir = $(root_srcdir)\libs\jpeg$(jpegver)
jpeglib = $(jpegdir)\$(objdir)\libjpeg.lib
!ifdef JPEG_DLL
jpegdll = $(jpegdir)\$(objdir)\libjpeg.dll
DEFS = $(DEFS) -DJPEG_DLL
PATH = $(jpegdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(jpegdir)
jpeg = $(jpeglib) $(jpegdll)
!endif

#
# The png library
#
!ifdef USE_PNG
pngdir=$(root_srcdir)\libs\libpng$(pngver)
pnglib = $(pngdir)\$(objdir)\libpng.lib
!ifdef PNG_DLL
pngdll = $(pngdir)\$(objdir)\libpng.dll
DEFS = $(DEFS) -DPNG_DLL
PATH = $(pngdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -DPNG_NO_GLOBAL_ARRAYS -DPNG_USE_PNGVCRD -D__FLAT__ -I$(pngdir)
png = $(pnglib) $(pngdll)
!endif

#
# The regex library
#
!ifdef USE_REGEX
regexdir=$(root_srcdir)\libs\regex$(regexver)
regexlib = $(regexdir)\$(objdir)\regex.lib
!ifdef REGEX_DLL
regexdll = $(regexdir)\$(objdir)\regex.dll
DEFS = $(DEFS) -DREGEX_DLL
PATH = $(regexdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(regexdir)
regex = $(regexlib) $(regexdll)
!endif

#
# The Type1 library
#
!ifdef USE_T1
t1dir=$(root_srcdir)\libs\T1$(t1ver)\lib\t1lib
t1lib = $(t1dir)\$(objdir)\t1lib.lib
!ifdef T1_DLL
t1dll = $(t1dir)\$(objdir)\t1lib.dll
DEFS = $(DEFS) -DT1_DLL
PATH = $(t1dir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(t1dir)
t1 = $(t1lib) $(t1dll)
!endif

#
# The TIFF library
#
!ifdef USE_TIFF
tiffdir=$(root_srcdir)\libs\libtiff$(tiffver)
tifflib = $(tiffdir)\$(objdir)\libtiff.lib
!ifdef TIFF_DLL
tiffdll = $(tiffdir)\$(objdir)\libtiff.dll
DEFS = $(DEFS) -DTIFF_DLL
PATH = $(tiffdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(tiffdir)
tiff = $(tifflib) $(tiffdll)
!endif

#
# The TrueType library
#
!ifdef USE_TTF
ttfdir = $(root_srcdir)\libs\libttf$(ttfver)
ttflib = $(ttfdir)\$(objdir)\libttf.lib
!ifdef TTF_DLL
ttfdll = $(ttflib:.lib=.dll)
DEFS = $(DEFS) -DTTF_DLL
PATH = $(ttfdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(ttfdir)
ttf = $(ttflib) $(ttfdll)
!endif

#
# The Freetype library
#
!ifdef USE_FREETYPE
freetypedir = $(root_srcdir)\libs\freetype$(freetypever)
freetypelib = $(freetypedir)\src\$(objdir)\freetype.lib
!ifdef FREETYPE_DLL
freetypedll = $(freetypelib:.lib=.dll)
DEFS = $(DEFS) -DFREETYPE_DLL
PATH = $(freetypedir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(freetypedir)\include
freetype = $(freetypelib) $(freetypedll)
!endif

#
# The Curl library
#
!ifdef USE_CURL
curldir = $(root_srcdir)\libs\curl$(curlver)
curllib = $(curldir)\lib\$(objdir)\libcurl.lib
!ifdef CURL_DLL
curldll = $(curllib:.lib=.dll)
DEFS = $(DEFS) -DCURL_DLL
PATH = $(curldir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(curldir)\include
curl = $(curllib) $(curldll)
!endif

#
# The www library
#
!ifdef USE_WWW
wwwdir = $(root_srcdir)\libs\libwww$(wwwver)
wwwlib = $(wwwdir)\$(objdir)\libwww.lib
!ifdef WWW_DLL
wwwdll = $(wwwdir)\$(objdir)\libwww.dll
DEFS = $(DEFS) -DWWW_DLL
# WWWLIB_FLAGS=-DWWW_WIN_DLL -DWWW_WIN_ASYNC
PATH = $(wwwdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(wwwdir)
www =  $(wwwlib) $(wwwdll)
!endif

#
# The xpdf library
#
!ifdef USE_XPDF
xpdfdir = $(root_srcdir)\libs\xpdf$(xpdfver)
xpdflib = $(xpdfdir)\xpdf\$(objdir)\libxpdf.lib
!ifdef XPDF_DLL
xpdfdll = $(xpdfdir)\xpdf\$(objdir)\libxpdf.dll
PATH = $(xpdfdir)\xpdf\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(xpdfdir) -I$(xpdfdir)\xpdf -I$(xpdfdir)\Goo
xpdf = $(xpdflib) $(xpdfdll)
!endif

#
# The zlib library
#
!ifdef USE_ZLIB
zlibdir=$(root_srcdir)\libs\zlib$(zlibver)
zliblib = $(zlibdir)\$(objdir)\zlib.lib
!ifdef ZLIB_DLL
zlibdll = $(zlibdir)\$(objdir)\zlib.dll
DEFS = $(DEFS) -DZLIB_DLL
PATH = $(zlibdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(zlibdir)
zlib = $(zliblib) $(zlibdll)
!endif

#
# The expat library
#
!ifdef USE_EXPAT
expatdir=$(root_srcdir)\libs\expat$(expatver)\lib
expatlib = $(expatdir)\$(objdir)\expat.lib
!ifdef EXPAT_DLL
expatdll = $(expatdir)\$(objdir)\expat.dll
DEFS = $(DEFS) -DEXPAT_DLL
PATH = $(expatdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(expatdir)
expat = $(expatlib) $(expatdll)
!endif

#
# The unzip library
#
!ifdef USE_UNZIP
unzipdir=$(root_srcdir)\libs\unzip$(unzipver)
unziplib = $(unzipdir)\$(objdir)\unzip.lib
!ifdef UNZIP_DLL
unzipdll = $(unzipdir)\$(objdir)\unzip.dll
DEFS = $(DEFS) -DUNZIP_DLL
PATH = $(unzipdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(unzipdir) -I$(unzipdir)\windll
unzip = $(unziplib) $(unzipdll)
!endif

#
# The geturl library
#
!ifdef USE_GETURL
geturldir=$(root_srcdir)\libs\geturl$(geturlver)
geturllib = $(geturldir)\$(objdir)\geturl.lib
!ifdef GETURL_DLL
geturldll = $(geturldir)\$(objdir)\geturl.dll
DEFS = $(DEFS) -DGETURL_DLL
PATH = $(geturldir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(geturldir)
geturl = $(geturllib) $(geturldll)
!endif

#
# The netpbm libraries
#
!ifdef USE_NETPBM
netpbmdir = $(root_srcdir)\utils\netpbm
netpbmlib = $(netpbmdir)\pbm\$(objdir)\libpbm.lib \
	$(netpbmdir)\pbm\$(objdir)\libpnm.lib \
	$(netpbmdir)\pbm\$(objdir)\libpgm.lib \
	$(netpbmdir)\pbm\$(objdir)\libppm.lib
!ifdef NETPBM_DLL
netpbmdll = $(netpbmdir)\pbm\$(objdir)\libpbm.dll \
	$(netpbmdir)\pbm\$(objdir)\libpnm.dll \
	$(netpbmdir)\pbm\$(objdir)\libpgm.dll \
	$(netpbmdir)\pbm\$(objdir)\libppm.dll
DEFS = $(DEFS) -DNETPBM_DLL # -DPNM_DLL -DPGM_DLL -DPPM_DLL
PATH = $(netpbmdir)\pbm\$(objdir);$(netpbmdir)\pnm\$(objdir);$(netpbmdir)\pgm\$(objdir);$(netpbmdir)\ppm\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(netpmdir)\pbm -I$(netpmdir)\pnm -I$(netpmdir)\pgm -I$(netpmdir)\ppm
netpbm = $(netpbmlib) $(netpbmdll)
!endif

#
# The wmf libraries
#
!ifdef USE_WMF
wmfdir = $(root_srcdir)\utils\libwmf
wmfapidir = $(wmfdir)\src\api
wmfapilib = $(wmfdir)\$(objdir)\libwmfapi.lib
wmfapidll = $(wmfdir)\$(objdir)\libwmfapi.dll
wmfipadir = $(wmfdir)\src\ipa
wmfipalib = $(wmfdir)\$(objdir)\libwmfipa.lib
wmfipadll = $(wmfdir)\$(objdir)\libwmfipa.dll
wmflib = $(wmfapilib) $(wmfipalib)
!ifdef WMF_DLL
wmfdll = $(wmfapidll) $(wmfipadll)
DEFS = $(DEFS) -DWMF_DLL
PATH = $(wmfapidir)\$(objdir);$(wmfipadir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(wmfapidir) -I$(wmfipadir) -I$(wmfdir)\src -I$(wmfdir)\include
wmfapi = $(wmfapilib) $(wmfapidll)
wmfipa = $(wmfipalib) $(wmfipadll)
wmf = $(wmflib) $(wmfdll)
!endif

#
# The XPM library
#
!ifdef USE_XPM
xpmdir=$(gnuroot_srcdir)\xpm$(xpmver)
xpmlib = $(xpmdir)\lib\$(objdir)\libxpm.lib
!ifdef XPM_DLL
xpmdll = $(xpmdir)\lib\$(objdir)\libxpm.dll
DEFS = $(DEFS) -DXPM_DLL
PATH = $(xpmdir)\lib\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(xpmdir)
xpm = $(xpmlib) $(xpmdll)
!endif

#
# The openldap library
#
!ifdef USE_OPENLDAP
openldapdir=$(gnuroot_srcdir)\openldap$(openldapver)
openlavllib = $(openldapdir)\libraries\libavl\$(objdir)\openlavl.lib
openlutillib = $(openldapdir)\libraries\liblutil\$(objdir)\openlutil.lib
openlberlib = $(openldapdir)\libraries\liblber\$(objdir)\openlber.lib
openldaplib = $(openldapdir)\libraries\libldap\$(objdir)\openldap.lib
openldap_rlib = $(openldapdir)\libraries\libldap_r\$(objdir)\openldap_r.lib
openldbmlib = $(openldapdir)\libraries\libldbm\$(objdir)\openldbm.lib
openldiflib = $(openldapdir)\libraries\libldif\$(objdir)\openldif.lib
openlunicodelib = $(openldapdir)\libraries\liblunicode\$(objdir)\openlunicode.lib
!ifdef OPENLDAP_DLL
openlavldll = $(openldapdir)\libraries\libavl\$(objdir)\openlavl.dll
openlutildll = $(openldapdir)\libraries\liblutil\$(objdir)\openlutil.dll
openlberdll = $(openldapdir)\libraries\liblber\$(objdir)\openlber.dll
openldapdll = $(openldapdir)\libraries\libldap\$(objdir)\openldap.dll
openldap_rdll = $(openldapdir)\libraries\libldap_r\$(objdir)\openldap_r.dll
openldbmdll = $(openldapdir)\libraries\libldbm\$(objdir)\openldbm.dll
openldifdll = $(openldapdir)\libraries\libldif\$(objdir)\openldif.dll
openlunicodedll = $(openldapdir)\libraries\liblunicode\$(objdir)\openlunicode.dll
DEFS = $(DEFS) -DOPENLDAP_DLL
PATH = $(openldapdir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(openldapdir)\include
openlavl = $(openlavllib) $(openlavldll)
openlutil = $(openlutillib) $(openlutildll)
openlber = $(openlberlib) $(openlberdll)
openldap = $(openldaplib) $(openldapdll)
openldap_r = $(openldap_rlib) $(openldap_rdll)
openldbm = $(openldbmlib) $(openldbmdll)
openldif = $(openldiflib) $(openldifdll)
openlunicode = $(openlunicodelib) $(openlunicodedll)
!endif

#
# The compface library
#
!ifdef USE_COMPFACE
compfacedir=$(gnuroot_srcdir)\compface$(compfacever)
compfacelib = $(compfacedir)\$(objdir)\libcompface.lib
!ifdef COMPFACE_DLL
compfacedll = $(compfacedir)\$(objdir)\libcompface.dll
DEFS = $(DEFS) -DCOMPFACE_DLL
PATH = $(compfacedir)\$(objdir);$(PATH)
!endif
DEFS = $(DEFS) -I$(compfacedir)
compface = $(compfacelib) $(compfacedll)
!endif

#
# The TeX engines
#
!ifdef USE_TEX
!ifdef TEX_DLL
DEFS = $(DEFS) -DTEX_DLL
!endif
!endif

#
# Setargv expands arguments
# Unfortunately, when linking with msvcrt.dll (as it is recommended under NT)
# it is not possible to provide his own setargv() routine. Globbing is hardcoded
# in the dll and triggered only if you link in the dummy MS setargv.obj
#
setargvobj = setargv.obj

# End of libsdef.mak
#
# Local Variables:
# mode: Makefile
# End:
