package RDFTpm;
#use strict;
use Carp;
use XML::DOM;
use Exporter ();
  
@ISA = qw( XML::DOM::Node, Exporter );

@EXPORT_OK = qw (new TPMcreateAbout TPMaddNode TPMcreateBag TPMaddBag TPMcreateFiles
		TPMcreateProvides TPMcreateRequires TPMcreateRequiredBy
		TPMcreateInstallation);

use vars qw( @ISA );

sub new {
  my ($class) = @_;
  my $parser = new XML::DOM::Parser;
  my $doc = $parser->parse("<?xml version=\"1.0\"?>\n\
<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns\#\"\n\
 xmlns:TPM=\"http://texlive.dante.de/\">\n");
  return $doc;

}

# Specific TPM descriptions
# Build the about node
sub TPMcreateAbout {
  my ($self, $about, $href) = @_;
  local $tpmdesc = $self->createElement("rdf:Description");
  local $tpmabout = $self->createAttribute("about", $about);
  local $tpmhref = $self->createAttribute("href", $href);
  $tpmdesc->setAttributeNode($tpmabout);
  $tpmdesc->setAttributeNode($tpmhref);
  $self->getElementsByTagName("rdf:RDF")->item(0)->appendChild($tpmdesc);
}

# Add a node to the about node
sub TPMaddNode {
  my ($self, $node, $value, $attr) = @_;
  local $tpmnode = $self->createElement("TPM:$node");
  local $nl = $self->getElementsByTagName("rdf:Description");
  local $key;
  if ($value) {
    local $val = $self->createTextNode($value);
    $tpmnode->appendChild($val);
  }
  foreach $key (keys %{$attr}) {
    local $attribute = $self->createAttribute($key, ${$attr}{$key});
    $tpmnode->setAttributeNode($attribute);
  }
  $nl->item($nl->getLength - 1)->appendChild($tpmnode);
}

# Create a list of provided resources
sub TPMcreateBag {
  my ($self, $bagname, @list) = @_;
  local $tpmnode = $self->createElement("TPM:$bagname");
  local $tpmbag = $self->createElement("RDF:Bag");
  map {
    my $n = $self->createElement("TPM:Resource");
    my $a = $self->createAttribute(href, $_);
    my $e = $_;
    print "TPM:Resource = $_\n";
    $e =~ s@/([^/\\]*)\.tpm$@\1@;
    $e = $self->createTextNode($e);
    $n->setAttributeNode($a);
    $n->appendChild($e);
    $tpmbag->appendChild($n);
  } @list;
  $tpmnode->appendChild($tpmbag);
  local $nl = $self->getElementsByTagName("rdf:Description");
  $nl->item($nl->getLength - 1)->appendChild($tpmnode);
}

# sub TPMaddBag {
#   my ($self, $bagname, $value, $href) = @_;
#   local $tpmnode = $self->getElementsByTagName("TPM:$bagname")->item(0);
#   $tpmnode = $tpmnode->getElementsByTagName("RDF:Bag");
#   local $att_href = $self->createAttribute("href", $href);
#   $tpmnode->setAttributeNode($att_href);

# }

# Create the list of files in the package
sub TPMcreateFiles {
  my ($self, $filetype, $attr, @files) = @_;
  my $strfiles = "";
  local $tpmnode = $self->createElement("TPM:$filetype");
  foreach $k (keys %{$attr}) {
    my $tpmattr = $self->createAttribute("$k", "${$attr}{$k}");
    $tpmnode->setAttributeNode($tpmattr);
  }
  $strfiles = join "\n", @files;
  local $textnode = $self->createTextNode("$strfiles\n");
  $tpmnode->appendChild($textnode);
  local $nl = $self->getElementsByTagName("rdf:Description");
  $nl->item($nl->getLength - 1)->appendChild($tpmnode);
}

sub TPMcreateProvides {
  my ($self, $name) = @_;
  my $tpmnode = $self->createElement("TPM:Provides");
  my $a = $self->createTextNode("$name");
  $tpmnode->appendChild($a);
  local $nl = $self->getElementsByTagName("rdf:Description");
  $nl->item($nl->getLength - 1)->appendChild($tpmnode);
}

sub TPMcreateRequires {
  my ($self, $name, %list) = @_;
  my $tpmnode = $self->createElement("TPM:Requires");
  my $k;
  foreach $k (keys %list) {
    my @taglist = @{$list{$k}};
#    for (my $i = 0; $i <= $#taglist; $i++) {
#      my $tag = $taglist[$i];
#    print "taglist " . $#taglist . "\n";
    for my $tag (@taglist) {
      my $tpmbin = $self->createElement("TPM:" . ucfirst($k));
      my $a = $self->createAttribute("name", $tag);
      $tpmbin->setAttributeNode($a);
      $tpmnode->appendChild($tpmbin);
    }
  }
  local $nl = $self->getElementsByTagName("rdf:Description");
  $nl->item($nl->getLength - 1)->appendChild($tpmnode);
}

sub TPMcreateRequiredBy {
  my ($self, $name) = @_;
  my $tpmnode = $self->createElement("TPM:RequiredBy");
  local $nl = $self->getElementsByTagName("rdf:Description");
  $nl->item($nl->getLength - 1)->appendChild($tpmnode);
}

sub TPMcreateInstallation {
  my ($self, @installation) = @_;
  my $tpmnode = $self->createElement("TPM:Installation");
  for(my $i = 0 ; $i <= $#installation; $i++) {
    my $tpmexec = $self->createElement("TPM:Execute");
    my $attr = $self->createAttribute("function", ${$installation[$i]}{"function"});
    $tpmexec->setAttributeNode($attr);
    print "kfunc = ". ${$installation[$i]}{"function"} . "\n" if ($opt_debug);
    foreach my $kparam (keys %{$installation[$i]}) {
      print "kparam = $kparam\n" if ($opt_debug);
      if ($kparam ne "function") {
	$attr = $self->createAttribute($kparam, ${$installation[$i]}{$kparam});
        $tpmexec->setAttributeNode($attr);
      }
    }
    $tpmnode->appendChild($tpmexec);
  }
  local $nl = $self->getElementsByTagName("rdf:Description");
  $nl->item($nl->getLength - 1)->appendChild($tpmnode);
}

1;

