#
# Build RDF files for teTeX/fpTeX/TeXLive
# Author: F. Popineau
# Date  : 19/12/99
# Time-stamp: <02/03/02 23:37:44 popineau>
#
# Usage: 
#        build-lists
#

require "newgetopt.pl";
use lib "../perl";
use file_utils qw(canon_dir cleandir make_link newpath 
		  walk_dir diff_list
		  normalize substitute_var_val dirname);
use Cwd;
use XML::DOM;
use tpm;
# use diagnostics;

$opt_texdir = "d:/Source/TeXLive/Master";
$opt_zip = 0;
$opt_check=0;

$rdf_template = <<END;
<?xml version="1.0" ?> 
 <RDF:RDF xmlns:RDF="http://www.w3.org/TR/WD-rdf-syntax#" xmlns:TPM="http://texlive.dante.de/">
 </RDF:RDF>
END

@arch_list = (
#	      "main",
	      "data",
	      "alphaev5-osf4.0d",
	      "hppa1.1-hpux10.10",
	      "i386-linux",
	      "i386-linux-libc5",
	      "mips-irix6.2",
	      "resource",
	      "rs6000-aix4.1.4.0",
	      "sparc-solaris2.5.1",
	      "win32"
	     );

my $parser;

&main;

exit 0;



sub globexpand_push {
  my ($dir, @l) = @_;
  my ($file);
  $dir =~ s@\\@/@g;
  foreach $file (@l) { 
    if (-f "$dir/$file") {
      push @listglob, "$dir/$file"; 
    }
  } 
}

sub globexpand {
  my ($recurse, $pattern) = @_;
  local @listglob = ( );
  while ( <${pattern}> ) {
    print "elt $_\n" if $opt_verbose;
    if (-f $_) {
      push @listglob, $_;
    }
    elsif ((-d $_) && ($recurse)) {
      &walk_dir($_, \&globexpand_push);
    }
  }
  print "list = $pattern => " . @listglob . "\n" if $opt_verbose;
  return @listglob;
}

sub printdate {
  my @time = gmtime;
  return $time[5] . $time[4]+1 . $time[3] . $time[2];
}

sub process_bin {
  my ($arch, $os, @files) = @_;

  @files = map { 
    $_ =~ s@\$ARCH@$arch-$os@;
    if ($os ne "win32") {
      $_ =~ s@^(.*)\.[^\./\\]*$@$1@;
    }
    &globexpand (1, $_);
  } @files;
  print @files if $opt_verbose;
  return @files;
}

sub process_data {
  my (@files) = @_;
  @files = map { 
    &globexpand (1, $_);
  } @files;
  print @files if $opt_verbose;
  return @files;
}

#
#
#
# Get the fptex pkg description, returns a structured description using RDF format.
# Needs to generate several RDF structures for each architecture supported by the
# TeXLive. Add one RDF structure for the architecture independent files. 
# Will compute file sizes.
# 
sub read_package {
  my ($pkgname, $doc) = @_;
  my ($name, $name, $version, $level);
  my ($description, $zipfile, $reqlist);
  my ($a);
  local ($binfiles, $datafiles);

  print "Package $pkgname\n" if $opt_debug;

  @datafiles = ();
  @binfiles = ();
  open IN, "<fptex/$pkgname.pkg";
  while (<IN>) {
    # skip comments
    if (! defined $_) { next; }
    if ($_ =~ m@^\s*(\#.*)?$@) { next; }
    if ($_ =~ m@^\[description\]@) {
      &read_description;
      redo;
    }
    elsif ($_ =~ m@^\[bin\]@) {
      &read_bin;
      redo;
    }
    elsif ($_ =~ m@^\[data\]@) {
      &read_data;
      redo;
    }
    elsif ($_ =~ m@^\[required\]@) {
      &read_required;
      redo;
    }
    else {
      print STDERR "Unmatched line: $_\n";
    }
  }
  close(IN);
  
  $cwd = &getcwd;
  chdir "$opt_texdir";
  # Build the main package description
  foreach $a (@arch_list) {
#    if ($a =~ m@^main$@) { next; }
    $a =~ m@^([^-]*)-(.*)$@;
    $arch = $1;
    $os = $2;
    $doc{'main'}->Tpm::TPMcreateAbout("http://texlive.dante.de/texlive/$a/$name.tgz", "../$a/$name.tpm");
    $doc{'main'}->Tpm::TPMaddNode("Name", $name);
    $doc{'main'}->Tpm::TPMaddNode("Version", $version);
    $doc{'main'}->Tpm::TPMaddNode("Release", $release);
    $doc{'main'}->Tpm::TPMaddNode("Arch", $arch);
    $doc{'main'}->Tpm::TPMaddNode("Os", $os);
    $doc{'main'}->Tpm::TPMaddNode("Distribution", $texlive);
    $doc{'main'}->Tpm::TPMaddNode("Vendor", "TeX Users Group");
    $doc{'main'}->Tpm::TPMaddNode("Date", &printdate);
    $doc{'main'}->Tpm::TPMaddNode("Creator", "popineau");
  }

  # Build each architecture specific RDF description
  foreach $a (@arch_list) {
    if ($a =~ m@^main$@) { next; }
    $a =~ m@^([^-]*)-(.*)$@;
    $arch = $1;
    $os = $2;
    $doc{$a}->Tpm::TPMcreateAbout("http://texlive.dante.de/texlive/$a/$name.tgz");
    $doc{$a}->Tpm::TPMaddNode("Name", $name);
    $doc{$a}->Tpm::TPMaddNode("Version", $version);
    $doc{$a}->Tpm::TPMaddNode("Release", $release);
    $doc{$a}->Tpm::TPMaddNode("Arch", $arch); 
    $doc{$a}->Tpm::TPMaddNode("Os", $os);
    $doc{$a}->Tpm::TPMaddNode("Distribution", $texlive);
    $doc{$a}->Tpm::TPMaddNode("Vendor", "TeX Users Group");
    $doc{$a}->Tpm::TPMaddNode("Summary");
    $doc{$a}->Tpm::TPMaddNode("Description");
    $doc{$a}->Tpm::TPMaddNode("Copyright");
    $doc{$a}->Tpm::TPMaddNode("Sources");
    $doc{$a}->Tpm::TPMaddNode("BuildDate");
    $doc{$a}->Tpm::TPMaddNode("BuildHost");
    $doc{$a}->Tpm::TPMaddNode("Date", &printdate);
    $doc{'main'}->Tpm::TPMaddNode("Creator", "popineau");
    $doc{$a}->Tpm::TPMaddNode("Size");
    $doc{$a}->Tpm::TPMcreateBag("Provides");
    $doc{$a}->Tpm::TPMaddBag("Provides", "$pkgname", "../resources/$pkgname.rdf");
    if ($a eq 'noarch') {
      $doc{$a}->Tpm::TPMaddBag("Provides", "$pkgname-data", "../resources/$pkgname-data.rdf");
    }
    else {
      $doc{$a}->Tpm::TPMaddBag("Provides", "$pkgname-bin", "../resources/$pkgname-bin.rdf");
    }
    if ($#reqlist > -1) {
      $doc{$a}->Tpm::TPMcreateBag("Requires");
      map {
	$doc{$a}->Tpm::TPMaddBag("Requires", "../resources/$_.rdf");
      } @reqlist;
    }
    if ($a eq 'noarch') {
#      if ($#datafiles > -1) {
	$doc{$a}->Tpm::TPMcreateFiles(&process_data(@datafiles));
#      }
    }
    else {
#      if ($#binfiles > -1) {
	$doc{$a}->Tpm::TPMcreateFiles(&process_bin($arch, $os, @binfiles));
#      }
    }
    print $doc{$a}->toString if $opt_verbose;
  }
  chdir $cwd;
}

sub read_description {
  while(<IN>) {
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^([^=]+)=(.*)\s*$@) {
      ${$1} = $2;
    }
  }
}

sub read_bin {
  while(<IN>) {
    last if ($_ =~ m@^\[@);
    if ($_ =~ m@^file=(.*)\s*$@) {
      push @binfiles, $1;
    }
  }
}

sub read_data {
  while(<IN>) {
    last if ($_ =~ m@^\[@);
    if ($_ =~ m@^file=(.*)\s*$@) {
      push @datafiles, $1;
    }
  }
}

sub read_required {
  $#reqlist = -1;
  while(<IN>) {
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^package=(.*)\s*$@) {
      push @reqlist, $1;
    }
  }
}

#
# Try to enhance the RDF structure using the Catalogue.
# Get the license information, version and release numbers
#
sub complete_using_catalogue {
  my ($pkgname, $doc) = @_;
  my ($catname, $catdoc);
  local($author, $version, $license, $caption, $description);

  $catname = "catalogue/entries/${pkgname}.xml"; 
  if (! -f $catname) {
    my ($candidates);
    @candidates = <catalogue/entries/${pkgname}*.xml>;
    if (($#candidates > 0) || ($#candidates == -1)) {
      print "Warning, package $pkgname, candidates : ";
      foreach $c (<catalogue/entries/${pkgname}*.xml>) {
        print "$c ";
      }
      print "\n";
      return;
    }
    $catname = $candidates[0];
    print "Found only 1 candidate : $catname\n";
  }

  $catdoc = $parser->parsefile($catname);
  $author = $catdoc->getElementsByTagName("author");
  $version = $catdoc->getElementsByTagName("version");
  $license = $catdoc->getElementsByTagName("license");
  $caption = $catdoc->getElementsByTagName("caption");
  $description = $catdoc->getElementsByTagName("description");
  
}

#
# Build zip files for fpTeX.
#
sub build_zip {
  my ($pkgname, $doc) = @_;
  local @files = ( );
  local $cwd = &getcwd;

  $zipname = "$cwd/zip/$pkgname.zip";
  unlink ($zipname) if (-f $zipname);
  $cmd = "zip -9or $zipname ";

  chdir "d:/Local/TeX";

  push @files, $doc{'win32'}->Tpm::TPMgetFiles;
  push @files, $doc{'noarch'}->Tpm::TPMgetFiles;
  print "Run: " . $cmd . join(" ", @files) . "\n";
  system ($cmd . join(" ", @files));
  chdir $cwd;
}

#
# writes the RDF file(s)
#
sub write_rdf {
  my ($pkgname, $doc) = @_;
  if (! -d "TPMS/resources") {
    mkdir "TPMS/resources", 666;
  }
  $doc{'main'}->printToFile("TPMS/resources/$pkgname.rdf");
  foreach $a (@arch_list) {
    if (! -d "TPMS/$a") {
      mkdir "TPMS/$a", 666;
    }
    $doc{$a}->printToFile("TPMS/$a/$pkgname.rdf");
  }
}

sub help
{
    print "Usage: build [options] [pkg_1] ... [pkg_n]\n";
    print "\n";
    print "    --check          Try to diff list of installed files against list\n";
    print "                     of files in packages.\n";
    print "    --texdir <path>  TeX directory where things have been installed.\n";
    print "    --zip            Build zip files.\n";
    print "\n";		       
    print "By default, all packages in fptex/*.pkg are build or checked.\n";
    print "If packages are specified on the command line, only those ones\n";
    print "are processed.\n";
    print "\n";
    print "    Support commands:\n";
    print "    --debug          Build debug versions;\n";
    print "    --help           displays this help text and exits\n";
    print "    --verbose        some more messages\n";
}

sub main {

  my($doc, $list_pkg);
  my($checkl, $truefiles, $res);

  unless (&NGetOpt ("texdir=s", "check", "zip",
		    "debug", "verbose", "help")) {
    print STDERR "Try `configure --help'";
    exit 1;
  }

  if ($opt_help) {
    &help;
    exit 0;
  }

  @list_pkg = @ARGV;

  if ($#list_pkg == -1) {
    @list_pkg = map { s@fptex/(.*)\.pkg@\1@; $_; } <fptex/*.pkg>;
  }

  foreach $pkgname (@list_pkg) {
    $rdfname = "rdf/$pkgname.rdf";
    
    $parser = new XML::DOM::Parser;

    $doc{'main'} = $parser->parse($rdf_template);
    foreach $a (@arch_list) {
      $doc{$a} = $parser->parse($rdf_template);
    }
    # Read the current package
    $doc = read_package($pkgname, $doc);

    # Try to find information from the catalogue
    $doc = complete_using_catalogue($pkgname, $doc);

    if ($opt_check) {
      push @checkl, $doc{'win32'}->Tpm::TPMgetFiles;
      push @checkl, $doc{'noarch'}->Tpm::TPMgetFiles;
    }
    
    if ($opt_zip) {
      # Build zip file
      build_zip($pkgname, $doc);
    }

    # Write RDF data
    write_rdf($pkgname, $doc);
  }

  if ($opt_check) {
    chdir ("$opt_texdir");
    push @truefiles, <bin/win32/*>;
    push @truefiles, &globexpand(1, "texmf/doc/*");
    push @truefiles, <info/*>;
    push @truefiles, &globexpand(0, "include/win32/*");
    push @truefiles, <lib/win32/*>;
    push @truefiles, &globexpand(1, "man/*");
    push @truefiles, <texmf/web2c/*>;
    push @truefiles, <texmf/tex/plain/dvips/*>;
    push @truefiles, <texmf/tex/latex/tex4ht/*>;
    push @truefiles, &globexpand(1, "texmf/tex4ht/*");
    push @truefiles, <texmf/ispell/*>;
    push @truefiles, <texmf/dvips/base/*>;
    push @truefiles, <texmf/dvips/gfstopk/*>;
    push @truefiles, <texmf/dvips/psutils/*>;
    push @truefiles, <texmf/ltx2rtf/*>;
    push @truefiles, <texmf/metapost/base/*>;
    push @truefiles, <texmf/metapost/config/*>;
    push @truefiles, <texmf/hbf2gf/*>;
    push @truefiles, <texmf/fontname/*>;
    
    print "$#truefiles, $#checkl\n";
    local (@l1, @l2);
    &diff_list (\@truefiles, \@checkl, \@l1, \@l2);
  }
}
