#
# Diff tool for Web2C/teTeX/TeXLive vs fpTeX
# Author: F. Popineau
# Date  : 22/12/99
# Time-stamp: <02/04/27 03:05:38 popineau>
#
# Motivation : the GNU diff command is clever, but can't exclude
# files based on path matching, but only on filename part matching.
# Moreover, this script automates the creation of diff files, package
# by package.
#
require "newgetopt.pl";
# use diagnostics;
use lib "./perl";
use file_utils qw(member);

use Cwd;

$opt_from = "d:/source/texlive/Build/source/TeX";

$opt_to = "d:/source/fptex";

$destdir = "$opt_to/win32/diff";

$opt_dist = "texlive";

%exclude = (
	    "web2c" => {
			"libs" => [ "expat", "geturl", "geturl_static", "gifreader", "jpeg",
				    "libttf", "libwww", "ncurses", "ncurses-ac", "regex", "T1",
				    "t1lib" ],
			"texk" => [ "bibtex8", "chktex", "cjkutils",  "dtl", "dvidvi", "dvipdfm",
				    "dvipsk", "gsftopk", "hbf2gf", "ispell", "kpathsea-fp", 
				    "kpathsea-web2c", "kpsexx", "lacheck", "latex2html",
				    "ltx2rtf", "ltx2rtf-old", "makeindexk", "odvipsk", 
				    "owindvi", "oxdvik", "ps2pkm", "tetex", "texdoctk", "texdoctk-3.0",
				    "musixflx", "ps2pk16beta1a", "seetexk", "tex4htk", "texlive",
				    "ttf2pfb", "ttf2pk", "ttfdump",
				    "tth", "win32", "windvi", "windvi-0.68", "xdvik" ]
#  			"texk.development" => [ "bibtex8", "dtl", "dvidvi", "dvipdfm", "lacheck",
#  						"musixflx", "ps2pk16beta1a", "seetexk", "tex4htk",
#  				   "tth", "win32", "windvi-0.68"]
			 },
	    "tetex" => { 
			"libs" => ["gifreader", "jpeg", "libttf", "regex", "T1", "contrib" ],
			"texk" => ["bibtex8", "dtl", "dvidvi", "dvipdfm", "lacheck",
				   "musixflx", "ps2pk16beta1a", "seetexk", "tex4htk",
				   "tth", "win32", "windvi-0.68"],
			"texk.development" => ["bibtex8", "dtl", "dvidvi", "dvipdfm", "lacheck",
				   "musixflx", "ps2pk16beta1a", "seetexk", "tex4htk",
				   "tth", "win32", "windvi-0.68"]
		       },
	    "texlive" => {
			  "libs" => ["gifreader", "jpeg", "libttf", "regex", "T1", "contrib" ],
			  "texk" => ["ps2pk16beta1a", "win32", "windvi-0.68", "kpathsea-shared",
				    "kpsexx"],
			  "win32" => ["factory", "worked-on", "ttfinst", "texconfig", "setupw32"],
			  "utils" => ["l2h"]
			 }
	   );

@win32 = ( 
	  "build.pl",
	  "ChangeLog",
	  "configure.pl",
	  "convert.bat",
	  "def",
	  "doc",
#	  "fpsetup",
	  "include",
	  "lib",
	  "lnexe",
#	  "lprgs",
	  "make",
#	  "pdftex-win32",
	  "perl",
	  "rc",
	  "sed",
	  "setvars.bat",
#	  "texconfig",
#	  "ttfinst",
#	  "various",
	  "win32.mak"
#	  "windvi-win32"
	 );

&main;

exit 0;

# What this script does...
sub help {
  print "Usage: diff-unix [options] pack_1 ... pack_n\n";
  print "\n";
  print "    --from <path>                 directory tree from which the diffs will occur.\n";      
  print "    --to <path>                   directory tree to which the diffs will occur.\n";
  print "    --dest <path>                 directory where the files will be stored.\n";
  print "    --dist [web2c|tetex|texlive]  directory where the files will be stored.\n";
  print "    --win32                       build an archive of relevant win32 files.\n";
  print "    The rest of the arguments are used to match the packages that will be\n";
  print "    diffed. Those that do not match together with those specified above for\n";
  print "    each distribution are excluded from the processing.\n";
  print "    Moreover, a win32.tar.gz archive of the relevant files in the win32\n";
  print "    directory can be built.\n";
  print "\n";
  print "    Support commands:\n";
  print "    --help                          displays this help text and exits\n";
}

sub main {

  unless (&NGetOpt ("from=s", "to=s", "dest=s", "dist=s", "win32",
		    "help", "verbose")) {
    print STDERR "Try `diff-unix --help'";
    exit 1;
  }
  # Give help, exit immediately.
  if ($opt_help) {
    help;
    exit 0;
  }

  if ($opt_from) {
    $from_srcdir = $opt_from;
  }
  print "from_srcdir=$from_srcdir\n";

  if (! -d $opt_from) {
    print STDERR "$from_srcdir is not a directory. Aborting ...\n";
    exit 1;
  }

  if ($opt_to) {
    $to_srcdir = $opt_to;
  }
  print "to_srcdir=$to_srcdir\n";

  if (! -d $to_srcdir) {
    print STDERR "$to_srcdir is not a directory. Aborting ...\n";
    exit 1;
  }

  if ($opt_dest) {
    $destdir = $opt_dest;
  }

  if (! -d $destdir) {
    print STDERR "$destdir is not a directory. Aborting ...\n";
    exit 1;
  }

  if ($opt_dist) {
    $dist = $opt_dist;
  }
  if ($dist !~ m/^(web2c|tetex|texlive)/) {
    print STDERR "$dist does not match one of the known distributions.\n";
    exit 1;
  }
  
  if (-f "$destdir/$dist.exclude") {
    $excl_arg = " -X $destdir/$dist.exclude ";
  }

#  print "test " . $exclude{"texlive"} . "\n";

  foreach $d (keys %{$exclude{$dist}}) {
    chdir ("$to_srcdir/$d");
    print "Entering $to_srcdir/$d\n";
    print "excluded : ${$exclude{$dist}}{$d}\n";
    foreach $sd (<*>) {
      next if (! -d $sd);
      next if (&member($sd, ${$exclude{$dist}}{$d}));
      if ($#ARGV >= 0 && !member("$d/$sd", @ARGV)) { next; }
#       if ($d eq "texk") {
# 	$dev = ".development";
#       }
#       else {
# 	$dev = "";
#       }
#       print "dev = $dev\n";
      $cmd = "diff -ruN -x *~ $excl_arg $from_srcdir/$d${dev}/$sd $sd > $destdir/$d-$sd.diff";
      print "$cmd\n";
      system($cmd);
    }
  }

  if ($opt_win32) {
    chdir ("$to_srcdir/texk/win32");
    print "Entering $to_srcdir/texk/win32\n";
    $cmd = "zip -9or $destdir/win32.zip ";
    $cmd .= join (" ", @win32);
    system("$cmd");
  }
    
  print "Done.\n";
}
