#
# Perl script to build patch file for TeX-Live
# Author  : F. Popineau
# Version : 1.00
# Date    : <07/11/1998>
# On one side, a directory with multiple .tgz files for TeXLive
# on the other side the directory where it is installed.
#

require "newgetopt.pl";
use file_utils qw(dirname basename normalize walk_dir member
		  build_tree print_tree walk_tree cleandir check_path);

# $verbose = 1;

@orig_files = ( );

@dest_files = ( );

&main;

exit 0;

sub usage
{
    print "Usage: $ARGV[0] [options]\n";
    print "\n";
    print "    --orig <path>                   source directory directory for TL .tgz files;\n";
    print "    --dest <path>                   destination directory with actual TL tree;\n";
    print "\n";
    print "    Support commands:\n";
    print "    --help                          displays this help text and exits;\n";
    print "    --verbose                       displays more information.\n";
}

sub dir_listing
{
    # Arguments to the process function
    local ($dir, @list) = @_;
    
    foreach $f (@list) {
      push (@listfiles, &normalize($dir . "\\" . $f)) if (-f $dir . "\\" . $f);
    }
}

sub main
{
  local ($dest, $level);
  local ($root, $f, $k);

  unless (&NGetOpt ("orig=s", "dest=s", "help", "verbose")) {
    print STDERR "Try `configure --help'";
    exit 1;
  }
 
  if ($opt_help) {
    usage();
    exit(0);
  }

  if (! -d $opt_orig) {
    print STDERR "$0: not a directory $opt_orig\n";
    exit 1;
  }

  if (! -d $opt_dest) {
    print STDERR "$0: not a directory $opt_dest\n";
    exit 1;
  }
  $opt_reg = "^" . quotemeta($opt_dest) . "\\\\";

  # list original files
  while (<$opt_orig/*.tgz>) {
    &read_tgz(\%orig_files, $_);
  }
  print "Read all .tgz files.\n";

  # list files in actual ditribution
  &walk_dir($opt_dest, \&list_files);

  # compare both
  print "Those files are in original tree, but do not exist or differ in destination tree:\n";
  foreach $f (keys %orig_files) {
    if (! exists $dest_files{$f}) {
      print "$f does not exist\n";
    }
    elsif ($orig_files{$f}->[1] != $dest_files{$f}->[1]) {
      print "$f has not same size : " . $orig_files{$f}->[1] . " and " . $dest_files{$f}->[1] . "\n";
    }
  }
  
  print "Those files are in destination tree, but do not exist or differ in source tree:\n";
  foreach $f (keys %dest_files) {
    if (! exists $orig_files{$f}) {
      print "$f does not exist\n";
    }
    elsif ($dest_files{$f}->[1] != $orig_files{$f}->[1]) {
      print "$f has not same size : " . $orig_files{$f}->[1] . "(orig) and " . $dest_files{$f}->[1] . "(dest)\n";
    }
  }
  
}

sub read_tgz
{
  local (*list, $name) = @_;
  local (@l);
  print "opening $name ...\n";
  open IN, "gzip -dc $name | tar tvf - |";
  while (<IN>) {
    chomp;
    @l = split(" ", $_);
    $list{$l[7]} = [$l[7], $l[2]];
#    print "Name $l[7], size $l[2]\n";
  }
  close IN;
}

sub list_files
{
  local ($dir, @l) = @_;
  local ($f, $d, $reg, $size, @s);

# No directories ?
#  $dest_files{$dir} = [ $dir, 0 ];

  foreach $f (@l) {
    next if ($f =~ m/~$/);
    $name = &normalize("$dir\\$f");
    @s = stat($name);
    $size = $s[7];
    $name =~ s/$opt_reg//;
#    print "name = $name, $size\n";
    $dest_files{$name} = [ $name, $size ];
  }
}
