#
# TPM Build tool for TeXLive
# Author: F. Popineau
# Date  : 27/03/2001
# Time-stamp: <02/04/09 11:38:12 popineau>
#
# Usage: 
#        tpm-build <option>
#		(--tpm2pkg|--pkg2tpm|--tpm_import)
#		--pkg_dir=<directory>
#		--tpm_dir=<directory>
#		(--all|--name=<package>)	
#		--log=logfile
#		--debug
#		--verbose
#

require "newgetopt.pl";
use lib ("./perl", "../perl");
use file_utils qw(canon_dir cleandir make_link newpath 
		  normalize substitute_var_val dirname
		  rec_rmdir sync_dir walk_dir start_redirection stop_redirection);
use tpm qw(Tpm::Pkg2Tpm Tpm::Pkg2Zip Tpm::Tpm2Zip 
	   Tpm::Tpm2Pkg Tpm::OldTpm2Pkg Tpm::extract_remote);
use Cwd;

use Strict;

$direction = 'pkg2tpm';
$opt_pkg_dir = "./factory/pkg";
$opt_tpm_dir = "./factory/texmf/tpm";
$opt_texdir = "d:/source/TeXLive/Master";
$opt_import_tpm_dir = "$opt_texdir/texmf/tpm";
#$opt_debug = 0;
$opt_verbose = 0;

%srcdst = (
	   "pkg-scm" => "schemes",
	   "pkg-col" => "collections",
	   "pkg-dat" => "packages",
	   "pkg-bin" => "binaries",
	   "pkg-sup" => "support"
	  );

&main;

1;

sub help {
    print "Usage: ${ARGV[0]} [options]\n";
    print "\n";
    print "    --tpm2pkg               converts the factory tpm files to pkg files;\n";
    print "    --pkg2tpm               converts the factory pkg files to tpm files;\n";
    print "    --tpm_import            imports old tpm files to factory pkg files;\n";
    print "    These targets are exclusive !\n";
    print "\n";		       
    print "    Directories :\n";
    print "    --pkg_dir=<path>        directory for pkg files;\n";
    print "    --tpm_dir=<path>        directory for tpm files;\n";
    print "    --import_tpm_dir=<path> directory for old tpm files;\n";
    print "\n";		       
    print "    Packages :\n";
    print "    --name=<pkg name>       process only the named package file;\n";
    print "    --all                   process all package files;\n";
    print "\n";
    print "    Support commands:\n";
    print "    --log=<file name>       log activity to file;\n";
    print "    --debug                 turn on debug trace;\n";
    print "    --verbose               some more messages\n";
    print "    --help                  displays this help text and exits\n";
}

sub main {

  unless (&NGetOpt ("tpm2pkg", "pkg2tpm", "tpm_import",
		    "pkg_dir=s", "tpm_dir=s", "texdir=s",
		    "old_tpm_dir=s", "name=s",
		    "all", "log=s",
		    "verbose", "debug", "help"))
    {
	print STDERR "Try `$0 --help'";
	exit 1;
    }

  if ($opt_help) {
    &help;
    exit 0;
  }

  if (($opt_tpm2pkg && $opt_pkg2tpm)
     || ($opt_tpm2pkg && $opt_tpm_import)
     || ($opt_pkg2tpm && $opt_tpm_import)) {
    print STDERR "$0: `--tpm2pkg', `--pkg2tpm' and `--tpm_import' are incompatible.\n";
    exit 1;
  }

  if ($opt_name && $opt_all) {
    print STDERR "$0: `--all' and `--name' are incompatible.\n";
    exit 1;
  }

  if ($opt_log) {
#    $logfile = "$topdir\\win32\\$opt_log";
    if (! $opt_log =~ /\.log$/) {
      $opt_log .= ".log";
    }
    # start redirection if asked
    &start_redirection($opt_log);
 }

  if ($opt_tpm_import) {
    my $pkgname = "";
    if ($opt_name) {
      $opt_name =~ s@\.tpm$@@;
      foreach $src_subdir (keys %srcdst) {
	print STDERR "source subdirectory = $src_subdir\n";
	my $dst_subdir = $srcdst{$src_subdir};
	if (-f "$opt_import_tpm_dir/$dst_subdir/$opt_name.tpm") {
	  $pkgname = "$opt_name.pkg";
	  Tpm::OldTpm2Pkg("$opt_import_tpm_dir/$dst_subdir/$opt_name.tpm",
			  $pkgname, $opt_texdir, "$opt_pkg_dir/$src_subdir");
	  last;
	}
      }
      if ($pkgname eq "") {
	print STDERR "$0: $opt_name.tpm not found !\n";
	$retcode = 1;
      }
    }
    elsif ($opt_all) {
      foreach $src_subdir (keys %srcdst) {
	my $dst_subdir = $srcdst{$src_subdir};
	while (<$opt_import_tpm_dir/$dst_subdir/*.tpm>) {
	  $pkgname = $_;
	  $pkgname =~ s@^.*/([^/]*)\.tpm$@\1.pkg@;
	  print "processing $_\n\t-> $src_subdir/$pkgname\n";
	  Tpm::OldTpm2Pkg($_, $pkgname, $opt_texdir, "$opt_pkg_dir/$src_subdir");
	}
      }
    }
  }

  elsif ($opt_pkg2tpm) {
    my $tpmname = "";
    if ($opt_name) {
      $opt_name =~ s@^.*[/\\]([^\//]*)\.pkg$@\1@;
      foreach $subdir (keys %srcdst) {
	if (-f "$opt_pkg_dir/$subdir/$opt_name.pkg") {
	  $tpmname = "$opt_name.tpm";
	  Tpm::Pkg2Tpm($tpmname, "$opt_pkg_dir/$subdir/$opt_name.pkg", 
		       $opt_texdir, "$opt_tpm_dir/" . $srcdst{$subdir});
	  last;
	}
      }
      if ($tpmname eq "") {
	print STDERR "$0: $opt_name.pkg not found !\n";
	$retcode = 1;
      }
    }
    elsif ($opt_all) {
      foreach $subdir (keys %srcdst) {
	while (<$opt_pkg_dir/$subdir/*.pkg>) {
	  $tpmname = $_;
	  my $destdir = $srcdst{$subdir};
	  print "processing $_ -> $destdir/$tpmname\n";
#	  next if (/-static/);
	  $tpmname =~ s@^.*/([^/]*)\.pkg$@\1.tpm@;
	  Tpm::Pkg2Tpm( $tpmname, $_, $opt_texdir, "$opt_tpm_dir/$destdir");
	  open IN, "<$opt_tpm_dir/$destdir/$tpmname";
	  my $s = "";
	  while (<IN>) {
	    next if ($_ =~ /^\s*$/);
	    $s .= $_;
	  }
	  close IN;
	  open OUT, ">$opt_tpm_dir/$destdir/$tpmname.new";
	  $s =~ s@(\s*)<(TPM:[^>]*)>\n([^\n]*)\n\s*</\2>@\1<\2>\3</\2>@gms;
	  print OUT $s;
	  close OUT;
	  rename ("$opt_tpm_dir/$destdir/$tpmname.new", "$opt_tpm_dir/$destdir/$tpmname");
	}
      }
    }
  }

  elsif ($opt_tpm2pkg) {
    if ($opt_name) {

    }
    elsif ($opt_all) {
      while (<$opt_pkg_dir/data/*.tpm>) {
      }
    }
  }

  if ($opt_log) {
    &stop_redirection($opt_log)
  }
}
