#
# TPM Build tool for TeXLive
# Author: F. Popineau
# Date  : 27/03/2001
# Time-stamp: <02/05/16 23:16:04 popineau>
#

require "newgetopt.pl";
use lib ("./perl", "../perl");
use file_utils qw(canon_dir cleandir make_link newpath member
		  normalize substitute_var_val dirname diff_list
		  rec_rmdir sync_dir walk_dir rec_mkdir
		  start_redirection stop_redirection);
use tpm qw(Tpm::Pkg2Tpm Tpm::Pkg2Zip Tpm::Tpm2Zip 
	   Tpm::OldTpm2Pkg Tpm::extract_remote);
use Cwd;

use Strict;

$direction = 'pkg2tpm';
$opt_check = 0;
$opt_pkg_dir = "./factory/pkg";
$opt_tpm_dir = "./factory/texmf/tpm";
$opt_tex_dir = "${Tpm::masterdir}";
$opt_dest_dir = "${Tpm::ftpdir}/${Tpm::fptexversion}";
$opt_import_tpm_dir = "$opt_tex_dir/texmf/tpm";
$opt_arch = "win32";
$opt_log = 0;
$opt_debug = 0;
$opt_verbose = 0;

$opt_tpm_srcdir = "$opt_tex_dir/texmf/tpm";
$opt_tpm_dstdir = "./factory/original/tpm";

%srcdst = (
	   "pkg-scm" => "schemes",
	   "pkg-col" => "collections",
	   "pkg-dat" => "packages",
	   "pkg-bin" => "binaries",
	   "pkg-sup" => "support"
	  );

&main;

1;

sub main {

  unless (&NGetOpt ("tpm_srcdir=s", "tpm_dstdir=s", "dry",
		    "verbose", "debug", "help")) {
    print STDERR "Try `$0 --help'";
    exit 1;
  }
  &rec_mkdir($opt_tpm_dstdir) if (! -d $opt_tpm_dstdir);
  # read all files from tpmdir1 and rewrite them to tpmdir2
  my $parser = new XML::DOM::Parser;
  for my $subdir (values %srcdst) {
    next if (! -d "$opt_tpm_srcdir/$subdir");
    my $dstdir = "$opt_tpm_dstdir/$subdir";
    mkdir $dstdir if (! -d $dstdir);
    while (<$opt_tpm_srcdir/$subdir/*.tpm>) {
      $_ =~ s@([^/]*)$@\1@;
      my $name = $1;
      my $doc = $parser->parsefile($_);
      my $dstfilename = "$opt_tpm_dstdir/$subdir/$name";
      &rec_mkdir("$opt_tpm_dstdir/$subdir") if (! -d "$opt_tpm_dstdir/$subdir");
      print "processing $dstfilename\n";
      my $str = $doc->toString;
#      $str =~ s/\n\n*/\n/gio;
      if (! $opt_dry) {
	open OUT, ">$dstfilename";
	print OUT $str;
	close OUT;
      }
    }
  }

}
