package Tpm;

BEGIN {

# $Exporter::Verbose = 1;
use Exporter ();
use vars qw( @ISA @EXPORT_OK);
@ISA = qw( Exporter );
@EXPORT_OK = qw ( 
		 &Pkg2Tpm 
		 &Pkg2Zip 
		 &Tpm2Zip 
		 &TpmGetFiles
		 &TpmGetRequires
		 &OldTpm2Pkg 
		 &extract_remote 
		 $masterdir
		 $ftpdir 
		 $fptexversion 
		 @TpmSubdirs
		 @arch_list
		);

}

use Carp;
use vars (@EXPORT_OK);

use XML::DOM;
use XML::DOM::RDFTpm qw(TPMcreateAbout TPMcreateInstallation);
use Cwd;
use Time::gmtime;
use file_utils qw(basename globexpand sort_uniq push_uniq diff_list 
		  is_absolute calc_file_size remove_list);

use Strict;

# use diagnostics;

$masterdir = "d:/source/TeXLive/Master";
$catalogue_dir = "${masterdir}/texmf/doc/html/catalogue";
$ftpdir = "d:/InetPub/ftp/fptex";
$fptexversion = "0.6";

@TpmSubdirs = ( 
	       "collections", 
#	       "binaries", 
	       "packages", 
	       "schemes", 
	       "support");

@arch_list = (
	      "alpha-linux",
	      "alphaev5-osf4.0d",
#	      "hppa2.0-hpux10.20",
#	      "i386-freebsd",
	      "i386-linux",
	      "i386-solaris2.8",
	      "mips-irix6.5",
	      "powerpc-aix4.3.3.0",
	      "powerpc-darwin5.3",
#	      "rs6000-aix4.2.1.0",
	      "sparc-solaris2.7",
#	      "sparc-solaris2.8",
	      "win32"
	     );


# FEATURE consists of
#  About one entity (.tgz file or .zip file)
#  Name, Version, Arch, OS, Distribution, Vendor, Date, Size
#
# PKG consists of :
#  About description
#  Attributes:
#    Name, Author, Version, Build Date,
#    Title, Description,
#    Creator, Arch, OS, Distribution, Vendor, 
#    TPM Date, TPM Creator
#  Provide feature
#
#  Requires features list
#  RequiredBy features list
#
#  Bin files
#  Data files
#  Source files
#  Documentation files
#
#$opt_debug = 1;
#$opt_verbose = 1;
#
# Print the current date
#

sub process_bin {
  my ($archos, @files) = @_;
  if ($archos =~ "win32") {
    $archos = "win32";
    @files = grep { /win32/ } @files;
  }
  else {
    @files = grep { $_ =~ m@/\${ARCH}/@ && $_ =~ s@/\${ARCH}/@/${archos}/@ } @files;
#      @files = map { 
#        #    print "expanding $_\n";
#        if ($_ !~ m@/win32/@) {
#  	$_ =~ s@\${ARCH}@${archos}@;
#        }
#        $_;
#  #        if ($archos ne "win32") {
#  #  	$_ =~ s@^(.*)\.[^\./\\]*$@$1@;
#  #        }
#        #    print "expanding $_\n";
#  #      &globexpand (1, $_);
#      } @files;
  #  print @files;			# if $opt_verbose;
  }
  return @files;
}

sub process_data {
  my (@files) = @_;
  @files = sort_uniq( map {
    &globexpand (0, $_);
  } @files);
  return @files;
}

sub compress_bin {
  my ($texdir, @files) = @_;
  my $str = join " ", @files;
  for my $arch (@arch_list) {
    next if ($arch =~ /win32/);
    $str =~ s@/${arch}/@/\${ARCH}/@gmo;
  }
  @files = split " ", $str;
  @files = sort_uniq(@files);
}

sub feature_list {
  my (@files) = @_;
  my %result = ();
#  print "files = $#files\n";
  map {
#    print "file = $_ ... ";
    if (-f "./texmf/tpm/binaries/$_.tpm"
	|| -f "./pkg/pkg-bin/$_.pkg") {
#      print " binary\n";
      $result{$_} =  "binaries";
    }
    elsif (-f "./texmf/tpm/packages/$_.tpm"
	|| -f "./pkg/pkg-data/$_.pkg") {
#      print " package\n";
      $result{$_} =  "package";
    }
    elsif (-f "./texmf/tpm/collections/$_.tpm"
	|| -f "./pkg/pkg-res/$_.pkg") {
#      print " collection\n";
      $result{$_} =  "collection";
    }
    elsif (-f "./texmf/tpm/support/$_.tpm"
	|| -f "./pkg/pkg-supp/$_.pkg") {
#      print " support\n";
      $result{$_} =  "support";
    }
    else {
#      print STDERR "No feature type found for $_!\n";
      $result{$_} = undef;
    }
  } @files;
  return %result;
}

#
#
#
# Get the fptex pkg description, returns a structured description using RDF format.
# Needs to generate several RDF structures for each architecture supported by the
# TeXLive. Add one RDF structure for the architecture independent files. 
# Will compute file sizes.
# 
sub Pkg2Tpm {
  my ($tpmname, $pkgname, $texdir, $destdir) = @_;
  local ($name, $date, $version, $level, $description, $creator);
  my ($zipfile);
  my ($a, $basename);
  local (@binfiles, @runfiles, @docfiles, @sourcefiles, @remotefiles,
	 @provlist, %reqlist, @reqbylist, %flags, @installation);

  $name = $pkgname;
  $name =~ s@^.*[/\\]([^/\\][^/\\]*)\.pkg@$1@;
  $basename = $name;

  print "Package $pkgname\n" if $opt_debug;

  ($author, $version, $license, $title, $description) = &complete_using_catalogue($pkgname);

  @runfiles = ();
  @binfiles = ();
  @docfiles = ();
  @sourcefiles = ();
  @remotefiles = ();
  @provlist = ();
  %reqlist = ();
  @reqbylist = ();
  @installation = ();
  %flags = ( );
  open IN, "<$pkgname";
  while (<IN>) {
    # skip comments
    next if (! defined $_);
    next if ($_ =~ m@^\s*(\#.*)?$@);

    if ($_ =~ m@^\[description\]@i) {
      &read_description;
      redo;
    } elsif ($_ =~ m@^\[flags\]@i) {
      &read_flags;
      redo;
#     } elsif ($_ =~ m@^\[title\]@) {
#     } elsif ($_ =~ m@^\[title\]@) {
#       &read_title;
#       redo;
    } elsif ($_ =~ m@^\[bin\]@i) {
      @binfiles = &read_file_list;
      redo;
    } elsif ($_ =~ m@^\[run\]@i) {
      @runfiles = &read_file_list;
      redo;
    } elsif ($_ =~ m@^\[source\]@i) {
      @sourcefiles = &read_file_list;
      redo;
    } elsif ($_ =~ m@^\[documentation\]@i) {
      @docfiles = &read_file_list;
      redo;
    } elsif ($_ =~ m@^\[remote\]@i) {
      @remotefiles = &read_file_list;
      redo;
    } elsif ($_ =~ m@^\[provides\]@i) {
      &read_provides;
      redo;
    } elsif ($_ =~ m@^\[requires\]@i) {
      &read_requires;
      redo;
    } elsif ($_ =~ m@^\[required-by\]@i) {
      &read_required_by;
      redo;
    } elsif ($_ =~ m@^\[installation\]@i) {
      &read_installation;
      redo;
    } else {
      print STDERR "Unmatched line: $_\n";
    }
  }
  close(IN);
  $name = lc($name);
  $cwd = &getcwd;
  print "Changing cwd for $texdir\n" if $opt_debug;
  chdir "$texdir";
  @runfiles = &process_data(@runfiles);
  @docfiles = &process_data(@docfiles);
  @sourcefiles = &process_data(@sourcefiles);
  my @tmpremotefiles = &process_data(@remotefiles);
  if ($#tmpremotefiles < 0) {
    my $cwd1 = &getcwd;
    print "Changing cwd for $texdir\n" if $opt_debug;
    chdir "$ftpdir/$fptexversion";
    @tmpremotefiles = &process_data(@remotefiles);
    chdir $cwd1;
  }
  @remotefiles = @tmpremotefiles;
  $newdate = 0;
  # $newdate = &max_date($newdate, @runfiles);
  $newdate = &max_date($newdate, @doc);
  $newdate = &max_date($newdate, @runfiles);
  chdir $cwd;
  # Build the resource package description
  my $parser = new XML::DOM::Parser;
  $doc{'resource'} = $parser->parse("<!DOCTYPE rdf:RDF\n\
  SYSTEM \"../tpm.dtd\">\n\
<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns\#\"\n\
 xmlns:TPM=\"http://texlive.dante.de/\">\n</rdf:RDF>\n");
  $doc{'resource'}->RDFTpm::TPMcreateAbout("http://texlive.dante.de/texlive/$basename.zip");
  $doc{'resource'}->RDFTpm::TPMaddNode("Name", $name);
  $doc{'resource'}->RDFTpm::TPMaddNode("Date", &printdate($date));
  $doc{'resource'}->RDFTpm::TPMaddNode("Version", $version);
  $creator = "popineau" if ($creator eq undef);
  $doc{'resource'}->RDFTpm::TPMaddNode("Creator", $creator);
  #    $doc{'resource'}->RDFTpm::TPMaddNode("Release", $release);
  $doc{'resource'}->RDFTpm::TPMaddNode("Title", $title);
  $doc{'resource'}->RDFTpm::TPMaddNode("Description", $description);
  map { $_ =~ m/(.*)=["]?([^="]*)["]?/;
	$flags{$1} = $2; } split(" \n\t", $flags);
  $doc{'resource'}->RDFTpm::TPMaddNode("Flags", "", \%flags);
  if ($#runfiles > -1) {
    $size = &calc_file_size($texdir, @runfiles);
    %attr = ( "size" => "$size" );
    $doc{'resource'}->RDFTpm::TPMcreateFiles("RunFiles", \%attr, @runfiles);
  }
  if ($#binfiles > -1) {
    foreach $a (@arch_list) {
      #    if ($a =~ m@^resource$@) { next; }
      $a =~ m@^([^-]*)-?(.*)$@;
      $arch = $1;
      $os = $2;
      if ($arch eq "win32") {
	$arch = "i586";
	$os = "win32";
      }
	  $cwd = &getcwd;
      chdir "$texdir";
      @archbinfiles = &process_bin("$arch-$os", @binfiles);
      $newdate = &max_date($newdate, @archbinfiles);
      chdir $cwd;
      next if ($#archbinfiles < 0);
      $size = &calc_file_size($texdir, @archbinfiles);
      %attr = ( "arch" => "$a", "size" => "$size" );
      $doc{'resource'}->RDFTpm::TPMcreateFiles("BinFiles", \%attr, @archbinfiles);
    }
  }
  my $nodes = $doc{'resource'}->getElementsByTagName("TPM:Date");
  if ($nodes->getLength > 0) {
    my $node = $nodes->item(0);
    # print "old date : " . $node->toString();
    my $oldChild = $node->getFirstChild;
    my $val = $doc{'resource'}->createTextNode(&formatdate(localtime($newdate)));
    $node->replaceChild($val, $oldChild);
    # print "new date : " . $node->toString();
  }
  if ($#docfiles > -1) {
    $size = &calc_file_size($texdir, @docfiles);
    %attr = ( "size" => "$size" );
    $doc{'resource'}->RDFTpm::TPMcreateFiles("DocFiles", \%attr, @docfiles);
  }
  if ($#sourcefiles > -1) {
    $size = &calc_file_size($texdir, @sourcefiles);
    %attr = ( "size" => "$size" );
    $doc{'resource'}->RDFTpm::TPMcreateFiles("SourceFiles", \%attr, @sourcefiles);
  }
  # FIXME : the remote files should depend on architecture.
  if ($#remotefiles > -1) {
    $size = &calc_remote_file_size($texdir, @remotefiles);
    %attr = ( "size" => "$size" );
    $doc{'resource'}->RDFTpm::TPMcreateFiles("RemoteFiles", \%attr, @remotefiles);
      }
  $doc{'resource'}->RDFTpm::TPMcreateProvides($basename);
  if (scalar(%reqlist) > -1) {
    $doc{'resource'}->RDFTpm::TPMcreateRequires($name, %reqlist);
  }
  if ($#installation > -1) {
    $doc{'resource'}->RDFTpm::TPMcreateInstallation(@installation);
  }
  my $str = $doc{'resource'}->toString;
  $str =~ s@\n\n*@\n@gio;
  open OUT, ">$destdir/$tpmname";
#  $doc{'resource'}->printToFileHandle(\*OUT);
  print OUT $str;
  close OUT;
  return;
}

sub calc_remote_file_size {
  my($dir, @files) = @_;
  my ($f, $size, $failure);
  $size = 0;
  map {
    $failure = 0;
    $f = $_;
    print STDERR "calc_remote_file_size $f\n"; # if $opt_debug;
    if (! &is_absolute($f)) {
      $f = "$dir/$_";
      if (! -f $f) {
	$f = "$ftpdir/$fptexversion/$_";
	if (! -f $f) {
	  printf STDERR "$0: file $_ not found!\n";
	  return 0;
	}
      }
    }
    print "\n";
    if ($f =~ /(\.exe|\.zip)$/) {
      my @ziplst;
      open ZIP, "unzip -v $f|";
      while (<ZIP>) {
	chomp;
	push @ziplst, $_;
      }
      close ZIP;
      if ($ziplst[$#ziplst] =~ /^\s*([^\s]+)\s+([^\s]+)/x) {
	print "size = $1\n" if $opt_debug;
	$size += $1;
	$failure = 1 if ($1 == 0);
      }
    }
    print STDERR "failure = $failure\n";
    if ($failure) {
      # fall back
      my @st = stat($f);
      print STDERR "size($f) = " . $st[7] . "\n";
      $size += $st[7];
    }
  } @files;
  return $size;
}

sub read_description {
  while (<IN>) {
    chomp;
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^\s*([^= ]+)\s*=\s*(.*)\s*$@) {
      my ($key, $val) = ($1, $2);
      if ($val =~ /<</) {
	$val = "";
	while (<IN>) {
	  last if ($_ =~ /^<<EOF$/);
	  $val .= $_
	}
	# make sure not too many \n at the end.
	$val =~ s/[ \n]*$/\n/;
	$val =~ s@^\n\n*@@;
      }
      $key =~ s/^\s*(.*)\s*$/$1/;
#      print "key =$key, val = $val\n";
      ${$key} = $val;
    }
  }
}

sub read_file_list {
  my @files = ();
  while (<IN>) {
    chomp;
    last if ($_ =~ m@^\[@);
    if ($_ =~ m@^file=(.*)\s*$@) {
      push @files, $1;
    }
  }
  return @files;
}

sub read_bin {
  while (<IN>) {
    chomp;
    last if ($_ =~ m@^\[@);
    if ($_ =~ m@^file=(.*)\s*$@) {
      push @binfiles, $1;
    }
  }
}

sub read_data {
  while (<IN>) {
    chomp;
    last if ($_ =~ m@^\[@);
    if ($_ =~ m@^file=(.*)\s*$@) {
      push @runfiles, $1;
    }
  }
}

sub read_flags {
  %flags = ( );
  while (<IN>) {
    chomp;
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^([^=]*)\s*=\s*(.*)\s*$@) {
      $flags{$1} = $2;
    }
  }
}

sub read_provides {
  $#provlist = -1;
  while (<IN>) {
    chomp;
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^feature=(.*)\s*$@) {
      push @provlist, $1;
    }
  }
}

sub read_requires {
  %reqlist = ();
  my $i = 0;
  while (<IN>) {
    chomp;
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^([^=]*)=(.*)\s*$@) {
#      ${$reqlist{$1}}[$i++] = $2;
      push @{${reqlist}{$1}}, $2;
    }
  }
}

sub read_required_by {
  $#reqbylist = -1;
  @reqbylist = ();
  while (<IN>) {
    chomp;
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^package=(.*)\s*$@) {
      push @reqbylist, $1;
    }
  }
}

sub read_installation {
  my $i = -1;
  while (<IN>) {
    chomp;
    if ($_ =~ m@^\[@) {
      return;
    }
    if ($_ =~ m@^function=(.*)\s*$@) {
      $i++;
      ${$installation[$i]}{"function"} = $1;
    }
    elsif ($_ =~ m@^([^=]*)=(.*)\s*$@) {
      ${$installation[$i]}{$1} = $2;
    }
  }
}

sub get_catalogue_candidate {
  my ($pkgname) = @_;
  my (@candidates, $cname);
  
  $pkgname =~ s/\.[^\.]*$//;
  print STDERR "Looking for $pkgname in the Catalogue.\n" if $opt_verbose;
  $cname = "${catalogue_dir}/entries/${pkgname}.xml"; 
  if (-f $cname) {
    push @candidates, $cname;
  } 
# No reason to match on the beginning, either
# there is an exact entry or nothing.
#  else {
#    @candidates = <${catalogue_dir}/entries/${pkgname}*.xml>;
#  }
  return @candidates;
}

#
# Try to enhance the RDF structure using the Catalogue.
# Get the license information, version and release numbers
#
sub complete_using_catalogue {
  my ($pkgname) = @_;
  my ($catname, $catdoc);
  local($author, $version, $license, $caption, $description);

  $pkgname =~ s@^.*\/([^\/\\]*)\.[^\.]*$@$1@;
  @candidates = &get_catalogue_candidate($pkgname);

  if ($#candidates > 0) {
    print STDERR "Warning, package $pkgname, candidates : " if $opt_verbose;
    map { print $_ . "\n"; } @candidates;
    return;
  } elsif ( $#candidates == 0) {
    $catname = $candidates[0];
    print STDERR "Found only 1 candidate : $catname\n" if $opt_verbose;
  } else {
    print STDERR "Found no candidates for $pkgname !\n" if $opt_verbose;
    return;
  }
  my $parser = new XML::DOM::Parser;
  my $nl, $i;
  $catdoc = $parser->parsefile($catname);
  $nl = $catdoc->getElementsByTagName("author");
  $author = $nl->item(0)->getElementsByTagName("name")->item(0)->getFirstChild->toString;
  for ($i = 1; $i < $nl->getLength; $i++) {
    $author .= " and " . $nl->item($i)->getElementsByTagName("name")->item(0)->getFirstChild->toString;
  }
  $nl = $catdoc->getElementsByTagName("version")->item(0);
  if ($nl && $nl->getElementsByTagName("number")->item(0)) {
    $version = $nl->getElementsByTagName("number")->item(0)->getFirstChild;
    if ($version ne undef) {
      $version = $version->toString;
    }
  }
  $nl = $catdoc->getElementsByTagName("license")->item(0);
  if ($nl) {
    $license = $nl->getAttribute("type");
  }
  $nl = $catdoc->getElementsByTagName("caption")->item(0);
  if ($nl) {
    $caption = $nl->getFirstChild->toString;
  }
  $nl = $catdoc->getElementsByTagName("description")->item(0);
  if ($nl) {
    $description = $nl->getFirstChild->toString;
  }
  return ($author, $version, $license, $caption, $description);
}

sub regexpify {
  my ($texdir, @files) = @_;
  my (@regexp);
  print STDERR "files = $#files, texdir = $texdir\n" if $debug;
  # Try to generate patterns by this simple rule of thumb
  @regexp = map {
    if ($_ !~ m@^\s*$@) {
      my $r = $_;
      $r =~ s@[/\\][^/\\]+$@/$1*@;
      $r =~ s@^\s*@@;
      $r;
    }
  } @files;

  @regexp = &sort_uniq(@regexp);
  @files = &sort_uniq(@files);
  my @pkgfiles = ();
  my @tmp = map { "$texdir/" . $_ } @files;
  map {
    my $dir = $_;
    my @tmp1 = &globexpand(0, "$texdir/" . $_);
    @tmp1 = map {
      $_ =~ s@^$texdir/@@;
      $_;
    } @tmp1;
    $dir =~ s@[/\\]\*$@@;
    my @tmp2 = ();
    map {
      if (/^$dir[\/\\][^\/\\]*$/) {
	push @tmp2, $_;
      }
    } @files;
    local (@l3, @l4);
    &diff_list(\@tmp1, \@tmp2, \@l3, \@l4);
    if ($#l3 == -1 && $#l4 == -1) {
      push @pkgfiles, "$dir/*";
    } else {
      push @pkgfiles, @tmp2
    }
  } @regexp;
  return @pkgfiles;
}

sub extract_remote {
  my ($files) = @_;
  my (@remote, @tmp);
  @tmp = map {
    if ($_ =~ m@^(http|ftp)://@) {
      push @remote, $_;
    } else {
      $_;
    }
  } @{$files};
  @{${files}} = @tmp;
  return @remote;
}

sub tagname2stringlist {
  my ($doc, $tagname) = @_;
  my @stringlist = ( );

  my $nodes = $doc->getElementsByTagName($tagname);
  return @stringlist if ($nodes->getLength <= 0);
  my $node = $nodes->item(0)->getFirstChild;
  my $files = $node->toString;
  $files =~ s/\n/ /gomsx;
  push @stringlist, split(" ", $files);
  return @stringlist;
}

sub tagname2string {
  my ($doc, $tagname) = @_;

  my $nodes = $doc->getElementsByTagName($tagname);
  return "" if ($nodes->getLength == 0);
  if ($nodes->getLength != 1) {
    print STDERR "$0: tagname2string found multiple occurences for tag $tagname.\n";
    print STDERR "$0: ". $nodes->item(0)->getFirstChild->toString . "\n";
  }
  if ($nodes->item(0)->getFirstChild != undef) {
    my $s = $nodes->item(0)->getFirstChild->toString;
    while (chomp($s) > 0) { };
    $s =~ s/^\n\n*//;
    if ($s =~ /\n/) {
      $s = "<<\n" . $s . "\n<<EOF";
    }
    return $s;
  } else {
    print STDERR "$0: tagname2string found an undef value for tag $tagname.\n";
    return "";
  }
}

sub tagname2alist {
  my ($doc, $tagname) = @_;
  my $nodes = $doc->getElementsByTagName($tagname);
  return "" if ($nodes->getLength == 0);
  if ($nodes->getLength != 1) {
    print STDERR "$0: tagname2string found multiple occurences for tag $tagname.\n";
    print STDERR "$0: ". $nodes->item(0)->getFirstChild->toString . "\n";
  }
  my $node = $nodes->item(0);
  my $s;
  foreach my $k (@{$node->getAttributes->getValues}) {
    $k = $k->{Name};
    $s .= "$k=" . $node->getAttribute($k) . "\n";
  }
  chomp($s);
  return $s;
}

sub multipletagname2stringlist {
  my($doc, $tagname, $attr) = @_;
  my($i, @s);
  my $nodes = $doc->getElementsByTagName($tagname);
#  print STDERR "length = " . $nodes->getLength . "\n";
  @s = ( );
  if ($nodes != undef) {
    for ($i = 0; $i < $nodes->getLength; $i++) {
      if ($attr) {
	$s[$i] = $nodes->item($i)->getAttribute("name");
      }
      else {
	$s[$i] = $nodes->item($i)->getFirstChild->toString;
      }
#      print STDERR "s[$i] = " . $s[$i] . "\n";
    }
  }
  return @s;
}

sub OldTpm2Pkg {
  local ($tpmname, $pkgname, $texdir, $destdir) = @_;

  print "Processing $tpmname\n" if $opt_verbose;
  #   open IN, "<$listname";
  #   while (<IN>) {
  #     chop $_;
  #     next if ($_ =~ m@^\s*$@);
  #     $_ =~ s@^\s*@@;
  #     push @files, $_;
  #   }
  #   close IN;
  my $parser = new XML::DOM::Parser;
  my $doc = $parser->parsefile($tpmname);
  # print "texdir = $texdir\n";
  my @docregexp = &regexpify($texdir, &tagname2stringlist($doc, "TPM:DocFiles"));
  my @srcregexp = &regexpify($texdir, &tagname2stringlist($doc, "TPM:SourceFiles"));
  my @runregexp = &regexpify($texdir, &tagname2stringlist($doc, "TPM:RunFiles"));
  my @remotefiles = &regexpify($texdir, &tagname2stringlist($doc, "TPM:RemoteFiles"));

  my @binregexp = &compress_bin($texdir, &multipletagname2stringlist($doc, "TPM:BinFiles", ""));

  open OUT, ">$destdir/$pkgname";
  print OUT "## $pkgname:\n";
  print OUT "## Time-Stamp: \"" . &printdate("") . " Fabrice Popineau\"\n\n";
  print OUT "[Description]\n";
  print OUT "name=" . &tagname2string($doc, "TPM:Name") . "\n";
  print OUT "date=" . &tagname2string($doc, "TPM:Date") . "\n";
  print OUT "version=" . &tagname2string($doc, "TPM:Version") . "\n";
  print OUT "creator=" . &tagname2string($doc, "TPM:Creator") . "\n";
  print OUT "title=" . &tagname2string($doc, "TPM:Title") . "\n";
  print OUT "description=" . &tagname2string($doc, "TPM:Description") . "\n";
#  print OUT "level=" . &tagname2string($doc, "TPM:Level") . "\n";
#  print OUT "collection=" . &tagname2string($doc, "TPM:Collection") . "\n";
  my @candidates = &get_catalogue_candidate($pkgname);
  if ($#candidates == 0) {
    print OUT "catalogue=" . $candidates[0] . "\n";
  } else {
    print OUT "catalogue=\n";
    print STDOUT "!!!Warning, no catalogue entry for $pkgname.\n";
  }
  # Flags
  print OUT "[flags]\n" . &tagname2alist($doc, "TPM:Flags") . "\n";

  # Provides
  my $feature = $pkgname;
  $feature =~ s/\.[^\.]*//;
  print OUT "[provides]\n";
  print OUT "feature=$feature\n";

  # Requires
  print OUT "[requires]\n";
  my $requirements = $doc->getElementsByTagName("TPM:Requires")->item(0);
  if ($requirements != undef) {
    map { 
      print OUT "package=" . $_ . "\n" 
    } multipletagname2stringlist($requirements, "TPM:Package", "name");
    map { 
      print OUT "collection=" . $_ . "\n" 
    } multipletagname2stringlist($requirements, "TPM:Collection", "name");
  }
#  $requirements = $requirements->getElementsByTagName("rdf:bag");
#  map { print OUT "require=$_\n"; } $requirements;

  #  map { } @source_files;

  if ($#binregexp >= 0) {
    print OUT "[bin]\n";
    map { print OUT "file=" . $_ . "\n" } @binregexp;
  }

  if ($#runregexp >= 0) {
    print OUT "[run]\n";
    map { print OUT "file=" . $_ . "\n" } @runregexp;
  }

  if ($#docregexp >= 0) {
    print OUT "[documentation]\n";
    map { print OUT "file=" . $_ . "\n" } @docregexp;
  }

  if ($#srcregexp >= 0) {
    print OUT "[source]\n";
    map { print OUT "file=" . $_ . "\n" } @srcregexp;
  }

  #   print OUT "[binaries]\n";
  #   map { } @source_files;
  
  if ($#remotefiles >= 0) {
    print OUT "[remote]\n";
    map { print OUT "file=" . $_ . "\n" } @remotefiles;
  }

  # Installation instructions
  my $nodes = $doc->getElementsByTagName("TPM:Installation");
  if ($nodes->getLength > 0) {
    print $nodes->getLength . "\n";
    my $instructions = $doc->getElementsByTagName("TPM:Execute");
    print OUT "[installation]\n";
    for ($i = 0; $i < $instructions->getLength; $i++) {
      my $inst = $instructions->item($i);
      print OUT "function=" . $inst->getAttribute("function") . "\n";
      foreach my $attr (@{$inst->getAttributes->getValues}) {
	$attr = $attr->{Name};
	if ($attr ne "function") {
	  print OUT $attr . "=" . $inst->getAttribute($attr) . "\n";
	}
      }
    }
  }

  close OUT;
}

sub Pkg2Zip {
  local ($pkgname, $texdir, $destdir) = @_;

  print "Processing $pkgname\n"; # if $opt_verbose;
  local ($name, $version, $level);
  my ($description, $zipfile, $reqlist);
  my ($a);
  local ($binfiles, $runfiles);

  @runfiles = ();
  @binfiles = ();
  open IN, "<$pkgname";
  while (<IN>) {
    # skip comments
    if (! defined $_) {
      next;
    }
    if ($_ =~ m@^\s*(\#.*)?$@) {
      next;
    }
    if ($_ =~ m@^\[description\]@i) {
      &read_description;
      redo;
    } elsif ($_ =~ m@^\[bin\]@i) {
      &read_bin;
      redo;
    } elsif ($_ =~ m@^\[run\]@i) {
      &read_data;
      redo;
    } elsif ($_ =~ m@^\[required\]@i) {
      &read_required;
      redo;
    } else {
      print STDERR "Unmatched line: $_\n";
    }
  }
  close(IN);
  print "Finished reading description\n";
  my $cwd = &getcwd;
  chdir($texdir);
  print "Changing dir from $cwd to $texdir\n";

  print"process binfiles, $#binfiles files\n";
  my @files = &process_bin("win32", @binfiles);
  print"processed bin files, $#files files\n";

  push @files, &process_data(@runfiles);
  print"processed data files, $#files files\n";

  my $zipname = "$destdir/" . basename($pkgname);
  print "zipname = $zipname\n";

  if ($pkgname =~ m/-static\.pkg/) {
    # static packages are expected to have more complete names
    $zipname =~ s/\.[^\.]*$//;
    $zipname =~ s/-static$//;
    $zipname .= "-$version-win32.zip";
  } else {
    $zipname =~ s/\.[^\.]*$/.zip/;
  }
  print "zipname = $zipname\n";
  #
  # Beware : not safe at all.
  #
  if (-f $zipname) {
    unlink $zipname;
  }
  print "Building $zipname, $#files.\n" ;   # if $opt_verbose;
  open ZIP, "| zip -9\@or " . $zipname . " > nul";
  map { 
    if (! -f $_) {
      print STDERR "!!!Error: non-existent $_\n";
    } else {
      print ZIP "$_\n";
    } 
  } @files;
  close ZIP;
  chdir($cwd);

}

sub formatdate {
  return sprintf("%4d/%02d/%02d %02d:%02d:%02d", 
	       $_[5]+1900, $_[4]+1, $_[3], $_[2], $_[1], $_[0]);
}

sub printdate {
  my ($strDate) = @_;
  my @mytime;
  my $s, $strTime;

  ($strDate, $strTime) = split " ", $strDate;
  # print "strDate = $strDate; strTime = $strTime\n";
  if ($strDate =~ m@(\d\d\d\d|\d\d)/(\d\d)/(\d\d)@) {
    $mytime[5] = eval $1;
    $mytime[4] = eval $2;
    $mytime[3] = eval $3;
    if ($strTime =~ m@(\d\d):(\d\d):(\d\d)@) {
      $mytime[2] = eval $1;
      $mytime[1] = eval $2;
      $mytime[0] = eval $3;
    }
    $mytime[5] -= 1900 if ($mytime[5] > 1900);
    $mytime[4] -= 1;
  }
  else {
    @mytime = localtime;
  }

  return &formatdate(@mytime);
}

sub max_date
{
  local ($olddate, @files) = @_;
  my $f, $tpmdate;
  foreach $f (@files) {
    if ($f =~ m@^texmf/tpm/.*\.tpm@) {
      $tpmdate = ${stat($f)}[9];
    }
    my @st = stat($f);
    if ($st[9] > $olddate) {
      $olddate = $st[9];
    }
  }
  if ($olddate == 0) {
    $olddate = $tpmdate;
  }
  return $olddate;
}

#  sub change_tpm_date
#  {
#    local ($tpm, $d) = @_;
#    my $sd = printdate (localtime($d));

#  #    my $parser = new XML::DOM::Parser;
#  #    my $doc = $parser->parsefile($tpm);
#  #    my $oldDateElement = $doc->getElementsByTagName("TPM:Date")->item(0);
#  #    print "node = ". $oldDateElement->toString . "\n";
#  #    my $newDateElement = $doc->createElement("TPM:Date");
#  #    my $newDateTextElement = $doc->createTextNode($sd);
#  #    $newDateElement->appendChild($newDateTextElement);
#  #    $doc->replaceChild($oldDateElement, $newDateElement);
#    open IN, "<$tpm";
#    open OUT, ">$tpm.new";
#    while (<IN>) {
#      if ($_ =~ m/TPM:Date/) {
#        print OUT "<TPM:Date>$sd</TPM:Date>\n";
#      }
#      else {
#        print OUT $_;
#      }
#    }
#    close IN;
#    close OUT;
#    rename($tpm, "$tpm.old");
#    rename("$tpm.new", $tpm);
#    unlink "$tpm.old";
#  }

sub TpmGetFiles {
  local ($name, $arch) = @_;

  my $parser = new XML::DOM::Parser;
  my $doc = $parser->parsefile($name);
  my @file_list = ( );
  foreach $tag ("TPM:RunFiles", "TPM:DocFiles", "TPM:SourceFiles", "TPM:BinFiles") {
    my $nodes = $doc->getElementsByTagName($tag);
    next if ($nodes->getLength <= 0);
    if ($tag eq "TPM:BinFiles") {
      for (my $i = 0; $i < $nodes->getLength; $i++) {
	my $binarch = $nodes->item($i)->getAttributeNode("arch");
	if ((${arch} eq "all")
	  || ($binarch && ($binarch->toString eq "arch=\"${arch}\""))) {
	  my $node = $nodes->item($i)->getFirstChild;
	  my $files = $node->toString;
	  $files =~ s/\n/ /gomsx;
	  push @file_list, split(" ", $files);
	}
      }
    }
    else {
      my $node = $nodes->item(0)->getFirstChild;
      my $files = $node->toString;
      $files =~ s/\n/ /gomsx;
      push @file_list, split(" ", $files);
    }
  }
  return @file_list;
}

sub TpmGetRequires {
  local ($name) = @_;
  my $parser = new XML::DOM::Parser;
  my $doc = $parser->parsefile($name);
  my @file_list;
  my $nodes = $doc->getElementsByTagName("TPM:Package");
  for (my $i = 0; $i < $nodes->getLength; $i = $i+1) {
    my $package = $nodes->item($i)->getAttributeNode("name")->toString;
    $package =~ s/^name="(.*)"/\1/;
    push @file_list, "packages/$package";
  }
  my $nodes = $doc->getElementsByTagName("TPM:Collection");
  for (my $i = 0; $i < $nodes->getLength; $i = $i+1) {
    my $package = $nodes->item($i)->getAttributeNode("name")->toString;
    $package =~ s/^name="(.*)"/\1/;
    push @file_list, "collections/$package";
  }
  my $nodes = $doc->getElementsByTagName("TPM:Support");
  for (my $i = 0; $i < $nodes->getLength; $i = $i+1) {
    my $package = $nodes->item($i)->getAttributeNode("name")->toString;
    $package =~ s/^name="(.*)"/\1/;
    push @file_list, "win32-support/$package";
  }
#  print "$name => $#file_list\n";
  return @file_list;
}

sub Tpm2Zip {
  local ($name, $texdir, $destdir, $arch) = @_;
  print "Processing $name\n"; # if $opt_verbose;
  my $parser = new XML::DOM::Parser;
  my $doc = $parser->parsefile($name);
  my @file_list = TpmGetFiles($name, $arch);;

  if (! -d "$destdir") {
    print STDERR "Error, $destdir is not a directory!\n";
    return;
  }

  my $tpmname = $name;
  $tpmname =~ s/^.*\/tpm\/(.*)$/$1/;
  $tpmname = "texmf/tpm/$tpmname";
  print "tpmname = $tpmname\n" if $debug;
  &remove_list(\@file_list, $tpmname);

  my $cwd = &getcwd;
  chdir($texdir);
  my $zipname = "$destdir/" . basename($name);
  $zipname =~ s/\.[^\.]*$/.zip/;
  if (-f $zipname) {
    unlink $zipname;
  }
  if ($#file_list >= 0) {
    print "Building $zipname, $#file_list files.\n" ;   # if $opt_verbose;
    open ZIP, "| zip -9\@or " . $zipname . " > nul";
    map { 
      if (! -f $_) {
	print STDERR "!!!Error: non-existent $_\n";
      } else {
	print ZIP "$_\n";
      }
    } @file_list;
    close ZIP;

  }
  else {
    print "Empty file_list, building nothing.\n";
    chdir($cwd);
  }
}

END { }

1;
