#
# Build Zip files for fpTeX from tpm files (Version TL5)
# Author: F. Popineau
# Date  : 10/07/00
# Time-stamp: <02/04/10 00:00:35 popineau>
#
# Usage: 
#        tpm2zip
#

BEGIN {

# $Exporter::Verbose = 1;

require "newgetopt.pl";
use lib "./perl";
use Cwd;
use XML::DOM;
use tpm qw(
	   &Tpm::Tpm2Zip 
	   &Tpm::TpmGetFiles
	   &Tpm::TpmGetRequires
	   @Tpm::arch_list 
	   $Tpm::ftpdir 
	   $Tpm::masterdir 
	   $Tpm::fptexversion
	   @Tpm::TpmSubdirs
	  );
use file_utils qw(
		  canon_dir 
		  cleandir 
		  make_link 
		  newpath
		  walk_dir 
		  basename 
		  is_absolute 
		  canon_dir
		  normalize 
		  substitute_var_val 
		  dirname 
		  member
		  move
		  start_redirection
		  stop_redirection
		  remove_list
		  diff_list
		  walk_dir
		 );
# use diagnostics;
}

$opt_texdir = "${Tpm::masterdir}";
$opt_zip = 0;
$opt_check=0;
$opt_destdir = "${Tpm::ftpdir}/${Tpm::fptexversion}";
$opt_tpmdir = "$opt_texdir/texmf/tpm";
$opt_arch = "win32";
$opt_log = "";

#
# Some supplement packages are found elsewhere than in $opt_texdir
#
#  %zip_src_files = (
#  		  # "winedt5-cnf" => "c:/Program Files/Winedt",
#  		  "xemacs" => "d:/Local/XEmacs",
#  		  "netpbm" => "d:/Local/NetPBM",
#  		  "imagick" => "d:/Local/ImageMagick",
#  		  "perl" => "d:/Local/Perl"
#  		 );

&main;

exit 0;

sub help
{
    print "Usage: $0 [options]\n";
    print "\n";
    print "    --texdir=<path>     Root directory of the TeX distribution.\n";
    print "    --tpmdir=<path>     Where to get the tpm files (\$texdir/texmf/tpm by default).\n";    
    print "    --destdir=<path>    Destination directory for the zip files.\n";
    print "    --all               Process all the tpm packages.\n";
    print "    --subdir=<path>     Process the tpm files in subdirectory <path>.\n";
    print "    --tpm=<name>        Process only the named tpm file.\n";
    print "    --arch=<(all|name)> Process only the named tpm file.\n";
    print "    --check=<(dep|cov)> Check dependencies or coverage for the tpm files.\n";
    print "\n";		       
    print "    Support commands:\n";
    print "    --log=<file>     Redirects output to some log <file>.\n";
    print "    --debug          Build debug versions;\n";
    print "    --help           displays this help text and exits\n";
    print "    --verbose        some more messages\n";
}

sub build_zip {
  local ($subdir, $tpmname) = @_;
  Tpm::Tpm2Zip("$opt_tpmdir/$subdir/$tpmname.tpm", $opt_texdir, "$opt_destdir/$subdir", $opt_arch);
}

sub read_dependencies {
  local ($subdir, $tpmname) = @_;
  @{$req{"$subdir/$tpmname"}} = &Tpm::TpmGetRequires("$opt_tpmdir/$subdir/$tpmname.tpm");
}

sub read_files {
  local ($subdir, $tpmname) = @_;
  return Tpm::TpmGetFiles("$opt_tpmdir/$subdir/$tpmname.tpm", $opt_arch);
}

sub collect_tpmfiles {
  local ($subdir, $tpmname) = @_;
  push @tpm_files, "$subdir/$tpmname";
}

sub do_subdir {
  local ($subdir, $fn) = @_;
#  print "subdir = $subdir\n";
  while (<$subdir/*>) {
    local $tpmname = $_;
    next if ($tpmname =~ m/~$/);
    $tpmname =~ m/.*\/([^\/]*)\/([^\/]*)\.tpm$/;
    $subdir = $1;
    $tpmname = $2;
    print "Doing $tpmname\n" if ($opt_verbose);
    &{$fn}($subdir, $tpmname);
  }
}

#
# Check that all tpm package is required by another one at least
#
sub check_dependencies {
  local %req, @tpm_files;
  local $s, $k, @l;

  for $s ("collections", "packages", "support") {
    $opt_subdir = $s;
    if (! -d $opt_subdir) {
      if (! -d "$opt_tpmdir/$opt_subdir") {
	printf STDERR "$0: $opt_subdir is not a tpm subdirectory, aborting!\n";
	exit 1;
      }
      else {
	$opt_subdir = "$opt_tpmdir/$opt_subdir";
      }
    }
    &do_subdir($opt_subdir, \&read_dependencies);
    &do_subdir($opt_subdir, \&collect_tpmfiles);
  }

  @l = ("collections/texlive");
  while ($#l >= 0) {
    my $p = pop @l;
    remove_list(\@tpm_files, $p);
    push @l, @{$req{$p}};
  }
  print "***********************************\n";
  print "Packages not required by anything :\n";
  print "***********************************\n";
  map { print "$_\n"; } @tpm_files;
  print "\n";
  print "***********************************\n";
}

sub collect_files {
  local ($d, @lf) = @_;
  my ($f);
  foreach $f (@lf) {
    my $path = "$d/$f";
    if (-f "$path") {
      $path =~ s@^${opt_texdir}/@@;
      $path =~ s@\\@/@g;
      if ($path !~ m@^texmf/tpm/lists/@) { push @texmf_files, $path; }
    }
  }
}

sub collect_coverage {
  local ($subdir, $tpmname) = @_;
  local (@l) = &read_files($subdir, $tpmname);
  print "Pushing $#l files from $subdir/$tpmname\n" if $opt_debug;
  push @tpm_files, @l;
}

sub find_multiple {
  local (*files) = @_;
  my (@l) = sort(@files);
  my(@res, $e1, $e2);
  @files = ( );
  $e1 = pop(@l);
  while ($e1) {
    push @files, $e1;
    while (($e2 = pop(@l)) eq $e1) {
      push @res, $e2;
    }
    $e1 = $e2;
  }
  return @res;
}

#
# Check that all files in tpm files exist
# and that all files in the texmf tree
# are requested by one tpm file.
#
sub check_coverage {
  local (@texmf_files, @tpm_files);
  my($s);

  &walk_dir("$opt_texdir/texmf", \&collect_files);
  &walk_dir("$opt_texdir/bin", \&collect_files);
  &walk_dir("$opt_texdir/info", \&collect_files);
  &walk_dir("$opt_texdir/man", \&collect_files);
  &walk_dir("$opt_texdir/Books", \&collect_files);
  &walk_dir("$opt_texdir/usergrps", \&collect_files);

  for $s ("collections", "packages", "support") {
    $opt_subdir = $s;
    if (! -d $opt_subdir) {
      if (! -d "$opt_tpmdir/$opt_subdir") {
	printf STDERR "$0: $opt_subdir is not a tpm subdirectory, aborting!\n";
	exit 1;
      }
      else {
	$opt_subdir = "$opt_tpmdir/$opt_subdir";
      }
    }
    &do_subdir($opt_subdir, \&collect_coverage);
  }
  print "Found $#tpm_files files in tpm files.\n";
  local (@doublons) = find_multiple(\@tpm_files);
  print "*****************************************\n";
  print "Files found multiple times in tpm files :\n";
  map { print "$_\n"; } @doublons;
  print "*****************************************\n";
  local (@l1, @l2);
  &diff_list(\@texmf_files, \@tpm_files, \@l1, \@l2);
  print "*****************************************\n";
  print "Files in texmf tree not in tpm files :\n";
  map { print "$_\n"; } @l1;
  print "*****************************************\n";
  print "Files in texmf tree not in tpm files :\n";
  print "*****************************************\n";
  map { print "$_\n"; } @l2;
  print "*****************************************\n";

}

sub main {

  unless (&NGetOpt ("texdir=s", "destdir=s", "all", "tpm=s",
		    "tpmdir=s", "subdir=s", "arch=s", "log=s",
		    "check=s",
		    "debug", "verbose", "help")) {
    print STDERR "Try `$0 --help'";
    exit 1;
  }
  
  if ($opt_help) {
    &help;
    exit 0;
  }

  if (! -d $opt_texdir) {
    print STDERR "$0: $opt_texdir is not a directory, aborting.\n";
    exit 1;
  }

  if (! -d $opt_destdir) {
    print STDERR "$0: $opt_destdir is not a directory, aborting.\n";
    exit 1;
  }

  if (! is_absolute($opt_destdir)) {
    my $cwd = &getcwd;
    $opt_destdir = canon_dir("$cwd/$opt_destdir");
  }

  if (! -d $opt_tpmdir) {
    $opt_tpmdir = $opt_texdir . "/texmf/tpm";
    if (! -d $opt_tpmdir) {
      print STDERR "$0: $opt_tpmdir is not a directory, aborting.\n";
      exit 1;
    }
  }

  if ($opt_arch 
      && !(($opt_arch eq "all")
	   || &member($opt_arch, @Tpm::arch_list))) {
    print STDERR "$0: $opt_arch is neither `all' nor a valid architecture, aborting.\n";
    exit 1;
  }

  &start_redirection($opt_log);

  # Check coverage of packages: do the tpm files cover
  # the list of files in the texmf tree?
  if ($opt_check && $opt_check =~ m/^cov/) {
    print "Checking coverage:\n";
    $opt_arch = "all";
    &check_coverage;
  }
  # Check dependencies: are all the tpm packages required
  # by some other package?
  elsif ($opt_check && $opt_check =~ m/^dep/) {
    print "Checking dependencies:\n";
    &check_dependencies;
  }
  # Build zip files
  else {
    if ($opt_all) {
      local $s;
      for $s ("collections", "packages", "support") {
	$opt_subdir = $s;
	if (! -d $opt_subdir) {
	  if (! -d "$opt_tpmdir/$opt_subdir") {
	    printf STDERR "$0: $opt_subdir is not a tpm subdirectory, aborting!\n";
	    exit 1;
	  } else {
	    $opt_subdir = "$opt_tpmdir/$opt_subdir";
	  }
	}
	print "subdir = $opt_subdir\n";
	&do_subdir($opt_subdir, \&build_zip);
      }
    } 
    elsif ($opt_subdir) {
      if (! -d $opt_subdir) {
	if (! -d "$opt_tpmdir/$opt_subdir") {
	  printf STDERR "$0: $opt_subdir is not a tpm subdirectory, aborting!\n";
	  exit 1;
	} else {
	  $opt_subdir = "$opt_tpmdir/$opt_subdir";
	}
      }
      &do_subdir($opt_subdir, \&build_zip);
    } 
    elsif ($opt_tpm) {
      if (! -f $opt_tpm) {
	if (! -f "$opt_tpmdir/$opt_tpm") {
	  printf STDERR "$0: $opt_tpm non existent tpm file, aborting!\n";
	  exit 1;
	}
      }
      $opt_tpm =~ m/([^\/]*)\/(.*)$/;
      local $subdir = $1;
      if (! -d "$opt_destdir/$subdir") {
	mkdir "$opt_destdir/$subdir";
      }
      Tpm::Tpm2Zip("$opt_tpmdir/$opt_tpm", $opt_texdir, "$opt_destdir/$subdir", $opt_arch);
    }

    #
    # Rebuild tpm.zip file
    #
    &move("${opt_destdir}/tpm.zip", "${opt_destdir}/tpm.zip.old");
    $cwd = &getcwd();
    chdir($opt_texdir);
    open ZIP, "| zip -9\@or ${opt_destdir}/tpm.zip  > nul";
    foreach $e (@Tpm::TpmSubdirs) {
      while (<texmf/tpm/$e/*>) {
	print ZIP "$_\n";
      }
    }
    close ZIP;
    chdir($cwd);
    unlink("${opt_destdir}/tpm.zip.old");
  }
  &stop_redirection($opt_log);

}

END { }
